###################################################################
package BU::CommonFunc;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::Logger;
use MIME::Base64 qw(encode_base64 decode_base64);

@ISA = qw(Exporter);
@EXPORT = qw(
askInteractive
ver2int
a2u u2a
genSaltForCrypt
getPwData
url_encode
url_decode
shellArgQuote
escapeFileNameSpecChar
);

my $SIGN = '==base64';

sub askInteractive($; $ $)
{
	my ($msg, $ask, $confirm) = @_;

	#####################################
	#	By default assume [Yes] variant
	#####################################
	if (not defined $ask) {
		$ask = 'y';
	}

	if (not defined $confirm) {
		$confirm = $ask;
	}

	print("$msg [$ask]:");
	$ch = getc;

	################################################
	#	User's answer may be equal to confirmation
	################################################
	if (($ch eq $confirm) or ($ch eq (uc $confirm))) {
		return 1;
	}

	#########################################
	#	Or, if the confirmation is equal to 
	#	answer - user can just hit 'Enter'
	#########################################
	if (($ask eq $confirm) and ($ch eq "\n")) {
		return 1;
	}

	return '';
}


sub ver2int($; $)
{
	my ($ver, $use_patch_version) = @_;

	$ver =~ m/^([0-9]+)\.([0-9]+)(\.([0-9]+)){0,1}/;

	my $major_ver = $1;
	my $minor_ver = $2;
	my $patch_ver = ($use_patch_version && $4) ? $4 : 0; #FROM psa 2.5.1 in BU version will be checked only first two digits x.y.

	$ver = sprintf "%03u%03u%03u", $major_ver, $minor_ver, $patch_ver;

	return $ver + 0;
}


sub a2u($) # input param type: string
{
	my ($str) = @_;
	my $enc_str = $str;

	if ($str =~ m/^$SIGN/) {
		$enc_str = $SIGN . encode_base64($str);
	} elsif ($str =~ m/[<>"&]|[\x0-\x19]|[\x23-\x26]|[\x7E-\xFF]/) {
		$enc_str = $SIGN . encode_base64($str);
	}

	chomp $enc_str;
	chomp $enc_str;
	chomp $enc_str;

	return $enc_str;
}


sub u2a($) # input param type: string
{
	my ($str) = @_;

	if ($str =~ m/^$SIGN([.\n\w\W]*)/) {
		return decode_base64($1);
	}

	return $str;
}


sub genSaltForCrypt()
{
	my $symbols = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz';

	my $num1 = int(100 * rand());
	my $num2 = int(100 * rand());

	my $num1_div = int($num1 / length($symbols));
	my $num2_div = int($num2 / length($symbols));

	my $salt = substr($symbols, $num1 - $num1_div*length($symbols), 1)
			.  substr($symbols, $num2 - $num2_div*length($symbols), 1);

	return $salt;
}


sub getPwData($)
{
	my ($login) = @_;
	@res = getpwnam $login;
# <<< Patch Code -- 0001 >>>
	return @res;	
}


sub url_decode
{
	($_) = @_;

	s/\+/ /g;
	s/%([0-9A-H]{2})/pack('C',hex($1))/ge;

	return $_;
}


sub url_encode
{
	($_) = @_;

	s/([^ A-Za-z0-9-])/'%' . sprintf("%02X", unpack('C', $1))/ge;
	s/ /\+/g;

	return $_;
}


sub shellArgQuote($)
{### use for quoting spec symbols in arguments during some external util executing
	($_) = @_;
	s/'/'\\''/g;
	return "'$_'";
}


sub escapeFileNameSpecChar($)
{### use for add slahes for special characters in file pathname
	($_) = @_;
	s/([\\!&'";\$()\s])/\\\1/g;
	return $_;
}



return 1;

