###################################################################
package BU::FileFunc;
###################################################################

require 5.004;
require Exporter;
use Carp;

use Fcntl qw(:DEFAULT :flock);

@ISA = qw(Exporter);
@EXPORT = qw(
getUniqFileName randFileName tempFileName existFile existDir isExecutableFile whereIs
lock_file unlock_file
);

sub getUniqFileName($)
{
	my ($fpath) = @_;

	$fpath =~ s/\/+$//;
	my $result_fpath = '';
	my $tries = 10;
	do {
		$result_fpath = randFileName("$fpath/file"); 
	 } while (-e $result_fpath && ($tries > 0));

	$result_fpath =~ s/^.*\/([^\/]+)$/\1/;

	return $result_fpath;
}

sub randFileName($)
{
	my ($fname) = @_;

	my $random = int(rand() * 1000000000); 

	return $fname . '.' . $random;
}


sub tempFileName($)
{
	my ($fname) = @_;

	return '/tmp/' . randFileName($fname);
}


sub existDir($)
{
	my ($path) = @_;

	return '' unless $path;

	my ($dev, $inode, $mode) = stat($path);
 
 	return '' unless $dev;

	return ($mode & 0x4000) >> 14;
}


sub existFile($)
{
	my ($path_file) = @_;

	return '' unless $path_file;

	my ($dev, $inode, $mode) = stat($path_file);
 
 	return '' unless $dev;

	return ($mode & 0x8000) >> 15;
}


sub isExecutableFile($)
{
	my ($path_file) = @_;

	return '' unless $path_file;

	my ($dev, $inode, $mode) = stat($path_file);
 
 	return '' unless $dev;

	return ($mode & 0x0001) & (($mode & 0x8000) >> 15);
}


sub whereIs
{
	foreach my $name (@_) {
		if (isExecutableFile("$name")) {
			return "$name";
		}

		my @whereis_res = split / /, `whereis -b $name`, 10;
		foreach my $path_name (@whereis_res) {
			chomp $path_name;
			if (($path_name =~ m/\/bin\//) && isExecutableFile($path_name)) {
				return $path_name;
			}
		}
	}

	return '';
}


sub lock_file($; $)
{
	my ($file_name, $nonblock) = @_;

	$nonblock = '' unless defined($nonblock);

	my %lock_struct =  ('fname' => $file_name, 'fdescriptor' => \*fd, 'error' => '', 'error_message' => '');

	unless (open (fd, "+>$file_name")) {
		$lock_struct{error} = 1;
		$lock_struct{error_message} = sprintf("Unable to open file '%s': %s\n", $file_name, $!);
		return \%lock_struct;
	}

	$lock_struct{'fdescriptor'} = \*fd;

	unless (flock(fd, LOCK_EX | ($nonblock ? LOCK_NB : 0))) {
		$lock_struct{error} = 1;
		$lock_struct{error_message} = sprintf("Unable to lock file '%s': %s\n", $file_name, $!);
		close fd;
		return \%lock_struct;
	}

	return \%lock_struct;
}


sub unlock_file($)
{
	my ($lock_struct) = @_;

	if (!defined($lock_struct) || !$lock_struct) {
		printf_stderr("Unable to unlock file '%s': lock struct is undefined.\n");
		return '';
	}

	unless (flock($$lock_struct{fdescriptor}, LOCK_UN)) {
		$$lock_struct{error} = 1;
		$$lock_struct{error_message} = sprintf("Unable to unlock file '%s': %s\n", $lock_struct{fname}, $!);
		close $$lock_struct{fdescriptor};
		unlink($$lock_struct{fname});
		return '';
	}

	close $$lock_struct{fdescriptor};
	unlink($$lock_struct{fname});

	return 1;
}

return 1;

