###################################################################
package BU::Init;
###################################################################

require 5.004;
require Exporter;

use BU::Logger;
use BU::FileFunc;
use POSIX qw(geteuid strftime);

@ISA = qw(Exporter);
@EXPORT = qw(
setOSParams
%const
$debug
FULL CONF BASE64 RPM STANDARD
);

$debug = 0; # debug level

use constant FULL	=> 'FULL';
use constant CONF	=> 'CONF';
use constant BASE64	=> 'base64';

use constant RPM		=> 0;
use constant STANDARD	=> 1;

$debug = 0; # debug level

# global constants
%const = ();

sub setOSParams()
{
	open HOST, "hostname |" || warn_log "Unable to detect hostname: cannot execute hostname.\n";
	$_ = <HOST>;
	close HOST;
	$const{hostname} = $_;

	open TARGET_OS, 'uname -a |' || die_log "Unable to detect target Operating System: cannot execute 'uname -a'.\n";
	$OS = <TARGET_OS>;
	close TARGET_OS;

	if ($OS =~ m/FreeBSD[^#:]+\s([^\s]*)\s*#.:.*i386/) {
		$const{OS_VERSION} = $1;

		$const{tar}			= whereIs('/usr/local/bin/gtar', 'gtar', '/usr/bin/tar', 'tar');
		$const{zip}			= whereIs('/usr/bin/gzip', 'gzip', 'zip');
		$const{chpasswd}	= whereIs('/usr/bin/chpass', 'chpass');

		$const{OS} = "FreeBSD";
		$const{OSid} = 'freebsd';
		$const{named_osrelease} = 040100; # "must be >= 020218 for  named -u <named> "

		$const{MYSQL_CONF} = '/etc/my.cnf';

	} elsif ($OS =~ m/Linux.*2\..\..*i.86.*/) {

		$const{tar}			= whereIs('/bin/tar', 'tar', '/bin/gtar', 'gtar');
		$const{zip}			= whereIs('/bin/gzip', '/usr/bin/gzip', 'gzip', 'zip');
		$const{chpasswd}	= whereIs('/usr/sbin/usermod', 'usermod');

		$const{MYSQL_CONF} = '/etc/my.cnf';

		if (-e '/etc/mandrake-release') {
		    $const{OS} = "Linux Mandrake";
			$const{OSid} = 'linux';
		    $const{linux_distr} = "mandrake";
		    $const{named_osrelease} = `cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", \\\$F[0],\\\$F[1],\\\$F[2]'`;
			$_ = `cat /etc/mandrake-release`;
			unless (m/^[^0-9]*([0-9\.]+)/) {
				print_stderr("Error: unable to detect Mandrake version.\n");
				die;
			}
			$const{OS_VERSION} = $1;
		} elsif (-e '/etc/SuSE-release') {
		    $const{OS} = "Linux SuSE";
			$const{OSid} = 'linux';
		    $const{linux_distr} = "suse";
		    $const{named_osrelease} = `cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", \\\$F[0],\\\$F[1],\\\$F[2]'`;
			$_ = `cat /etc/SuSE-release | grep VERSION`;
			unless (m/^[^0-9]*([0-9\.]+)/) {
				print_stderr("Error: unable to detect SuSE version.\n");
				die;
			}
			$const{OS_VERSION} = $1;
		} elsif (-e '/etc/fedora-release') {
		    $const{OS} = "Linux Fedora";
			$const{OSid} = 'linux';
		    $const{linux_distr} = "fedora";
		    $const{named_osrelease} = `cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", \\\$F[0],\\\$F[1],\\\$F[2]'`;
			$_ = `cat /etc/fedora-release`;
			unless (m/^[^0-9]*([0-9\.]+)/) {
				print_stderr("Error: unable to detect Fedora version.\n");
				die;
			}
			$const{OS_VERSION} = $1;
		} elsif (-e '/etc/redhat-release') {
			$const{OSid} = 'linux';
		    $const{named_osrelease} = `cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", \\\$F[0],\\\$F[1],\\\$F[2]'`;
			$_ = `cat /etc/redhat-release`;
			chomp $_;
			if (m/^\s*CentOS/) {
		    	$const{OS} = "Linux CentOS";
		    	$const{linux_distr} = "centos";
			} else {
		    	$const{OS} = "Linux RedHat";
			    $const{linux_distr} = "redhat";
			}
			unless (m/^[^0-9]*([0-9\.]+)/) {
				print_stderr("Error: unable to detect linux distribution version.\n");
				die;
			}
			$const{OS_VERSION} = $1;
		} elsif (-e '/etc/debian_version') {
		    $const{OS} = "Debian";
			$const{OSid} = 'linux';
		    $const{linux_distr} = "debian";
		    $const{named_osrelease} = `cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", \\\$F[0],\\\$F[1],\\\$F[2]'`;
			$_ = `cat /etc/debian_version`;
			chomp $_;
			$const{OS_VERSION} = $_;

			$const{MYSQL_CONF} = '/etc/mysql/my.cnf';
		} else {
			die_log "Error: you are running a version of Linux OS which is not currently supported.\n";
		}
	} elsif ($OS =~ m/Linux.*2\..\..*86_64.*/) {

		$const{tar}			= whereIs('/bin/tar', 'tar', '/bin/gtar', 'gtar');
		$const{zip}			= whereIs('/bin/gzip', '/usr/bin/gzip', 'gzip', 'zip');
		$const{chpasswd}	= whereIs('/usr/sbin/usermod', 'usermod');

		$const{MYSQL_CONF} = '/etc/my.cnf';

		if (-e '/etc/SuSE-release') {
		    $const{OS} = "Linux SuSE";
			$const{OSid} = 'linux';
		    $const{linux_distr} = "suse";
		    $const{named_osrelease} = `cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", \\\$F[0],\\\$F[1],\\\$F[2]'`;
			$_ = `cat /etc/SuSE-release | grep VERSION`;
			unless (m/^[^0-9]*([0-9\.]+)/) {
				print_stderr("Error: unable to detect SuSE version.\n");
				die;
			}
			$const{OS_VERSION} = $1;
		}
	} else {
		die_log "Unable to detect target Operating System: cannot identify Operating System.\n";
	}
}


setOSParams();

return 1;
