###################################################################
package BU::MIME::Container;
###################################################################

require 5.000;
require Exporter;
use Carp;

@ISA = qw(Exporter);
@EXPORT = qw(initContainer makePartOfContainer saveContainer);

#use strict;
use MIME::Lite;
use BU::Init;
use BU::Logger;

my $con;
my $xmlpart;

@param_all =('dump-type', 'level', 'host', 'sqlVer', 'dumped-psa-version', 'dumped-psa-build'
			, 'creating-timestamp', 'bu-build');
sub initContainer {
	my %param = @_; 
	my $xml = $param{xml};

	$MIME::Lite::PARANOID=1;
	MIME::Lite->field_order('Date',@param_all,'MIME-Version','content-type');
	### Create message:
	$con = MIME::Lite->new(
                              Type	=> 'multipart/related',
							  );


	$con->scrub(['X-Mailer','Content-Disposition']);
	foreach my $i (@param_all) {
		if( exists $param{$i} ) {
			$con->attr( $i => $param{$i});
		} else {
			warn_log "Parameter '$i' don't set!\n";
		} 
	}
	$con->attr( Vendor => 'SWsoft Inc.');
	$con->attr( Package => $const{BU_NAME});

	$xmlpart = MIME::Lite->new( 
		Data => $xml,
		Encoding => 'binary',
		Type => 'text/xml');
	$xmlpart->scrub(['X-Mailer','MIME-Version','Date','Content-Disposition']);

	$xmlpart->attr('content-type.charset' => 'UTF8');
#	$xmlpart->attr('Content-Encoding' => 'gzip');
	$con->attach($xmlpart);
	return 0;
}


sub makePartOfContainer {
	my %param = @_; 
	my $enc = 'base64';

	croak "Run 'initContainer' before 'makePartOfContainer'\n" unless($con);

	warn_log "Parameter 'cid'  isn't defined!\n" unless $param{cid};
	warn_log "Parameter 'path' and 'data' isn't defined!\n" unless ($param{path} || $param{data});
	warn_log "Parameter 'type' isn't defined!\n" unless $param{type};

	printf_stderr("DEBUG makePartOfContainer: path=(%s), name=(%s), type=(%s), CID=(%s)\n",
					$param{path}, $param{name}, $param{type}, $param{cid}) if $debug > 3;

	$enc = $param{encoding} if( exists $param{encoding} );
   	my $part = MIME::Lite->new( 
		Id => $param{cid},
		Path => $param{path},
#		Filename => $i,
		Encoding => $enc,
		Name => $param{name},
		Type => $param{type},
		Data => $param{data});
	$part->scrub(['X-Mailer','MIME-Version','Date','Content-Disposition']);
	$part->attr('Content-Encoding' => 'gzip') if $param{gzip};
	$part->attr('Content-Encoding' => 'bzip2') if $param{bzip2};
	$con->attach($part);

	return $part;
}


sub saveContainer {
	my $out = shift;
	my %param = @_; 
	my $xml = $param{xml};

	croak "Run 'initContainer' befor 'saveContainer'\n" unless($con);

	if($xml) {
		$xmlpart->build(
			Data => $xml,
			Encoding => 'binary',
			Type => 'text/xml',
			);
		$xmlpart->scrub(['X-Mailer','MIME-Version','Date','Content-Disposition']);
	}

    return $con->print($out);
}	

1;
__END__

=head1 NAME

PSA::Container - The PSA module for working with MIME container.


=head1 SYNOPSIS

 use PSA::Container;

 initContainer(Host => 'hostname', xml => $doc);

 makePartOfContainer(
					cid => ...,
					path => ...,
					type => ...,
					encoding => ...
					);

 saveContainer(\*OUT, xml => $doc);

=head1 DESCRIPTION

Long drawn out discussion of the program. It's a good idea to break this up into subsections using the =head2 directives, like 

=head2 A Sample Subection

=head2 Yet Another Sample Subection

=head1 OPTIONS

Some people make this separate from the description. 

=head1 RETURN VALUE

What the program or function returns if successful. 

=head1 ERRORS

Exceptions, return codes, exit stati, and errno settings. 

=head1 EXAMPLES

Give some example uses of the program. 

=head1 ENVIRONMENT

Envariables this program might care about. 

=head1 FILES

All files used by the program. You should probably use the F<> for these. 

=head1 SEE ALSO

Other man pages to check out, like man(1), man(7), makewhatis(8), or catman(8). 

=head1 NOTES

Miscellaneous commentary. 

=head1 CAVEATS

Things to take special care with; sometimes called WARNINGS. 

=cut DIAGNOSTICS

All possible messages the program can print out--and what they mean. 

=head1 BUGS

