###################################################################
package BU::PSA::ActionLog;
###################################################################
require 5.000;
require Exporter;

use BU::PSA::Const;
use BU::PSA::DB;

@ISA = qw(Exporter);

@EXPORT = qw(getActionsByObjectType getActionId logAction);

$ACTIONS_LOADED = undef;
%ACTIONS = ();

%ACTIONS_BY_OBJECT = (
				'admin' => [ ('admin_update') ],

				'service' => [ (
						'service_stop',
						'service_start',
						'service_restart'
				) ],

				'ip_address' => [ (
						'ip_address_create',
						'ip_address_update',
						'ip_address_delete'
				) ],

				'client' => [ (
					'client_create',
					'client_update',
					'client_delete',
					'client_status_update'
				) ],

				'client_limits' => [ (
					'client_limits_update',
					'client_limit_traffic_reached',
					'client_limit_size_reached'
				) ],

				'client_permissions' => [ ('client_permissions_update') ],

				'client_preferences' => [ ('client_preferences_update') ],

				'client_ip_pool' => [ ('client_ip_pool_update') ],

				'dl_user' => [ ('dl_user_update') ],

				'domain' => [ (
					'domain_create',
					'domain_update',
					'domain_delete',
					'domain_status_update',
					'domain_dns_update'
				) ],

				'phys_hosting' => [ (
					'phys_hosting_create',
					'phys_hosting_update',
					'phys_hosting_delete'
				) ],

				'forwarding' => [ (
					'forwarding_create',
					'forwarding_update',
					'forwarding_delete'
				) ],

				'domain_limits' => [ (
					'domain_limits_update',
					'domain_limit_traffic_reached',
					'domain_limit_size_reached'
				) ],

				'subdomain' => [ (
					'subdomain_create',
					'subdomain_update',
					'subdomain_delete'
				) ],

				'webuser' => [ (
					'webuser_create',
					'webuser_update',
					'webuser_delete'
				) ],

#				'cp_user' => [ (
#					'cp_user_login',
#					'cp_user_logout'
#				) ],

				'mailname' => [ (
					'mailname_create',
					'mailname_update',
					'mailname_delete'
				) ],

				'maillist' => [ (
					'maillist_create',
					'maillist_update',
					'maillist_delete'
				) ],

				'phys_hosting' => [ (
					'phys_hosting_create',
					'phys_hosting_update',
					'phys_hosting_delete'
				) ],

				'forwarding' => [ (
					'forwarding_create',
					'forwarding_update',
					'forwarding_delete'
				) ],

				'webuser' => [ (
					'webuser_create',
					'webuser_update',
					'webuser_delete'
				) ],

				'siteapp' => [ (
					'siteapp_install',
					'siteapp_reconfigure',
					'siteapp_uninstall'
				) ],

				'siteapppkg' => [ (
					'siteapppkg_install',
					'siteapppkg_uninstall'
				) ],

#				'license' => [ ('license_update') ]
);

%COMPONENTS = (
			COMP_SERVICE		=>	'Service',
			COMP_STATUS			=>	'Status',
			COMP_CONTACT_NAME 	=> 	'Contact Name',
			COMP_LOGIN_NAME 	=> 	'Login Name',
			COMP_DOMAIN_NAME 	=> 	'Domain Name',
			COMP_SUBDOMAIN_NAME	=> 	'Subdomain Name',
			COMP_IP_ADDRESS 	=> 	'IP Address',
			COMP_IP_MASK 		=> 	'IP Mask',
			COMP_IFACE 			=> 	'Interface',
			COMP_IP_TYPE 		=> 	'IP Type',
			COMP_FORWARDING_TYPE=>	'Forwarding Type',
			COMP_URL			=>	'URL',
			COMP_WWW_PREFIX		=>	'WWW Prefix',
			COMP_STAT_TTL	 	=> 	'Retain traffic statistics',
			COMP_LICENSE		=>	'License',
			COMP_LICENSE_TYPE	=>	'License type',
			COMP_LICENSE_NAME	=>	'License name',

			COMP_PAGE_SIZE		=>	'Lines per page',
			COMP_LOCALE			=>	'Interface language',
			COMP_SKIN			=>	'Interface skin',
			COMP_MULTIPLE_SESSIONS =>	'Allow multiple sessions',

			COMP_SYSUSER_TYPE 		=> 	'System User Type',
			COMP_SYSUSER_NAME 		=> 	'System User',
			COMP_SYSUSER_PASSWD 	=> 	'System User Password',
			COMP_SYSTEM_SHELL 		=> 	'System Shell',
			COMP_HARD_DISK_QUOTA 	=> 	'Hard Disk Quota',

			COMP_FP_SUPPORT 		=> 	'FP Support',
			COMP_FP_SSL_SUPPORT 	=> 	'FP-SSL Support',
			COMP_FP_AUTHORIZATION 	=> 	'FP Authorization',
			COMP_FP_ADMIN_LOGIN 	=> 	'FP Admin Login',
			COMP_FP_ADMIN_PASSWD 	=> 	'FP Admin Password',

			COMP_SSI_SUPPORT		=> 	'SSI Support',
			COMP_PHP_SUPPORT		=> 	'PHP Support',
			COMP_CGI_SUPPORT		=> 	'CGI Support',
			COMP_PERL_SUPPORT		=> 	'Mod Perl Support',
			COMP_PYTHON_SUPPORT		=> 	'Mod Python Support',
			COMP_COLDFUSION_SUPPORT => 	'ColdFusion Support',
			COMP_ASP_SUPPORT		=> 	'Apache ASP Support',
			COMP_ASP_DOT_NET_SUPPORT=>	'ASP.NET Support',
			COMP_SSL_SUPPORT		=> 	'SSL Support',

			COMP_WEB_STATISTICS 	=> 	'Web Statistics',
			COMP_ERROR_DOCUMENTS	=> 	'Apache Error Documents',

			COMP_MAILNAME 		=> 	'Mailname',
			COMP_MAILBOX 		=> 	'Mailbox',
			COMP_MAIL_PASSWD 	=> 	'Password',
			COMP_MAILBOX_QUOTA 	=> 	'Mailbox Quota',
			COMP_MAIL_REDIRECT 	=> 	'Redirect',
			COMP_REDIRECT_ADDR 	=> 	'Redirect Address',
			COMP_MAIL_GROUP 	=> 	'Mail Group',
			COMP_MAIL_AUTORESP 	=> 	'Autoresponders',
			COMP_MAIL_CP_ACCESS =>	'Mail Controlpanel Access',
			COMP_SPAMFILTER 	=>	'Spam filter',
			COMP_ANTIVIRUS		=>	'Antivirus',

			COMP_WEBUSER_NAME 	=> 	'Webuser Name',
			COMP_WEBUSER_PASSWD => 	'Webuser Password',

			#limits
			COMP_LIMIT_MAX_DOM			=> 	'Maximum Domains',
			COMP_LIMIT_MAX_SUBDOM		=> 	'Maximum Subdomains',
			COMP_LIMIT_DISK_SPACE		=> 	'Maximum Disk Space',
			COMP_LIMIT_MAX_TRAFFIC		=> 	'Maximum Traffic',
			COMP_LIMIT_MAX_WU			=> 	'Maximum Webusers',
			COMP_LIMIT_MAX_DB			=> 	'Maximum Databases',
			COMP_LIMIT_MAX_BOX			=> 	'Maximum Mailboxes',
			COMP_LIMIT_MBOX_QUOTA		=> 	'Maximum Mailbox Quota',
			COMP_LIMIT_MAX_REDIR		=> 	'Maximum Mail Redirects',
			COMP_LIMIT_MAX_MG			=> 	'Maximum Mail Groups',
			COMP_LIMIT_MAX_RESP			=> 	'Maximum Mail Autoresponders',
			COMP_LIMIT_MAX_MAILLISTS	=> 	'Maximum Mail Lists',
			COMP_LIMIT_MAX_WEBAPPS		=>	'Maximum Tomcat Web Applications',
			COMP_LIMIT_EXPIRATION		=>	'Expiration Date',

			#permissions
			COMP_PERM_CREATE_DOMAINS	=> 'Domain creation',
			COMP_PERM_MANAGE_PHOSTING	=> 'Physical hosting management',
			COMP_PERM_MANAGE_QUOTA		=> 'Hard disk quota assignment',
			COMP_PERM_MANAGE_SUBDOMAINS	=> 'Subdomains management',
			COMP_PERM_CHANGE_LIMITS		=> 'Limits adjustment',
			COMP_PERM_MANAGE_DNS		=> 'DNS zone management',
			COMP_PERM_MANAGE_LOG		=> 'Log rotation management',
			COMP_PERM_MANAGE_CRONTAB	=> 'Crontab management',
			COMP_PERM_MANAGE_ANONFTP	=> 'Anonymous FTP management',
			COMP_PERM_MANAGE_WEBAPPS	=> 'Web applications management',
			COMP_PERM_MANAGE_SH_ACCESS	=> 'System access management',
			COMP_PERM_MANAGE_MAILLISTS	=> 'Mailing lists management',
			COMP_PERM_MANAGE_DRWEB		=> 'DrWeb managment',
			COMP_PERM_MAKE_DUMPS		=> 'Backup/restore functions',

			COMP_MAILLIST_NAME		=>	'Mail list name',
			COMP_MAILLIST_STATUS	=>	'Mail list enabled',

			COMP_COMPANY_NAME		=> 'Company Name',
			COMP_PHONE				=> 'Phone',
			COMP_FAX				=> 'Fax',
			COMP_EMAIL				=> 'E-mail',
			COMP_ADDRESS			=> 'Address',
			COMP_CITY				=> 'City',
			COMP_STATE_PROVINCE		=> 'State/Province',
			COMP_POSTAL_ZIP_CODE	=> 'Postal/ZIP Code',
			COMP_COUNTRY			=> 'Country',
			COMP_DOM_LEVEL_ACCESS	=> 'Allow Domain User Access',

			COMP_SITEAPP_NAME			=> 'Package name',
			COMP_SITEAPP_DOMAIN_TYPE	=> 'Domain type',
			COMP_SITEAPP_HTDOCS_DIR		=> 'Directory',
			COMP_SITEAPP_INSTALL_PREFIX => 'Installation prefix',
);

sub loadActions()
{
	my $query = dbDoQuery('SELECT * FROM actions');

	return unless ($query);

	while (my $hashref = $query->fetchrow_hashref()) {
		$ACTIONS{$$hashref{'name'}} = { (
								'id' => $$hashref{'id'},
								'desc' => $$hashref{'desc'},
								'enabled' => $$hashref{'enabled'}
							) } ;
	}
}

sub getActionId($)
{
	my ($action_name) = @_;

	unless ($ACTIONS_LOADED) {
		loadActions();
		$ACTIONS_LOADED = 1;
	}

	return 0 unless exists $ACTIONS{$action_name};

	return ${$ACTIONS{$action_name}}{'id'};
}

sub getActionsByObjectType($$)
{
	my ($object_id, $object_type) = @_;

	my $actions_list = $ACTIONS_BY_OBJECT{$object_type};

#	print "($actions_list)\n";
#
#	foreach my $i (@$actions_list) {
#		print "($i)\n";
#	}

	return $actions_list;
}


sub logAction($$$)
{
	my ($object_id, $action, $components) = @_;

#print "DEBUG (action): $action\n";

	return unless $object_id;

#print "DEBUG (object_id): $object_id\n";

	my $action_id = getActionId($action);
	return unless $action_id;

#print "DEBUG (action_id): $action_id\n";

	dbDoQuery('INSERT INTO log_actions(date, ip_address, user, action_id, object_id) VALUES(NOW(), "127.0.0.1", "admin", "' . $action_id . '", "' . $object_id . '")');
	my $logged_action_id = dbGetInsertId();
	return unless $logged_action_id;
	
#print "DEBUG (logged_action_id): $logged_action_id\n";

#print "DEBUG (diff): $diff\n";

	return unless $components;

	while (my ($component, $values) = each(%$components)) {
		next unless exists($COMPONENTS{$component});
		$component = $COMPONENTS{$component};
#print "DEBUG (component): ($component)\n";
		my ($value_old, $value_new) = @$values;
		$value_old = '' unless $value_old;
		$value_new = '' unless $value_new;
#print "DEBUG (component): ($component, $value_old => $value_new)\n";
		dbDoQuery('INSERT INTO log_components(action_id, component, old_value, new_value) VALUES('
					. "'$logged_action_id',"
					. dbQuote($component) . ','
					. dbQuote($value_old) . ','
					. dbQuote($value_new)
					. ')'
				);
	}
}



BEGIN {
}

