###################################################################
package BU::PSA::Backup::Client;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::XML::CommonFunc;
use BU::PSA::MIME::Add2Stream;

use BU::PSA::Backup::CommonFunc;
use BU::PSA::Backup::Common;
use BU::PSA::Backup::Domain;
use BU::PSA::Backup::ActionLog;

@ISA = qw(Exporter);

@EXPORT = qw(dumpClients dumpClientRef);

sub dumpClients
{
	my ($doc, $base_node, $dumpingBy) = @_;


	my $where_expression = undef;
	if (exists($$dumpingBy{domains_ids})) {
		my $dom_ids_list = '"' . (join '","', keys %{$$dumpingBy{domains_ids}}) . '"';
		$where_expression = "d.id IS NULL OR d.id in ($dom_ids_list)";
	} elsif (exists($$dumpingBy{domains_names})) {
		my $dom_names_list = '"' . (join '","', keys %{$$dumpingBy{domains_names}}) . '"';
		$where_expression = "d.name IS NULL OR d.name in ($dom_names_list)";
	} elsif (exists($$dumpingBy{domains_ids_skip})) {
		my $dom_ids_list = '"' . (join '","', keys %{$$dumpingBy{domains_ids_skip}}) . '"';
		$where_expression = "d.id IS NULL OR NOT(d.id in ($dom_ids_list))";
	} elsif (exists($$dumpingBy{domains_names_skip})) {
		my $dom_names_list = '"' . (join '","', keys %{$$dumpingBy{domains_names_skip}}) . '"';
		$where_expression = "d.name IS NULL OR NOT(d.name in ($dom_names_list))";
	}

	my $clients_ids = undef;
	my $clients_ids_skip = undef;
	my $clients_logins = undef;
	my $clients_logins_skip = undef;

	if (exists($$dumpingBy{clients_ids})) {
		$clients_ids = $$dumpingBy{clients_ids};
	} elsif (exists($$dumpingBy{clients_logins})) {
		$clients_logins = $$dumpingBy{clients_logins};
	} elsif (exists($$dumpingBy{clients_ids_skip})) {
		$clients_ids_skip = $$dumpingBy{clients_ids_skip};
	} elsif (exists($$dumpingBy{clients_logins_skip})) {
		$clients_logins_skip = $$dumpingBy{clients_logins_skip};
	}

	my $query_str = $where_expression ? 
	"SELECT DISTINCT c.*, a.type, a.password FROM clients c LEFT JOIN accounts a ON a.id=c.account_id LEFT JOIN domains d ON d.cl_id=c.id WHERE $where_expression"
	:
	"SELECT DISTINCT c.*, a.type, a.password FROM clients c LEFT JOIN accounts a ON a.id=c.account_id";

	my $query = dbDoQuery($query_str);
	while (my $href = $query->fetchrow_hashref()) {
		if ($clients_ids) {
			next if not exists($$clients_ids{$$href{id}});
		} elsif ($clients_logins) {
			next if not exists($$clients_logins{$$href{login}});
		} elsif ($clients_ids_skip) {
			next if exists($$clients_ids_skip{$$href{id}});
		} elsif ($clients_logins_skip) {
			next if exists($$clients_logins_skip{$$href{login}});
		}

		addSubNode($base_node, dumpClientRef($doc, $href, $dumpingBy));
	}
}


sub dumpClientRef
{
	my ($doc, $href, $dumpingByForDomains) = @_;

	my $base_node = undef;

	if ($$href{login} eq '') {
		die_log "Unable to store client with id=%s (pname: '%s'): login is empty.\n", "$id", $$href{pname};
	}

	$base_node = createUserRef($doc, 'client', '1', $$href{'login'}, $$href{'password'},
						{	'PW_TYPE' => $$href{'type'}, 
							'display_login' => $$href{'display_login'},
							'login_text' => $$href{'login_text'}
						});

	$base_node->setAttribute('id', $$href{id});

	addSubNode($base_node, createCardRef($doc, $href));

	$base_node->setAttribute('status',$$href{status} );
	$base_node->setAttribute('create_date',$$href{cr_date} );

	### dump client's prefs
	dumpClientPrefs($doc, $base_node, $$href{id});

	### dump client's domains
	dumpDomains($doc, $base_node, $$href{id}, $dumpingByForDomains);

	return $base_node;
}


sub dumpClientPrefs
{
	my ($doc, $base_node, $cl_id) = @_;
	my $query;

	return unless $cl_id;

	### dump action log
	addSubNode($base_node, dumpActionLogRef($doc, $cl_id, 'client'));

	### dump custom reports
	addSubNode($base_node, dumpCustomReportsRef($doc, $cl_id, $const{'IS_CLIENT'}));

	### dump custom buttons
	addSubNode($base_node, dumpCustomButtonsRef($doc, $cl_id, $const{'IS_CLIENT'}));

	### dump client's hosting skeleton
	my $path = $conf{HTTPD_VHOSTS_D} . '/.skel/'. $cl_id;
	if (-d $path) {
		my $node = $doc->createElement('vhost_skeleton');
		addDir2Stream($node, 0, '.', $path);
		$base_node->appendChild($node);
	}

	{### dump client's params
		my $query= dbDoQuery('SELECT param, val FROM cl_param WHERE cl_id="' . $cl_id . '"');
		my $node = undef;
		while (my ($param, $value) = $query->fetchrow_array) {
			$node = $doc->createElement('params') unless $node;
			$node->setAttribute($param, a2u($value));
		}
		addSubNode($base_node, $node);
	}

	{ ### dump Clients permissions
		my $query= dbDoQuery('SELECT p.permission, p.value FROM Permissions p, clients c WHERE c.perm_id=p.id AND c.id="' . $cl_id . '"');
		my $node = undef;
		while (my ($perm_name, $perm_value) = $query->fetchrow_array) {
			$node = $doc->createElement('permissions') unless $node;
			$node->setAttribute($perm_name, $perm_value);
		}
		addSubNode($base_node, $node);

		if ($node) {
			### dump permissions action log
			addSubNode($node, dumpActionLogRef($doc, $cl_id, 'client_permissions'));
		}
	}

	{### dump client's limits
		my $query= dbDoQuery('SELECT l.limit_name, l.value FROM Limits l, clients c WHERE c.limits_id=l.id AND c.id="' . $cl_id . '"');
		my $node = undef;
		while (my ($limit_name, $limit_value) = $query->fetchrow_array) {
			$node = $doc->createElement('limits') unless $node;
			$node->setAttribute($limit_name, $limit_value);
		}
		addSubNode($base_node, $node);

		if ($node) {
			### dump limit's action log for client
			addSubNode($node, dumpActionLogRef($doc, $cl_id, 'client_limits'));
		}
	}

	{### dump traffic statistic for client
		my $query = dbDoQuery('SELECT * FROM ClientsTraffic WHERE cl_id="' . $cl_id . '"');
		my $node = undef;
		while (my $hash = $query->fetchrow_hashref) {
			$node = $doc->createElement('traffic_statistic') unless $node;
			delete($$hash{cl_id});
			my $sn = createNodeFromHash($doc, 'record', $hash);
			$node->appendChild($sn);
		}
		addSubNode($base_node, $node);
	}

	{### dump client's logos
		my $query= dbDoQuery('SELECT l.name, l.fake, l.url, l.showMode FROM Logos l, clients c WHERE c.logo_id=l.id AND c.id="' . $cl_id . '"');
		my $node = undef;
		while (my ($file_name, $fake_file_name, $url, $showMode) = $query->fetchrow_array) {
			$node = $doc->createElement('logo') unless $node;
			$node->setAttribute('file', a2u($file_name));
			$node->setAttribute('url', $url) if $url;
			$node->setAttribute('show_mode', a2u($showMode)) if $showMode;
			addLogo2Stream($node, $fake_file_name) if $fake_file_name;
		}
		addSubNode($base_node, $node);
	}

	{### dump client's ip pool
		my $query = dbDoQuery('SELECT c.pool_id, r.component_id FROM clients c, Repository r WHERE c.pool_id=r.rep_id AND c.id="' . $cl_id . '"');
		my $node = undef;
		while (my ($ip_pool_id, $ip_address_id) = $query->fetchrow_array) {
			$node = $doc->createElement('ip_pool') unless $node;
			my $sn = $doc->createElement('ip_address');
			$sn->setAttribute('id', $ip_address_id);
			$node->appendChild($sn);
		}
		addSubNode($base_node, $node);

		if ($node) {
			### dump ip pool's action log for client
			addSubNode($node, dumpActionLogRef($doc, $cl_id, 'client_ip_pool'));
		}
	}

	{### dump client's site app pkgs pool
		my $query = dbDoQuery('SELECT r.component_id FROM clients c, Repository r WHERE c.sapp_pool_id=r.rep_id AND c.id="' . $cl_id . '"');
		my $node = undef;
		while (my ($sapp_pkg_id) = $query->fetchrow_array) {
			$node = $doc->createElement('site_app_packages_pool') unless $node;
			my $sn = $doc->createElement('site_app_package');
			$sn->setAttribute('id', $sapp_pkg_id);
			$node->appendChild($sn);
		}
		addSubNode($base_node, $node);
	}

	### dump domain templates
	addSubNode($base_node, dumpDomainTemplatesRef($doc, $cl_id));

}#sub dumpClientPrefs



return 1;

