###################################################################
package BU::PSA::Backup::Common;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::XML::CommonFunc;
use BU::PSA::MIME::Add2Stream;

use BU::PSA::Backup::CommonFunc;

@ISA = qw(Exporter);

@EXPORT = qw(
dumpCustomReportsRef
dumpCustomButtonsRef
dumpDomainTemplatesRef
dumpClientTemplatesRef
dumpCrontabRef
);


sub dumpCustomReportsRef
{
	my ($doc, $user_id, $user_type) = @_;

	my $base_node = undef;

	my $query = dbDoQuery("SELECT * FROM report WHERE user_id='$user_id' AND user_type='$user_type'");
	while (my $href = $query->fetchrow_hashref()) {
		$base_node = $doc->createElement('custom_reports') unless $base_node;

		my $report_id = $$href{'id'};
		delete $$href{'id'};
		delete $$href{'user_id'};
		delete $$href{'user_type'};

		my $node = createNodeFromHash($doc, 'report', $href);
		$base_node->appendChild($node);

		{
			my $query = dbDoQuery('SELECT * FROM report_section WHERE report_id=' . dbQuote($report_id));
			next unless $query;
			while (my $href = $query->fetchrow_hashref()) {		
				delete $$href{'id'};
				delete $$href{'report_id'};
				my $sn = createNodeFromHash($doc, 'section', $href);
				$node->appendChild($sn);
			}
		}
		{
			my $query = dbDoQuery('SELECT * FROM report_auto WHERE report_id=' . dbQuote($report_id));
			next unless $query;
			while (my $href = $query->fetchrow_hashref()) {		
				delete $$href{'id'};
				delete $$href{'report_id'};
				my $sn = createNodeFromHash($doc, 'auto', $href);
				$node->appendChild($sn);
			}
		}
	}

	return $base_node;
}


sub dumpCustomButtonsRef
{
	my ($doc, $level_id, $level) = @_;

	my $base_node = $doc->createElement('custom_buttons');

	my $query = dbDoQuery('SELECT * FROM custom_buttons WHERE level_id="' . $level_id. '" AND level="' . $level . '"');
	while (my $href = $query->fetchrow_hashref) {
		delete $$href{'id'};
		delete $$href{'level_id'};
		delete $$href{'level'};
		my $sn = createNodeFromHash($doc, 'button', $href);
		my $path = $conf{PRODUCT_ROOT_D} . '/admin/htdocs/images/custom_buttons';
		my $path = $conf{'PRODUCT_ROOT_D'} . '/admin/htdocs/images/custom_buttons/' . $file;
		my $file = $$href{'file'};
		addFile2Stream($sn, 'custom_button', "$path/$file", 'image/x-button') if ($file && -e "$path/$file");
		$base_node->appendChild($sn);
	}

	return $base_node;
}

sub dumpDomainTemplatesRef($;$)
{
	my ($doc, $cl_id) = @_;

	my $base_node = $doc->createElement('domain_templates');

	my $tmpl_rep_id = undef;
	if ($cl_id) {
		my $query = dbDoQuery('SELECT tmpl_id FROM clients WHERE id="' . $cl_id . '" AND tmpl_id>0');
		unless (($tmpl_rep_id) = $query->fetchrow_array()) {
			return undef;
		}
	} else {
		$tmpl_rep_id = getMiscParam('dom_tmpl_list_id');
	}

	my $query = dbDoQuery('SELECT tpl.id, tpl.name, tpl.note_id FROM Templates tpl, Repository r WHERE tpl.id=r.component_id AND r.rep_id="' . $tmpl_rep_id . '"');
	while (my($id, $name, $note_id) = $query->fetchrow_array()) {
		my $tpl_node = $doc->createElement('domain');
		$base_node->appendChild($tpl_node);

		$tpl_node->setAttribute('name', a2u($name));
		$tpl_node->setAttribute('note_id', $note_id);

		my $query2 = dbDoQuery('SELECT element, value FROM TmplData WHERE tmpl_id=' . $id);
		while (my ($field, $value) = $query2->fetchrow_array) {
			my $sn = undef;
			if ($field eq 'logrotation_id') {
				my $query3 = dbDoQuery('SELECT * FROM log_rotation WHERE id=' . dbQuote($value));
				if (my $href = $query3->fetchrow_hashref()) {
					$sn = $doc->createElement('log_rotation');
					while (my ($field, $value) = each (%$href)) {
						$sn->setAttribute($field, a2u($value));
					}	
				}
			} else {
				$sn = $doc->createElement(a2u($field));
				$sn->setAttribute('value', a2u($value));
			}
			$tpl_node->appendChild($sn);
		}
	}

	return $base_node;
}


sub dumpClientTemplatesRef 
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('client_templates');
	
	my $query = dbDoQuery("SELECT t.id, t.name, t.note_id FROM Templates t, Repository r, misc m WHERE t.id=r.component_id AND r.rep_id=m.val AND m.param='cl_tmpl_list_id'");
	while (my ($t_id, $t_name, $t_note_id) = $query->fetchrow_array) {
		my $t_node = $doc->createElement('client');
		$t_node->setAttribute('name', a2u($t_name));
		$t_node->setAttribute('note_id', $t_note_id);
		my $query2 = dbDoQuery("SELECT t.element, t.value from TmplData t WHERE t.tmpl_id='$t_id'");
		while (my ($d_element, $d_value) = $query2->fetchrow_array) {
			if ($d_element ne 'tmpl_pool_id') {
				my $param = $doc->createElement(a2u($d_element));
				$param->setAttribute('value', a2u($d_value));
				$t_node->appendChild($param);
			} else {
				# as we don't dump repositories as a single object, we are to
				# dump template pool here
				my $pool = $doc->createElement('ip_pool');
				my $query_ip = dbDoQuery("SELECT r.component_id FROM Repository r WHERE r.rep_id='$d_value'");
				while (my ($ip_id) = $query_ip->fetchrow_array) {
					my $ip_node = $doc->createElement('ip_address');
					$ip_node->setAttribute('id', $ip_id);
					$pool->appendChild($ip_node);
				}
				$t_node->appendChild($pool);
			}
		}
		$base_node->appendChild($t_node);
	}

	return $base_node;
}


sub dumpCrontabRef($ $; $)
{
	my ($doc, $user, $server) = @_;

	my $crontab = $doc->createElement('crontab');

	# set attribute username for server-wide users
	if ($server eq 'server') {
		$crontab->setAttribute('user', $user);
	}

	addCrontab2Stream($crontab, $user);

	return $crontab;
}


return 1;

