###################################################################
package BU::PSA::Backup::CommonFunc;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::XML::CommonFunc;
use BU::PSA::MIME::Add2Stream;
use BU::PSA::Map;

@ISA = qw(Exporter);

@EXPORT = qw(
createUserRef
createCardRef
createServicesRef
createEmailRef
readFPpw
addPGAdminToDB
);


sub createUserRef
{
	my ($doc, $type, $level, $login, $passwd, $href) = @_;

	my $base_node = $doc->createElement('user');

	$base_node->setAttribute('type', $type);
	$base_node->setAttribute('level', $level);

	if ("x$login" ne "x") {
		my $sn = $doc->createElement("login");
		$sn->setAttribute('name', a2u($login)) ;
		if( exists($$href{real}) and ($$href{real} eq 'true') ) {
			my ($uid,$pw) = (getPwData ("$login"))[2,1];
			print_stdout ("name = $login, uid = $uid\n") if $debug > 1;
			$$href{uid}=$uid;
		}
		print_stdout ("createUser: $login\n") if $debug > 1;

		if($passwd) {
			$sn->setAttribute('password', a2u($passwd)) ;
		} else {
			$$href{PW_TYPE} = 'empty';
		}

		foreach my $k (keys %$href) {
			print_stdout ("$k = ".$$href{$k}."\n") if $debug > 1;
			$sn->setAttribute($k, a2u($$href{$k}));
		}

		$base_node->appendChild($sn);
	}

	return $base_node;
}


sub createCardRef
{
	my ($doc, $href, $map_fields) = @_;

	my $base_node = undef;

	$map_fields = \%card_fields unless $map_fields;

	foreach my $i (keys %{$map_fields}) {
		next unless $$href{$i};

		$base_node = $doc->createElement('card') unless $base_node;

		my $sn = $doc->createElement($$map_fields{$i});
		$base_node->appendChild($sn); 
		$sn->appendChild($doc->createTextNode(a2u($$href{$i})));
	}

	return $base_node;
}


sub createServicesRef
{
	my ($doc, $href, @servs) = @_;

	my $base_node = $doc->createElement('services');

	foreach my $i (@servs) {
		$base_node->setAttribute($i, a2u($$href{$i}));
	}

	return $base_node;
}


sub createEmailRef
{
	my $doc = shift;
	my $addr = shift;
	my %attrs = @_;

    my $base_node = $doc->createElement('email');

	$base_node->appendChild($doc->createTextNode($addr));

	foreach my $attr (keys %attrs) {
		$base_node->setAttribute($attr, a2u($attr{$attr}));
	}

	return $base_node;
}


# need read /home/httpd/vhosts/make-5.nsk.bsgdesign.com/httpdocs/_vti_pvt/service.pwd
# exp:
## -FrontPage-
#fp_user:a16RxG/Jr0zys
## -FrontPage-
#fp_user:0Gco.FauZltj6

sub readFPpw
{
	my ($dom_dir) = @_;
	my $f="$dom_dir/httpdocs/_vti_pvt/service.pwd";
	my %fp_pw = ();

#	print_stdout ("readFPpw: file $f\n");
#   need read /home/httpd/vhosts/<domain_name>/httpdocs/_vti_pvt/service.pwd
	unless (open(FPPW, $f)) {
#		print_stderr("Unable to open file '$f': $!\n");
#	the situation when service.pwd is not exist is normal, and for restoration will
#	be used only fp_admin user and passwd (which was created throught psa control panel).
		return %fp_pw;
	}

	while(<FPPW>) {
		chomp;
		s/#.*$//;
		next if /^$/;
		my @p = split  ':';
		$fp_pw{$p[0]}=$p[1];
		print_stdout ("$p[0] = $p[1]\n") if $debug > 9;
	}
	close FPPW || print_stderr("Unable to close file '$f': $!\n");

	return %fp_pw;
}


#sub getSQLWhereStrByField($ $)
#{
#	my ($fieldName, $valuesHash) = @_;
#
#	my $sqlWhereStr = '';
#
#	foreach $fieldValue (keys %$valuesHash) {
#		if ($fieldValue) {
#			$sqlWhereStr .= ($sqlWhereStr ? ' or ' : '') . "$fieldName='$fieldValue'";
#		}
#	}
#
#	return $sqlWhereStr;
#}


sub addPGAdminToDB
{
	my ($db_name, $db_username, $add_del) = @_;

	return 1 if ($flags{'BACKUP_TYPE'} ne 'FULL');

	my $ignore_errors = 0;

#	my $path =  $conf{PIPE_UTIL}
#			. ' ' . semGetKey()
#			. ' "' . getProcessingLogFileName() . '"'
#			. ' "' . ($add_del eq 'add' ? 'add' : 'remove') . " permissions for postgresql admin to '$db_name' \" "
#			. " $ignore_errors "
#			. " \"\" "
#			. ' "umask 0022; ' . $conf{UTILS_DIR} . '/pg_manage ' . (($add_del eq 'add') ? 'add_db_usr' : 'rm_db_usr')
#			. ' ' . dbQuote($db_name) . ' ' . dbQuote($db_username) . '" |';

	my $path = pipe_util_execstr(
			($add_del eq 'add' ? 'add' : 'remove') . " permissions for postgresql admin to '$db_name'",
			$ignore_errors, 0,
			'',
			'umask 0022; ' . $conf{UTILS_DIR} . '/pg_manage ' . (($add_del eq 'add') ? 'add_db_usr' : 'rm_db_usr')
			. ' ' . shellArgQuote($db_name) . ' ' . shellArgQuote($db_username)
	);

	addFakeNode2Stream($path);

	return 1;
}



return 1;

