###################################################################
package BU::PSA::Backup::DNS;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::XML::CommonFunc;

@ISA = qw(Exporter);

@EXPORT = qw(
dumpDNS
dumpDNSTemplateRef
createHostRecordRef
);


sub dumpDNS
{
	my ($doc, $base_node, $dom_id) = @_;

	{### dump domain's dns zone recs
		my $node = $doc->createElement('dns');
		$base_node->appendChild($node);

		my $query = dbDoQuery('SELECT dns.id, dns.type, host, val, opt, time_stamp, ipa.id FROM dns_recs dns LEFT JOIN IP_Addresses ipa ON (dns.type="A" AND dns.val=ipa.ip_address) OR (dns.type="PTR" AND dns.host=ipa.ip_address) WHERE dom_id="' . $dom_id . '"');
		while (my ($id, $type, $host, $val, $opt, $time_stamp, $ip_address_id) = $query->fetchrow_array()) {
			if (!$val || !$host || !$type) {
				printf_stderr("Warning: incorrect dns record is detected for domain '%s'(dom_id:$dom_id) (host:'$host', val:'$val', type:'$type')\n", $domains{$dom_id}->getAttribute('name'));
				next;
			}

			my $sn = createHostRecordRef($doc, $type, $host, $val, $opt, $time_stamp, $ip_address_id);
			$sn->setAttribute('id', $id);
			$node->appendChild($sn);
		}
	}

	{### dump domain's dns master
		my $node = $doc->createElement('dns_masters');
		$base_node->appendChild($node);

		my $query = dbDoQuery('SELECT dom_id, id, ip_address FROM dns_masters WHERE dom_id="' . $dom_id . '"');
		while (my ($dom_id, $id, $ip) = $query->fetchrow_array()) {
			my $sn = $doc->createElement('record');
			$node->appendChild($sn);
			$sn->setAttribute('id', $id);
			$sn->setAttribute('ip', $ip);
		}
	}
}


sub dumpDNSTemplateRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('dns');

	$base_node->setAttribute('template', 1);

	my $query = dbDoQuery('SELECT  type, host, val, opt, time_stamp  FROM dns_recs_t');
	while (my ($type, $host, $val, $opt, $time_stamp) = $query->fetchrow_array()) {
		addSubNode($base_node, createHostRecordRef($doc, $type, $host, $val, $opt, $time_stamp));
	}

	return $base_node;
}


sub createHostRecordRef
{
	my ($doc, $type, $host, $val, $opt, $time_stamp, $ip_address_id) = @_;

	my $base_node = $doc->createElement('host');

	$base_node->setAttribute('type',$type);

	if( $type eq 'PTR' ) {
		if ($ip_address_id) {
			$base_node->setAttribute('ip_address_id',$ip_address_id);
		} else {
			$base_node->setAttribute('ip', $host);
		}

		$base_node->setAttribute('name', a2u($val));
		$base_node->setAttribute('netmask',$opt);
	} elsif( $type eq 'A' ) {
		$base_node->setAttribute('name', a2u($host));

		if ($ip_address_id) {
			$base_node->setAttribute('ip_address_id',$ip_address_id);
		} else {
			$base_node->setAttribute('ip',$val);
		}
	} elsif( $type eq 'NS' ) {
		$base_node->setAttribute('name', a2u($host));
		$base_node->setAttribute('ns', a2u($val));
	} elsif( $type eq 'CNAME' ) {
		$base_node->setAttribute('name', a2u($host));
		$base_node->setAttribute('cname', a2u($val));
	} elsif( $type eq 'MX' ) {
		$base_node->setAttribute('name', a2u($host));
		$base_node->setAttribute('mx', a2u($val));
		$base_node->setAttribute('priority',$opt);
	} elsif( $type eq 'TXT' ) {
		$base_node->setAttribute('name', a2u($host));
		$base_node->setAttribute('text', a2u($val));
	} else {
		print_stderr("Unknow type \'$type\' DNS in record.\n");
	}
	$base_node->setAttribute('time_stamp',$time_stamp);

	return $base_node;
}

return 1;

