###################################################################
package BU::PSA::Backup::Database;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::PSA::MIME::Add2Stream;
use BU::XML::CommonFunc;

use BU::PSA::Backup::CommonFunc;

@ISA = qw(Exporter);

@EXPORT = qw(
createDatabaseNodeRef
dumpSystemDB
dumpIMPHordeDBRef
dumpDatabases
dumpDatabaseUsers
);


my $MYSQL_USER_PRIVILEGES = 'Host, User, Password, Select_priv, Insert_priv, Update_priv'
							. ', Delete_priv, Create_priv, Drop_priv, Reload_priv, Shutdown_priv'
							. ', Process_priv, File_priv, Grant_priv, References_priv'
							. ', Index_priv, Alter_priv';

my $MYSQL_DB_PRIVILEGES = 'Host, Db, User, Select_priv, Insert_priv, Update_priv'
						. ', Delete_priv, Create_priv, Drop_priv, Grant_priv, References_priv'
						. ', Index_priv, Alter_priv';

if (ver2int($const{MYSQL_VERSION}) >= ver2int('4.0.0')) {
	$MYSQL_USER_PRIVILEGES .= ', Show_db_priv, Super_priv, Create_tmp_table_priv, Lock_tables_priv, Execute_priv, Repl_slave_priv, Repl_client_priv';
	$MYSQL_DB_PRIVILEGES .= ', Create_tmp_table_priv, Lock_tables_priv';
}


# Store mysq databases which managed by PSA
my %existsDatabases = ();
my $existsDatabasesList = undef;


sub dumpDatabases
{
	my ($doc, $base_node, $dom_id) = @_;

	return undef unless $backup_filter{'DATABASES'};

	### fetch existing databases
	unless ($existsDatabasesList) {
		my $query = dbDoQuery('show databases');
		while (my ($db_name) = $query->fetchrow_array) {
			$existsDatabases{$db_name} = 1;
		}
		$existsDatabasesList = 1;
	}

	my $query = dbDoQuery('SELECT id, name, type FROM data_bases WHERE dom_id="' . $dom_id . '"');
	while (my ($db_id, $name, $type) = $query->fetchrow_array()) {
		if ($type eq 'mysql') {
			if (!exists($existsDatabases{$name})) {
				print_stderr("Database '$name' backup is skipped: database is not exist.\n");
				next;
			}
		} elsif ($type eq 'postgresql') {
			my $res = checkPostgresDBExisting($name);
			if ($res < 0) {
				printf_stderr("Database %s backup is skipped: error during checking database existing.\n", $name);
				next;
			} elsif ($res == 0) {
				printf_stderr("Database %s backup is skipped: database is not exist.\n", $name);
				next;
			}
		}

		my $node = createDatabaseNodeRef($doc, $name, $type);
		$base_node->appendChild($node);

		### set database id for SiteApp related resources checking
		$node->setAttribute('id', $db_id);

		### dump database users
		addSubNode($node, dumpDatabaseUsers($doc, $node, $db_id, $type));
	}
}


sub createDatabaseNodeRef($ $ $)
{
	my ($doc, $name, $type) = @_;

	my $base_node = $doc->createElement('data_bases');
	$base_node->setAttribute('name', a2u($name));
	$base_node->setAttribute('type', $type);

	if ($type eq 'mysql') {
		addMysql2Stream($base_node, $name);

		my $query=dbDoQuery("SELECT $MYSQL_DB_PRIVILEGES FROM mysql.db WHERE db=" . dbCorrectDBName($name));
		while (my $href = $query->fetchrow_hashref) {
			delete($$href{'Db'});
			addSubNode($base_node, createNodeFromHash($doc, 'access_rights', $href));
		}
	} elsif ($type eq 'postgresql') {
		addPGAdminToDB($name, $conf{PG_ADMIN_LOGIN}, 'add') if ($conf{PG_ADMIN_LOGIN});
		addPostgres2Stream($base_node, $name);
		addPGAdminToDB($name, $conf{PG_ADMIN_LOGIN}, 'del') if ($conf{PG_ADMIN_LOGIN});
	}

	return $base_node;
}


sub dumpSystemDB
{
	my ($doc, $base_node) = @_;

	{ # dumping not psa databases
		my %excludedDBList = ();
		my $query = dbDoQuery('SELECT name FROM data_bases');
		return '' unless $query;
		while (my ($name) = $query->fetchrow_array) {
			$excludedDBList{$name} = 1;
		}
		$excludedDBList{$const{PSA_DB_NAME}} = 1;
		$excludedDBList{$const{WEBMAIL_DB_NAME}} = 1;
		$excludedDBList{'mysql'} = 1;
		$excludedDBList{'test\_%'} = 1;

		$query = dbDoQuery('show databases');
		return '' unless $query;

		while (my $href = $query->fetchrow_hashref) {
			my $name = $$href{'Database'};
			next if ($name eq '');

			unless (exists($excludedDBList{$name})) {
				$base_node->appendChild(createDatabaseNodeRef($doc, $name, 'mysql'));
			}
		}
	}

	{ # dumping not psa databse users
		my %excludedUsersList = ();
		my $query = dbDoQuery('SELECT login FROM db_users');
		return '' unless $query;
		while (my ($login) = $query->fetchrow_array) {
			$excludedUsersList{$login} = 1;
		}
		$excludedUsersList{$const{PSA_ADMIN_NAME}} = 1;
		$excludedUsersList{$const{WEBMAIL_ADMIN_NAME}} = 1;
		
		$query = dbDoQuery("SELECT $MYSQL_USER_PRIVILEGES FROM mysql.user GROUP BY User");
		return '' unless $query;
		while (my $href = $query->fetchrow_hashref) {
			my $login = $$href{'User'};
			next if ($login eq '');
			delete($$href{'User'});

			unless (exists($excludedUsersList{$login})) {
				$sn = createUserRef($doc, 'db_users', 5, $login, $$href{'Password'}, {'PW_TYPE'=>'crypt'});
				$sn->appendChild(createNodeFromHash($doc, 'access_rights', $href));

				$sn->setAttribute('not_psa_user', 'true');
				$base_node->appendChild($sn);
			}
		}
	}

	return 1;
}


sub dumpIMPHordeDBRef
{  
	my ($doc) = @_;

	my $base_node = $doc->createElement('data_bases');

	$base_node->setAttribute('name', 'horde');
	$base_node->setAttribute('type', 'mysql');
	addMysql2Stream($base_node, 'horde', {'force' => 1});

	return $base_node;
}


sub dumpDatabaseUsers
{
	my ($doc, $base_node, $db_id, $db_type) = @_;

	my $query = dbDoQuery('SELECT dbu.id, dbu.login, a.type, a.password FROM db_users dbu LEFT JOIN accounts a on dbu.account_id=a.id WHERE dbu.db_id="' . $db_id . '"');
	while (my ($id, $login, $password_type, $password) = $query->fetchrow_array()) {
		my $node = createUserRef($doc, 'db_users', '5', $login, a2u($password), {'id' => $id, 'PW_TYPE' => $password_type} );

		if ($db_type eq 'mysql') {
			my $query = dbDoQuery("SELECT $MYSQL_USER_PRIVILEGES FROM mysql.user WHERE User=". dbQuote($login));
			while (my $href = $query->fetchrow_hashref()) {
				delete($$href{'User'});
				$node->appendChild(createNodeFromHash($doc, 'access_rights', $href));
			}
		} elsif ($db_type eq 'postgresql') {
		}
		$base_node->appendChild($node);
	}
}


return 1;

