###################################################################
package BU::PSA::Backup::Domain;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::PSA::MIME::Add2Stream;
use BU::XML::CommonFunc;

use BU::PSA::Backup::CommonFunc;
use BU::PSA::Backup::Common;
use BU::PSA::Backup::Mail;
use BU::PSA::Backup::DNS;
use BU::PSA::Backup::MailList;
use BU::PSA::Backup::Hosting;
use BU::PSA::Backup::Database;
use BU::PSA::Backup::ActionLog;
use BU::PSA::Backup::DomainLevelUser;

@ISA = qw(Exporter);

@EXPORT = qw(dumpDomains dumpDomainRef);

sub dumpDomains
{
	my ($doc, $base_node, $cl_id, $dumpingBy) = @_;

	### make new node 'sites' for the client
	my $node = $doc->createElement('sites');
	$base_node->appendChild($node);

	my $domains_ids = undef;
	my $domains_ids_skip = undef;
	my $domains_names = undef;
	my $domains_names_skip = undef;

	if (exists($$dumpingBy{domains_ids})) {
		$domains_ids = $$dumpingBy{domains_ids};
	} elsif (exists($$dumpingBy{domains_names})) {
		$domains_names = $$dumpingBy{domains_names};
	} elsif (exists($$dumpingBy{domains_ids_skip})) {
		$domains_ids_skip = $$dumpingBy{domains_ids_skip};
	} elsif (exists($$dumpingBy{domains_names_skip})) {
		$domains_names_skip = $$dumpingBy{domains_names_skip};
	}

	my $query = dbDoQuery('SELECT * FROM domains d where cl_id="' . $cl_id . '"');
	while (my $href = $query->fetchrow_hashref()) {
		delete $$href{cl_id};

		if ($domains_ids) {
			next if not exists($$domains_ids{$$href{id}});
		} elsif ($domains_names) {
			next if not exists($$domains_names{$$href{name}});
		} elsif ($domains_ids_skip) {
			next if exists($$domains_ids_skip{$$href{id}});
		} elsif ($domains_names_skip) {
			next if exists($$domains_names_skip{$$href{name}});
		}

		addSubNode($node, dumpDomainRef($doc, $href));
	}
}


sub dumpDomainRef
{
	my ($doc, $href) = @_;

	my $base_node = $doc->createElement('domain');

	$base_node->setAttribute('id', $$href{id});

	######################################################
	#   For server dump:
	#   Determine case when given domain is currently in
	#   dump/restore state.
	#   If it's so - skip it to mantain dump integrity.
	#   Domain dump skips this step
	######################################################
	if (not exists $flags{SINGLE_DOMAIN_MODE} or not $flags{SINGLE_DOMAIN_MODE}) {
		my $query = dbDoQuery('SELECT status FROM domains WHERE id="' . $$href{id} . '"');
		my ($domain_status) = $query->fetchrow_array;

		if ($domain_status & 4) {
			################################################################
			#   Domain was turned off due to the backup/restore procedure.
			#   We can't include it into server dump because it can be
            #   inconsistent currently.
            ################################################################
#			my $query = dbDoQuery('SELECT name FROM domains WHERE id="' . $$href{id} . '"');
#			my ($domain_name) = $query->fetchrow_array;
#			addNotDumpedDomain($domain_name);
			next;
		}
	}

	my @dom_attr = ('name', 'displayName', 'cr_date', 'dns_zone', 'status', 'htype', 'dns_type');
	foreach my $i (@dom_attr) {
		$base_node->setAttribute($i, a2u($$href{$i}));
	}

	### dump disk usage of domain
	my $sn = $doc->createElement('size');
	$sn->setAttribute( 'real',$$href{real_size});
	$base_node->appendChild($sn);

	unless ($backup_filter{'HOSTING'}) {
		### dirty hack for htype (htype should be located in hosting table)
		$base_node->setAttribute('htype', 'none');
	} else {
		### dump domain's hosting
		if ($$href{htype} eq 'vrt_hst') {
			if (my $sn = dumpPhysHostingRef($doc, $$href{id})) {
				$base_node->appendChild($sn);
			}
		} elsif ($$href{htype} =~ /(std|frm)_fwd/) {
			if (my $sn = dumpForwardingRef($doc, $$href{id})) {
				$base_node->appendChild($sn);
			}
		}
	}

	### dump domain prefs (action log, custom reports and buttons, limits, certs repository ...)
	dumpDomainPrefs($doc, $base_node, $$href{id});

	### dump domain's dns prefs
	dumpDNS($doc, $base_node, $$href{id});

	### dump domain services (tomcat, mail, maillists)
	dumpDomainServices($doc, $base_node, $$href{id});

	### dump mail users
	dumpMailNames($doc, $base_node, $$href{id});

	### dump databases
	dumpDatabases($doc, $base_node, $$href{id});

	### dump domain level user
	addSubNode($base_node, dumpDomainLevelUserRef($doc, $$href{id}, $$href{name}));

	return $base_node;
}


sub dumpDomainPrefs
{
	my ($doc, $base_node, $dom_id) = @_;

	### dump domain's action log
	addSubNode($base_node, dumpActionLogRef($doc, $dom_id, 'domain'));

	### dump domain's custom reports
	addSubNode($base_node, dumpCustomReportsRef($doc, $dom_id, $const{'IS_DOMAIN_OWNER'}));

	### dump domain's custom buttons
	addSubNode($base_node, dumpCustomButtonsRef($doc, $id, $const{'IS_DOMAIN_OWNER'}));

	{### dump certificates repository for domain
		my $node = $doc->createElement('certificates_list');
		$base_node->appendChild($node);
		my $query = dbDoQuery('SELECT r.component_id FROM domains d, Repository r, certificates cert WHERE d.cert_rep_id=r.rep_id AND cert.id=r.component_id AND d.id="' . $dom_id . '"');
		while ( my ($cert_id) = $query->fetchrow_array ) {
			my $sn = $doc->createElement('certificate');
			$sn->setAttribute('id', $cert_id);
			$node->appendChild($sn);
		}
	}

	{### dump domain's limits
		my $node = $doc->createElement('limits');
		$base_node->appendChild($node);
		my $query = dbDoQuery('SELECT l.limit_name, l.value FROM domains d, Limits l WHERE d.limits_id=l.id AND d.id="' . $dom_id . '"');
		while ( my ($name, $value) = $query->fetchrow_array ) {
			$node->setAttribute($name, $value);
		}

		### dump action log for domain's limits
		addSubNode($node, dumpActionLogRef($doc, $dom_id, 'domain_limits'));
	}

	{### dump domain's parameters
		my $node = $doc->createElement('params');
		$base_node->appendChild($node);
		my $query = dbDoQuery('SELECT param, val FROM dom_param WHERE NOT(param LIKE "%_id") AND dom_id="' . $dom_id . '"');
		while (my ($param,  $value) = $query->fetchrow_array) {
			$node->setAttribute($param, a2u($value));
		}
	}

	{### dump domain's parameters (new params)
		my $node = $doc->createElement('ext_params');
		$base_node->appendChild($node);
		my $query = dbDoQuery('SELECT p.parameter, p.value FROM domains d, Parameters p WHERE d.params_id=p.id AND d.id="' . $dom_id . '"');
		while ( my ($name, $value) = $query->fetchrow_array ) {
			$node->setAttribute($name, a2u($value));
		}
	}

	### dump domain's disk usage
	if ($backup_filter{'HOSTING'} || $backup_filter{'MAIL'} || $backup_filter{'DATABASES'}) {
		my $query = dbDoQuery('SELECT * FROM disk_usage WHERE dom_id="' . $dom_id . '"');
		while ( my $href = $query->fetchrow_hashref ) {
			my $node = $doc->createElement('disk_usage');
			$base_node->appendChild($node);

			delete $$href{dom_id};
			while (my ($field, $value) = each (%$href)) {
				$node->setAttribute($field, $value);
			}
		}
	}

	{### dump domain's certs repository
		my $query = dbDoQuery('SELECT r.component_id FROM domains d, Repository r WHERE d.cert_rep_id=r.rep_id');
		while ( my ($cert_id) = $query->fetchrow_array ) {
			my $node = $doc->createElement('certificate');
			$base_node->appendChild($node);
			$node->setAttribute('id', $cert_id);
		}
	}

	{### dump domain traffic
		my $query = dbDoQuery('SELECT * FROM DomainsTraffic WHERE dom_id="' . $dom_id . '"');
		while (my $href = $query->fetchrow_hashref) {	
			my $node = $doc->createElement('traffic_statistic');
			$base_node->appendChild($node);

			delete($$href{dom_id});
			my $sn = createNodeFromHash($doc, 'record', $href);
			$node->appendChild($sn);
		}
	}
}


sub dumpDomainServices
{
	my ($doc, $base_node, $dom_id) = @_;

	{### dumping domain's services
		my $node = $doc->createElement('domain_services');
		$base_node->appendChild($node);

		my $SERVICES_FILTER = '';
		$SERVICES_FILTER .= $backup_filter{'HOSTING'}	? '"tomcat",' : '';
		$SERVICES_FILTER .= $backup_filter{'MAIL'}		? '"mail",' : '';
		$SERVICES_FILTER .= $backup_filter{'MAILLISTS'}	? '"maillists",' : '';

		if ($SERVICES_FILTER) {
			$SERVICES_FILTER =~ s/.$//;
			$SERVICES_FILTER = " AND (type in ($SERVICES_FILTER))";
		}
		my $query = dbDoQuery('SELECT * FROM DomainServices WHERE dom_id="' . $dom_id . '"' . $SERVICES_FILTER);
		while (my $href = $query->fetchrow_hashref) {
			delete($$href{dom_id});

			my $ds_params_id = $$href{'parameters_id'};
			delete($$href{'parameters_id'});

			my $sn = createNodeFromHash($doc, 'domain_service', $href);
			$node->appendChild($sn);

			if ($ds_params_id) {
				my $query = dbDoQuery('SELECT parameter, value FROM Parameters WHERE id=' . dbQuote($ds_params_id));
				my $sn2 = $doc->createElement('params');
				$sn->appendChild($sn2);
				while (my ($parameter, $value) = $query->fetchrow_array()) {
					$sn2->setAttribute($parameter, a2u($value));
				}
			}

			if ($$href{type} eq 'tomcat') {
				next if (($flags{BACKUP_TYPE} ne 'FULL'));

				# dump domain's web applications
				my $query = dbDoQuery('SELECT * FROM WebApps WHERE domain_service_id=' . $$href{id});
				while (my $href2 = $query->fetchrow_hashref) {
					delete($$href2{domain_service_id});
					my $webAppNode = createNodeFromHash($doc, 'web_app', $href2);
					$sn->appendChild($webAppNode);

					###############################
					#	Web application file name
					###############################
					my $filename = $$href2{name} . '.war';
					my $path = $conf{CATALINA_HOME} . '/psa-wars/' . $base_node->getAttribute('name') . '/' . $filename;
					addFile2Stream($webAppNode, $$href2{name}, $path, 'application/x-war-webapp', 0);
				}
			} elsif ($$href{type} eq 'maillists') {
				my $query = dbDoQuery('SELECT name, status FROM MailLists WHERE dom_id="' . $dom_id . '"');
				while (my ($ml_name, $ml_status) = $query->fetchrow_array()) {
				    $sn->appendChild(dumpMailList($doc, $ml_name, $ml_status));
				}
			} else { #elsif ($$href{type} eq 'mail')
				### do nothing
			}
		} # end while
	} ### dump domain's services
}


return 1;

