###################################################################
package BU::PSA::Backup::Hosting;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::XML::CommonFunc;
use BU::PSA::MIME::Add2Stream;

use BU::PSA::Backup::CommonFunc;
use BU::PSA::Backup::Common;
use BU::PSA::Backup::WebUser;
use BU::PSA::Backup::PDir;
use BU::PSA::Backup::SiteApp;
use BU::PSA::Backup::SubDomain;
use BU::PSA::Backup::AnonFTP;
use BU::PSA::Backup::ActionLog;

@ISA = qw(Exporter);

@EXPORT = qw(dumpPhysHostingRef dumpForwardingRef);


sub dumpPhysHostingRef
{
	my ($doc, $dom_id) = @_;

	my $base_node = undef;

	return undef unless $backup_filter{'HOSTING'};

	my @hu_services=('ssi', 'php', 'cgi', 'perl', 'asp', 'ssl', 'webstat', 'python', 'coldfusion', 'same_ssl');

	my $query = dbDoQuery('SELECT d.name, d.htype, h.*, su.*, a.password, a.type FROM hosting h, domains d, sys_users su LEFT JOIN accounts a on (su.account_id=a.id) WHERE d.id=h.dom_id AND h.sys_user_id=su.id AND d.id="' . $dom_id . '"');
	if ( my $href = $query->fetchrow_hashref ) {
		my $domainName = $$href{name};
		my $htype = $$href{htype};
		delete $$href{name};
		delete $$href{htype};

		$base_node = createUserRef($doc, 'hosting', '3', $$href{login}, $$href{password},
							{'real' => 'true', 'PW_TYPE' => $$href{type}, 'shell' => a2u($$href{shell}),
							 'quota' => $$href{quota}, 'home' => a2u($$href{home})});

		if (($flags{BACKUP_TYPE} eq 'FULL') && ($domains_backup_dirs{$htype})) {
			my $uid = getpwnam($$href{login});
			addDomain2Stream($base_node, $uid, $domainName);
			my $sys_data_node = $doc->createElement('system_data');
			$base_node->appendChild($sys_data_node);
			addDomain2Stream($sys_data_node, 0, $domainName);

			# dump crontab for system user
			addSubNode($base_node, dumpCrontabRef($doc, $$href{login}));
		}

		$base_node->setAttribute('at_domains', $$href{at_domains});

		{### dump ip address
			my $node = $doc->createElement('ip_address');
			$node->setAttribute('id', $$href{ip_address_id});
			$base_node->appendChild($node);
		}

		my $sn = createServicesRef($doc, $href, @hu_services);
		$base_node->appendChild($sn);

		if ($$href{'fp_adm'}) {
			my $sn2 = $doc->createElement('fp');
			$sn2->setAttribute('fp_auth',$$href{fp_enable});
			$sn2->setAttribute('ssl', (($$href{fp_ssl} eq 'true') ? 'true' : 'false'));
			$sn2->setAttribute('not_ssl', (($$href{fp} eq 'true') ? 'true' : 'false'));

			$sn2->appendChild(createUserRef($doc, 'fp', '7', $$href{'fp_adm'}, $$href{'fp_pass'},
								{'PW_TYPE' => 'plain', 'ADMIN' => 'true'}));

			if (($$href{fp} eq 'true') || ($$href{fp_ssl} eq 'true')) {
				### store additional fp users only if fp service is turned on
				my %pw = readFPpw($conf{HTTPD_VHOSTS_D} . '/' . $domainName);
				delete($pw{$$href{'fp_adm'}});

				foreach my $i (keys %pw) {
					$sn2->appendChild(createUserRef($doc, 'fp', '7', $i, $pw{$i},
						{"PW_TYPE" => "des"} ));
				}
			}
			$sn->appendChild($sn2);
		}


		### dump actions for physical hosting
		addSubNode($base_node, dumpActionLogRef($doc, $dom_id, 'phys_hosting'));

		{### dump traffic value
			my $sn = $doc->createElement('traffic');
			$sn->setAttribute('real',$$href{real_traffic});
			$base_node->appendChild($sn);
		}

		if (exists($$href{'rootchain'})) {
			my $sn = $doc->createElement('rootchain_certificate');
			$sn->setAttribute('status', $$href{'rootchain'});

			addDomainRootChainCert2Stream($sn, $domainName) if
				(($$href{'rootchain'} eq 'true') && ($flags{BACKUP_TYPE} eq 'CONF'));

			$base_node->appendChild($sn);
		}

		### dump subdomains
		addSubNode($base_node, dumpSubDomainsRef($doc, $dom_id, $domainName));

		### dump domain's site apps
		addSubNode($base_node, dumpSiteAppsRef($doc, 'domain', $dom_id));

		### dump anonymous ftp service
		addSubNode($base_node, dumpAnonFTPRef($doc, $dom_id));

		### dump webusers
		dumpWebUsers($doc, $base_node, $dom_id, $domainName);

		### dump protected dirs
		addSubNode($base_node, dumpPDirs($doc, $base_node, $dom_id));
	}

	return $base_node;
}


sub dumpForwardingRef
{
	my ($doc, $dom_id) = @_;

	my $base_node = undef;

	return undef unless $backup_filter{'HOSTING'};

	my $query = dbDoQuery('SELECT redirect, ip_address_id FROM forwarding WHERE dom_id="' . $dom_id . '"');
	if (my ($redirect, $ip_address_id) = $query->fetchrow_array()) {
		$base_node = $doc->createElement('forwarding');
		$base_node->setAttribute('redirect', $redirect);
		$base_node->setAttribute('ip_address_id', $ip_address_id);
	}

	return $base_node;
}


return 1;

