###################################################################
package BU::PSA::Backup::Mail;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::XML::CommonFunc;
use BU::PSA::MIME::Add2Stream;

use BU::PSA::Backup::CommonFunc;
use BU::PSA::Backup::SpamAssassin;

@ISA = qw(Exporter);

@EXPORT = qw(dumpMailNames dumpMailNameRef);


sub dumpMailNames
{
	my ($doc, $base_node, $dom_id) = @_;

	return unless $backup_filter{'MAIL'};

	$query = dbDoQuery('SELECT d.name as domain_name, m.*, a.type, a.password FROM domains d, mail m LEFT JOIN accounts a ON (m.account_id=a.id) WHERE m.dom_id=d.id AND d.id=' . dbQuote($dom_id));
	while ( my $href = $query->fetchrow_hashref()) {
		my $domainName = $$href{domain_name};
		delete $$href{domain_name};

		addSubNode($base_node, dumpMailNameRef($doc, $href, $domainName));
	}
}


sub dumpMailNameRef
{
	my ($doc, $href, $domainName) = @_;

	my $base_node = createUserRef($doc, 'mail', '4', $$href{mail_name}, $$href{password},
							{'PW_TYPE' => ($$href{type} ? $$href{type} : 'empty'),
							'cp_access' => $$href{cp_access}});

	my $mn_id = $$href{id};

	if ($flags{BACKUP_TYPE}	eq 'FULL') {
		addMailbox2Stream($base_node, "$domainName/".$$href{mail_name});
	}

	### dump mailname's services
	my @mail_services = ('postbox','mail_group','redirect','autoresponder');
	my $node = createServicesRef($doc, $href, @mail_services);
	$base_node->appendChild($node);

	my $sn = undef;

	### dump mailname's service "redirect"
	$sn = $doc->createElement('forwarding');
	$node->appendChild($sn);
	$sn->setAttribute('redirect', $$href{redirect});
	$sn->appendChild(createEmailRef($doc, $$href{redir_addr})) if $$href{redir_addr};

	### dump mailname's service "mailgroup"
	$sn = $doc->createElement('forwarding');
	$node->appendChild($sn);
	$sn->setAttribute('mailgroup', $$href{mail_group});
	my $query = dbDoQuery('SELECT address FROM mail_redir WHERE mn_id="' . $mn_id . '"');
	while (my ($email) = $query->fetchrow_array) {
		$sn->appendChild(createEmailRef($doc, $email));
	}

	### dump mailname's params
	addSubNode($base_node, dumpMailNameParamsRef($doc, $mn_id));

	### dump mailname's aliases
	addSubNode($base_node, dumpMailNameAliasesRef($doc, $mn_id));

	### dump attachment repositories
	my $attaches_dir = $conf{QMAIL_MAILNAMES_D}."/$domainName/".$$href{mail_name}.'/@attachments';
	if (my $sn = dumpMailNameAttachsRef($doc, $attaches_dir)) {
		$base_node->appendChild($sn);
	}

	### dump mailname's autoresponders
	dumpMailNameAutoresponders($doc, $base_node, $mn_id);

	### dump spamassassin prefs for mailname
	dumpMailNameSpamAssassin($doc, $base_node, $mn_id);

	return $base_node;
}


sub dumpMailNameAliasesRef
{
	my ($doc, $mn_id) = @_;

	my $base_node = undef;

	my $query = dbDoQuery('SELECT alias FROM mail_aliases WHERE mn_id="' . $mn_id . '"');
	while (my ($alias) = $query->fetchrow_array) {
		unless ($base_node) {
			$base_node = $doc->createElement('aliases');
		}
		my $node = $doc->createElement('alias');
		$node->setAttribute('name', a2u($alias));
		$base_node->appendChild($node);
	}

	return $base_node;
}


sub dumpMailNameParamsRef
{
	my ($doc, $mn_id) = @_;

	my $base_node = $doc->createElement('params');

	unless( $$href{mbox_quota} ==  -1 ) {
		$base_node->setAttribute('mbox_quota', $$href{mbox_quota});
	}

	my $query = dbDoQuery('SELECT param, val FROM mn_param WHERE mn_id="' . $mn_id . '"');
	while (my ($param, $value) = $query->fetchrow_array) {
		my $node = $doc->createElement('parameter');
		$node->setAttribute('name', a2u($param));
		$node->setAttribute('value', a2u($value));
		$base_node->appendChild($node);
	}

	return $base_node;
}


sub dumpMailNameAttachsRef
{
	my ($doc, $attaches_dir) = @_;

	my $base_node = undef;

	### get list or files in repositary for specified mail
	my @files_at_repository=();
	if (-d $attaches_dir) {
		unless (open ( FD , "ls $attaches_dir/ |" )) {
			print_stderr("Unable to retrieve the list of files in repository");
		} else {
			while (<FD>) {
				my $fname=$_;
				chomp $fname;
				print_stderr ("Processing file $fname\n") if ($debug>3);
				next unless ($fname);
				unless (-e "$attaches_dir/$fname") {
					print_stderr("Unable to find file $attaches_dir/$fname\n") ;
					next;
				}
				push ( @files_at_repository, $fname);
			}

			close FD or 
				warn ("Some problems occured during an attempt to close the pipe while getting files list from the repository");
				print_stderr ("Files in repository dir '$attaches_dir': ", join (", ", @files_at_repository), "\n") if ($debug>2);
		}
	}

	### dump attachment files
	if ($#files_at_repository >= 0) {
		$base_node = $doc->createElement('repository');

		foreach my $filename (@files_at_repository)	{
			my $node = $doc->createElement('file');
			$base_node->appendChild($node);
			$node->setAttribute('name', a2u($filename));
			addAttach2Stream($node, $attaches_dir, $filename);
		}
	}

	return $base_node;
}


sub dumpMailNameAutoresponders
{
	my ($doc, $base_node, $mn_id) = @_;

	my $query = dbDoQuery('SELECT * FROM mail_resp WHERE mn_id="' . $mn_id . '"');
	while ( my $href = $query->fetchrow_hashref()) {
		delete $$href{mn_id};
		my $rn_id = $$href{id};

		my $node = $doc->createElement('autoresponder');
		$base_node->appendChild($node);

		$node->setAttribute('name',$$href{resp_name});
		$node->setAttribute('turn',$$href{resp_on});

		my $sn = $doc->createElement('params');
		$node->appendChild($sn);
		$sn->setAttribute('ans_freq',$$href{ans_freq});
		$sn->setAttribute('mem_limit',$$href{mem_limit});
		$sn->setAttribute('reply_to',$$href{reply_to});

		my $sn = $doc->createElement('keystr');
		$node->appendChild($sn);
		$sn->setAttribute('where',$$href{key_where});
		$sn->appendChild($doc->createTextNode(a2u($$href{keystr})));
	
		my $sn = $doc->createElement('message');
		$node->appendChild($sn);
		$sn->setAttribute('subject',a2u($$href{subject}));
		$sn->appendChild($doc->createTextNode(a2u($$href{text})));

		{### dump autoresponder's forwards
			my $query = dbDoQuery('SELECT address FROM resp_forward WHERE rn_id="' . $rn_id . '"');
			while (my ($email) = $query->fetchrow_array()) {
				my $sn = $doc->createElement('forwarding');
				if ($email) {
					$sn->setAttribute('autoresponder', 'true');
					$sn->appendChild(createEmailRef($doc, $email));
				} else {
					$sn->setAttribute('autoresponder', 'false');
				}
				$node->appendChild($sn);
			}
		}

		{### dump autoresponder's attachments
			my $query = dbDoQuery('SELECT filename FROM resp_attach WHERE rn_id="' . $rn_id . '"');
			while ( my ($file) = $query->fetchrow_array ) {
				my $sn = $doc->createElement('attach');
				$sn->setAttribute('file', a2u($file));
				$node->appendChild($sn);
			}
		}
	}
}


sub dumpMailNameSpamAssassin
{
	my ($doc, $base_node, $mn_id) = @_;

	my $query = dbDoQuery('SELECT sa.* FROM sa_conf sa, mail m, domains d WHERE d.id=m.dom_id AND CONCAT(m.mail_name, "@", d.name)=sa.mailname AND m.id="' . $mn_id . '"');
	while (my $href = $query->fetchrow_hashref()) {
		my $mailname = $$href{'mailname'};
		delete $$href{'mailname'};

		addSubNode($base_node, dumpSpamAssassinConfRef($doc, $mailname, $href));
	}
}


return 1;

