####################################################
package BU::PSA::Backup::Main;
####################################################
require 5.000;
require Exporter;

use BU::PSA::Const;
use BU::PSA::CommonFunc;
use BU::PSA::Logger;
use BU::PSA::DB;
use BU::PSA::MIME::Add2Stream;
use BU::XML::CommonFunc;

use MIME::Base64 qw(encode_base64 decode_base64);

use BU::PSA::Backup::CommonFunc;
use BU::PSA::Backup::Common;
use BU::PSA::Backup::Server;
use BU::PSA::Backup::Admin;
use BU::PSA::Backup::Client;
use BU::PSA::Backup::Database;


@ISA = qw(Exporter);

@EXPORT = qw(dumpPSADocref);

sub dumpPSADocref($)
{
	my ($dumpingBy) = @_;

	$conf{PG_ADMIN_LOGIN} = getMiscParam('postgresql_admin_login');
	$conf{PG_ADMIN_PASSWORD} = getMiscParam('postgresql_admin_passwd');

	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parse($psa_xml_template);

	my $base_node = $doc->getChildNodes()->item(0);

	$base_node->setAttribute('sqlVer', getMiscParam('version'));
	$base_node->setAttribute('creating_timestamp', time());

	if ($conf{DESCRIPTION}) {
		my $descNode = $doc->createElement('description');
		$descNode->appendChild($doc->createTextNode(a2u($conf{DESCRIPTION})));
		$base_node->appendChild($descNode);
	}

	### dump server's side prefs
	addSubNode($base_node, dumpServer($doc));

	### dump admin's info and prefs
	addSubNode($base_node, dumpAdminUserRef($doc));

	dumpClients($doc, $base_node, $dumpingBy);

	if ($backup_filter{'SERVER'}) {
		### dump crontabs of the server page
		my @serverUsers = ($const{'ADM_USER'});

	
		$serverUsers[$#serverUsers] = 'mailman' if (defined $conf{MAILMAN_ROOT_D} && -d $conf{MAILMAN_ROOT_D});
		foreach my $user (@serverUsers) {
			addSubNode($base_node, dumpCrontabRef($doc, $user, 'server'));
		}
	}

	if ($backup_filter{'DATABASES'}) {
		my ($by_what, $tmp) = each(%$dumpingBy);

		### for fast work not PSA databases will be dumped first
		unless ($by_what) {
			my $systemNode = $doc->createElement('system');
			dumpSystemDB($doc, $systemNode);
			$systemNode->appendChild(dumpIMPHordeDBRef($doc));
			$base_node->appendChild($systemNode);
		}
	}

	return $doc;
}

return 1;

