###################################################
package BU::PSA::Backup::Server;
####################################################
require 5.000;
require Exporter;

use BU::PSA::Const;
use BU::PSA::CommonFunc;
use BU::PSA::Logger;
use BU::PSA::Map;
use BU::PSA::DB;
use BU::PSA::MIME::Add2Stream;
use BU::XML::File2xml;
use BU::XML::CommonFunc;

use BU::PSA::Backup::CommonFunc;
use BU::PSA::Backup::Common;
use BU::PSA::Backup::ActionLog;
use BU::PSA::Backup::SpamAssassin;
use BU::PSA::Backup::DNS;

use MIME::Base64 qw(encode_base64 decode_base64);

@ISA = qw(Exporter);

@EXPORT = qw(dumpServer dumpCertificatesRef dumpSiteAppPackagesRef);

sub dumpServer
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('server');

	my @server_params_attr = (
		'mbox_quota',
		'def_locale',
		'login_timeout', 'lock_interval', 'lock_attempts', 'lock_time',
		'relay', 'disable_smtp_auth', 'disable_pop_auth', 'poplock_time',
		'slave_request_timeout',
		'restart_apache_interval',
		'allow_short_pop3_names',
		'slave_services_list_with_non_obligatory_status_checking',
		'master_services_list_for_obligatory_status_checking',
		'send_announce',
		'include_mailboxes',
		'include_databases',
		'include_logs',
		'include_maillists',
		'include_servlets',
		'include_webapps',
		'include_domaindumps',
		'actionlog_rot_type', 'actionlog_rot_period',
		'actionlog_rot_num_periods', 'actionlog_rot_num_records',
		'postgresql_admin_login', 'postgresql_admin_passwd',
        'access_policy',
		'use_vocabulary', 'stat_ttl', 'traffic_accounting',
		'exp_warn_time', 'multiply_login'
# WHAT TODO ???
#def_skin_id
#default_certificate_id
#cert_rep_id
#stat_timestamp	 -      (  BillingSystem)
	);

	{ ### dump server params
		my $sn = undef;
		my $psn = $doc->createElement('params');
		$base_node->appendChild($psn);
		foreach my $i (@server_params_attr) {
			$psn->setAttribute($i, a2u(getMiscParam($i)));
		}

		### dump some params from /etc/psa/psa.conf (section: server params)
		my $sn = $doc->createElement('psa_conf');
		$psn->appendChild($sn);
		$sn->setAttribute('HTTPD_VHOSTS_D', $conf{HTTPD_VHOSTS_D});
		$sn->setAttribute('MAILMAN_ROOT_D', $conf{MAILMAN_ROOT_D});

		### dump host information (section: server params)
		$sn = $doc->createElement('ip');
		$psn->appendChild($sn);
		$sn->setAttribute('tagname','FullHostName');
		$sn->setAttribute('host',getMiscParam('FullHostName'));
	}

	if ($backup_filter{'SERVER'}) {
		{### dump vhost skel (section: server)
			$sn = $doc->createElement('vhost_skeleton');
			$base_node->appendChild($sn);
			$sn->setAttribute('admin', 'true');
			addDir2Stream($sn, 0, '.', $conf{HTTPD_VHOSTS_D} . '/.skel/0');
		}

		{### dump default domain (section: server)
			my $sn = $doc->createElement('domain');
			$base_node->appendChild($sn);
			$sn->setAttribute('name','default');
			addDir2Stream($sn, 0, '.', $conf{HTTPD_VHOSTS_D} . '/default');
		}

		### dump /etc/psa/psa.key (section: server)
		if (open (FD, $const{'PSA_KEY'})) {
			binmode FD;
			my $buff = undef;
			if (sysread(FD, $buff, $const{'PSA_KEY_BUFF_SIZE'})) {
				$sn = $doc->createElement('psa_key');
				$base_node->appendChild($sn);
				$sn->appendChild($doc->createTextNode(encode_base64($buff)));
			}
			close FD;
		} else {
			printf_stderr("Unable to open '%s' file: %s\n", $const{'PSA_KEY'}, $!);
		}

		### dump DNS template (section: server)
		if(my $sn = dumpDNSTemplateRef($doc)) {
			$sn->setAttribute('dns_zone_status', (getMiscParam('dns_zone_status') eq 'false') ? 'off' : 'on');
			$base_node->appendChild($sn);
		}

		{### dump additional services (section: server)
			my $esn = $doc->createElement('additional_services');
			$base_node->appendChild($esn);
			my @extras_arr = ('extras', 'cert_purchasing', 'domain_registration');
			foreach $i (@extras_arr) {
				my $en = $doc->createElement($i);
				$param_value = getMiscParam($i);
				$en->setAttribute('enabled', $param_value ? $param_value : 'true');
				$esn->appendChild($en);
			}
		}

		### dump store event handlers
		addSubNode($base_node, dumpEventHandlersRef($doc));

		### dump NTP service
		if (getMiscParam('ntp_server')) {
			my $sn = $doc->createElement('ntp_service');
			$base_node->appendChild($sn);
			$sn->setAttribute('enabled', (getMiscParam('ntp_enabled') eq 'true') ? 'true' : 'false');
			$sn->setAttribute('server', getMiscParam('ntp_server'));
		}

		### dump rbl service (spam protection)
		if (getMiscParam('rbl_server')) {
			my $sn = $doc->createElement('rbl_server');
			$base_node->appendChild($sn);
			$sn->setAttribute('status', (getMiscParam('rbl') eq 'true') ? 'true' : 'false');
			$sn->setAttribute('server', getMiscParam('rbl_server'));
		}

		{### dump spamassassin server configuration
			my $query = dbDoQuery('SELECT * FROM sa_conf WHERE isnull(mailname) or mailname=""');
			while (my $href = $query->fetchrow_hashref()) {
				addSubNode($base_node, dumpSpamAssassinConfRef($doc, '', $href));
			}
		}

		### dump white list of domains
		addSubNode($base_node, dumpMailWhiteListRef($doc));

		### dump bad_mail_from list of domains
		addSubNode($base_node, dumpBadMailFromListRef($doc));
	}

	{### store admin apache certificate
		my $sn = $doc->createElement('certificate');
		$base_node->appendChild($sn);
		addAdminCert2Stream($sn);
	}

	# dump admin's apache rootchain certificate
	if (getMiscParam('rootchain')) {
		my $sn = $doc->createElement('rootchain_certificate');
		$base_node->appendChild($sn);
		$sn->setAttribute('status', getMiscParam('rootchain'));
		addAdminRootChainCert2Stream($sn);
	}

	### dump ip addresses
	addSubNode($base_node, dumpIPAddressesRef($doc));

	### dump certificates
	addSubNode($base_node, dumpCertificatesRef($doc));

	if ($backup_filter{'SERVER'}) {
		### Server-wide system DNS settings
		addSubNode($base_node, dumpDNSAllowTransferRef($doc));

		### dump cp access prefs
		addSubNode($base_node, dumpCPAccessRef($doc));

		### dump certificates common(server) repository
		addSubNode($base_node, dumpServerCertsRepositoryRef($doc));

		### dump siteapps packages
		addSubNode($base_node, dumpSiteAppPackagesRef($doc));

		### dump action log prefs
		addSubNode($base_node, dumpActionLogPrefsRef($doc));

		### dump upgrade history
		addSubNode($base_node, dumpUpgradeHistoryRef($doc));

		### dump templates of mail notes
		addSubNode($base_node, dumpNotesRef($doc));

		### dump CP skins
		addSubNode($base_node, dumpSkinsRef($doc));

		### dump slave servers info (enterprise feature)
		addSubNode($base_node, dumpSlavesRef($doc));

		### dump mailer queries
		addSubNode($base_node, dumpMailerQueriesRef($doc));

		### dump client templates
		addSubNode($base_node, dumpClientTemplatesRef($doc));

		### dump domain templates
		addSubNode($base_node, dumpDomainTemplatesRef($doc));
	}

	return $base_node;
}


sub dumpEventHandlersRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('event_handlers_list');

	my $query = dbDoQuery('SELECT * FROM event_handlers');
	while (my $href = $query->fetchrow_hashref) {
		my $sn = createNodeFromHash($doc, 'event_handler', $href);
		$base_node->appendChild($sn);
	}

	return $base_node;
}

sub dumpBadMailFromListRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('badmailfrom');

	my $query = dbDoQuery('select * from badmailfrom');
	while (my $href = $query->fetchrow_hashref) {
		my $sn = $doc->createElement('domain');
		$sn->setAttribute('name', $$href{'domain'});
		$base_node->appendChild($sn);
	}

	return $base_node;
}


sub dumpCPAccessRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('cp_access');

	my $query = dbDoQuery('SELECT type, netaddr, netmask FROM cp_access');
	while (my ($type, $ip, $mask) = $query->fetchrow_array()) {
		my $sn = $doc->createElement('rule');
		$sn->setAttribute('type', $type);
		$sn->setAttribute('ip', $ip);
		$sn->setAttribute('mask', $mask);
		$base_node->appendChild($sn);
	}

	return $base_node;
}


sub dumpServerCertsRepositoryRef
{
	my ($doc) = @_;

	my $cert_rep_id = getMiscParam('cert_rep_id');
	return undef unless $cert_rep_id;

	my $base_node = $doc->createElement('admin_certificates_list');

	my $query = dbDoQuery('SELECT cert.id FROM certificates cert, Repository r WHERE cert.id=r.component_id AND r.rep_id=' . $cert_rep_id);
	while ( my ($cert_id) = $query->fetchrow_array() ) {
		my $sn = $doc->createElement('certificate');
		$sn->setAttribute('id', $cert_id);
		$base_node->appendChild($sn);
	}

	return $base_node;
}


sub dumpSiteAppPackagesRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('site_app_packages');

	my $query = dbDoQuery('SELECT * FROM SiteAppPackages');
	while (my $href = $query->fetchrow_hashref()) {
		my $sn = createNodeFromHash($doc, 'site_app', $href);
		$base_node->appendChild($sn);
	}

	return $base_node;
}


###############################################
#	Dumping DNS_Allow_Transfer% from psa.misc
#	<DNS_Allow_Transfer>
#		<transfer ip='...'/>
###############################################
sub dumpDNSAllowTransferRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('DNS_Allow_Transfer');

	my $query = dbDoQuery('SELECT val FROM misc WHERE param LIKE "DNS_Allow_Transfer%"');
	while (my ($at) = $query->fetchrow_array()) {
		my $sn = $doc->createElement('transfer');
		$sn->setAttribute('ip', $at);
		$base_node->appendChild($sn);
	}

	return $base_node;
}


sub dumpNotesRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('notifications');

	my %notes = ();
	my $query = dbDoQuery('SELECT * FROM Notes');
	while (my ($id, $text) = $query->fetchrow_array) {
		my $sn = $doc->createElement('message');
		$base_node->appendChild($sn);
		$notes{$id} = $sn;
		$sn->setAttribute('id', $id);
		$sn->appendChild($doc->createTextNode(a2u($text)));
	}

	$query = dbDoQuery('SELECT * FROM Notifications');
	while (my $href = $query->fetchrow_hashref) {
		my $note_id = $$href{'note_id'};
		delete $$href{'note_id'};
#		delete $$href{'id'};	# fix of bug #26745

		next unless exists ($notes{$note_id});

		my $sn = $doc->createElement('desc');
		$sn->appendChild($doc->createTextNode(a2u($$href{subj})));
		delete $$href{'subj'};
		while (my ($field, $value) = each (%$href)) {
			$sn->setAttribute($field, $value);
		}

		$notes{$note_id}->appendChild($sn);
	}

	return $base_node;
}


sub dumpSkinsRef($)
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('skins');

	my $server_default_skin_id = getMiscParam('def_skin_id');
	my $admin_skin_id = getMiscParam('admin_skin_id');

	my $query = dbDoQuery('SELECT id, name, place FROM Skins');
	while (my ($id, $name, $path) = $query->fetchrow_array()) {
		$path =~ m/^((.+)\/)?([^\/]+)(\/|)$/;
		$path = $1;
		my $skin_dir = $3;

		my $sn = $doc->createElement('skin');
		$sn->setAttribute('name', a2u($name));
		$sn->setAttribute('path', a2u($path));
		$sn->setAttribute('dir', a2u($skin_dir));

		# mark default skin
		$sn->setAttribute('server_default_skin', 'true') if ($server_default_skin_id && ($server_default_skin_id eq $id));

		# mark admin skin
		$sn->setAttribute('admin_skin', 'true') if ($admin_skin_id && ($admin_skin_id eq $id));

		if ($path =~ m/^[^\/]/) {
			$path = "/$path";
		}

		addDir2Stream($sn, 0, $skin_dir, $conf{PRODUCT_ROOT_D} . "/admin/htdocs/skins/$path" );

		$base_node->appendChild($sn);
	}

	return $base_node;
}


sub dumpUpgradeHistoryRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('upgrade_history');

	my $query = dbDoQuery('SELECT * FROM upgrade_history');
	while (my $href = $query->fetchrow_hashref) {
		my $sn = $doc->createElement('upgrade');
		delete $$href{id};
		while (my ($field, $value) = each (%$href)) {
			$sn->setAttribute($field, $value);
		}
		$base_node->appendChild($sn);
	}

	return $base_node;
}


sub dumpSlavesRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('slave_servers');

	my %slave_params = ();
	my $query = dbDoQuery('SELECT * FROM slaves');
	while (my ($id, $hostname) = $query->fetchrow_array) {
		my $sn = $doc->createElement('server');
		$base_node->appendChild($sn);

		$sn->setAttribute('hostname', $hostname);

		my $sn2 = $doc->createElement('params');
		$sn->appendChild($sn2);
		$slave_params{$id} = $sn2;
	}

	$query = dbDoQuery('SELECT slave_id, parameter, value FROM slave_params WHERE parameter!="cert_id" ORDER BY slave_id');
	while (my ($slave_id, $parameter, $value) = $query->fetchrow_array) {
		next unless exists $slave_params{$slave_id};
		my $sn = $doc->createElement($parameter);
		$sn->setAttribute('value', a2u($value));
		$slave_params{$slave_id}->appendChild($sn);
	}

	$query = dbDoQuery('SELECT * FROM certificates c, slave_params sp WHERE sp.parameter="cert_id" AND sp.value=c.id');
	while (my $href = $query->fetchrow_hashref ) {
		my $node = $doc->createElement('certificate');
		$node->setAttribute('name', a2u($$href{name}));
		$node->setAttribute('time_stamp', $$href{time_stamp});

		my $sn = $doc->createElement('pub_key');
		addTextCertKey2Stream($sn, $$href{cert});
		$node->appendChild($sn);

		if (exists($$href{pvt_key}) && $$href{pvt_key}) {
			my $sn = $doc->createElement('pvt_key');
			addTextCertKey2Stream($sn, $$href{pvt_key});
			$node->appendChild($sn);
		}

		if (exists($$href{csr}) && $$href{csr}) {
			my $sn = $doc->createElement('csr');
			addTextCertKey2Stream($sn, $$href{csr});
			$node->appendChild($sn);
		}

		if (exists($$href{ca_cert}) && $$href{ca_cert}) {
			my $sn = $doc->createElement('ca_cert');
			addTextCertKey2Stream($sn, $$href{ca_cert});
			$node->appendChild($sn);
		}

		$slave_params{$$href{slave_id}}->appendChild($node);
	}

	return $base_node;
}


sub dumpMailerQueriesRef($)
{
	my ($doc) = @_;

	my $node = $doc->createElement('mailer_queries');
	my $query = dbDoQuery('SELECT * FROM mailer_queries');
	while (my $href = $query->fetchrow_hashref) {
		my $sn = $doc->createElement('query');

		delete $$href{id};

		while (my ($field, $value) = each (%$href)) {
			$sn->setAttribute($field, a2u($value));
		}

		$node->appendChild($sn);
	}

	return $node;
}


#
# Save system ip addresses list.
#
sub dumpIPAddressesRef($)
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('ip_addresses_list');

	my $query = dbDoQuery('SELECT * FROM IP_Addresses');
	while (my $href = $query->fetchrow_hashref) {
		my $sn = createNodeFromHash($doc, 'ip_address', $href);
		$base_node->appendChild($sn);
		addSubNode($sn, dumpActionLogRef($doc, $$href{'id'}, 'ip_address'));
	}

	return $base_node;
}


sub dumpCertificatesRef
{
	my ($doc, $certs_hash) = @_;

	my $base_node = $doc->createElement('certificates_list');

	my $default_certificate_id = getMiscParam('default_certificate_id');

	############################################################
	#	Get id of the certificate, which has been used by BU
	############################################################
	my  $query       = dbDoQuery('SELECT val FROM misc WHERE param="bu_cert_id"');
	my ($bu_cert_id) = $query->fetchrow_array;

	$query = dbDoQuery('SELECT * FROM certificates');

	while (my $href = $query->fetchrow_hashref) {
		if ($certs_hash && !$$certs_hash{$$href{id}}) {
			next;
		}

		my $node = $doc->createElement('certificate');

		$node->setAttribute('id', $$href{id});
		$node->setAttribute('default', 'true') if ($$href{'id'} == $default_certificate_id);

		######################################
		#	Dump BU certificate distinctly.
		######################################
		if ($bu_cert_id eq $$href{id}) {
			######################################################
			#	Thats BU cert, add him an appropriate attribute.
			#	It has neither name nor time_stamp.
			######################################################
			$node->setAttribute('bu_cert', 'true');
		} else {
			$node->setAttribute('name', a2u($$href{name}));
			$node->setAttribute('time_stamp', $$href{time_stamp});
		}

		if (exists($$href{cert}) && $$href{cert}) {
			my $sn = $doc->createElement('pub_key');
			addTextCertKey2Stream($sn, $$href{cert});
			$node->appendChild($sn);
		}

		if (exists($$href{pvt_key}) && $$href{pvt_key}) {
			my $sn = $doc->createElement('pvt_key');
			addTextCertKey2Stream($sn, $$href{pvt_key});
			$node->appendChild($sn);
		}

		if (exists($$href{csr}) && $$href{csr}) {
			my $sn = $doc->createElement('csr');
			addTextCertKey2Stream($sn, $$href{csr});
			$node->appendChild($sn);
		}

		if (exists($$href{ca_cert}) && $$href{ca_cert}) {
			my $sn = $doc->createElement('ca_cert');
			addTextCertKey2Stream($sn, $$href{ca_cert});
			$node->appendChild($sn);
		}

		$base_node->appendChild($node);
	}

	return $base_node;
}


sub dumpMailWhiteListRef
{
	my ($doc) = @_;

	my $base_node = $doc->createElement('white_list');

	my $query = dbDoQuery('SELECT ip_address, ip_mask FROM smtp_poplocks WHERE lock_time IS NULL');
	while (my ($ip, $mask) = $query->fetchrow_array) {
		my $sn = $doc->createElement('address');
		$sn->setAttribute('ip', $ip);
		$sn->setAttribute('mask', $mask);
		$base_node->appendChild($sn);
	}

	return $base_node;
}



return 1;

