###################################################################
package BU::PSA::Backup::SingleDomain;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::CommonFunc;
use BU::XML::CommonFunc;

use BU::PSA::Backup::CommonFunc;
use BU::PSA::Backup::Common;
use BU::PSA::Backup::Domain;
use BU::PSA::Backup::Server;

@ISA = qw(Exporter);

@EXPORT = qw(dumpSingleDomainDocref);

sub dumpSingleDomainDocref($)
{
	my ($domain_name) = @_;

	$conf{PG_ADMIN_LOGIN} = getMiscParam('postgresql_admin_login');
	$conf{PG_ADMIN_PASSWORD} = getMiscParam('postgresql_admin_passwd');

	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parse($psa_xml_template);

	$psa_node=$doc->getChildNodes()->item(0);

	$psa_node->setAttribute('sqlVer', getMiscParam('version'));

	if ($conf{DESCRIPTION}) {
		my $descNode = $doc->createElement('description');
		$descNode->appendChild($doc->createTextNode(a2u($conf{DESCRIPTION})));
		$psa_node->appendChild($descNode);
	}

	$server_node = $doc->createElement('server');
	$psa_node->appendChild($server_node);

	my $psn = $doc->createElement('params');
	$server_node->appendChild($psn);
	my $sn = $doc->createElement('psa_conf');
	$sn->setAttribute('HTTPD_VHOSTS_D', $conf{HTTPD_VHOSTS_D});
	$sn->setAttribute('MAILMAN_ROOT_D', $conf{MAILMAN_ROOT_D});
	$psn->appendChild($sn);

	$sn = $doc->createElement('ip_addresses_list');
	$server_node->appendChild($sn);
	my $cert_id_for_ip = undef;
	{
		my $query = dbDoQuery('SELECT ipa.* FROM IP_Addresses ipa, domains d, hosting h WHERE d.id=h.dom_id AND ipa.id=h.ip_address_id AND d.name=' . dbQuote($domain_name));
		if (my $hash = $query->fetchrow_hashref()) {
			$sn->appendChild(createNodeFromHash($doc, 'ip_address', $hash));
			$cert_id_for_ip = $$hash{ssl_certificate_id};
		} else {
			my $query = dbDoQuery('SELECT ipa.* FROM IP_Addresses ipa, domains d, dns_recs dns WHERE d.id=dns.dom_id AND dns.type="A" AND dns.val=ipa.ip_address AND (dns.host=CONCAT(d.name, ".") OR dns.host=CONCAT(d.displayName, ".")) AND d.name=' . dbQuote($domain_name));
			if (my $hash = $query->fetchrow_hashref()) {
				$sn->appendChild(createNodeFromHash($doc, 'ip_address', $hash));
				$cert_id_for_ip = $$hash{ssl_certificate_id};
			} else {
				print_stderr("Unable to define ip address.\n");
			}
		}
	}

	{ # dump required certificates
		%certs_hash = ();
		$certs_hash{$cert_id_for_ip} = 1;
		$query = dbDoQuery('SELECT r.component_id FROM domains d, Repository r, certificates cert WHERE d.cert_rep_id=r.rep_id AND cert.id=r.component_id AND d.name=' . dbQuote($domain_name) . ' ORDER by d.id');
		while (my ($cert_id) = $query->fetchrow_array) {
			$certs_hash{$cert_id} = 1;
		}
		$server_node->appendChild(dumpCertificatesRef($doc, \%certs_hash));
	}

	$server_node->appendChild(dumpSiteAppPackagesRef($doc));


	my $query = dbDoQuery('SELECT cl_id FROM domains d, clients c WHERE c.id=d.cl_id AND name=' . dbQuote($domain_name));
	my ($cl_id) = $query->fetchrow_array() if $query;
	unless ($cl_id) {
		print_stderr("Unable to locate client which is owner of domain '$domain_name'\n");
		return undef;
	}

	my %domains_names = ($domain_name => 1);
	my %dumpingBy = ('domains_names' => \%domains_names);
	dumpDomains($doc, $psa_node, $cl_id, \%dumpingBy);

	return $doc;
}

return 1;

