####################################################
package BU::PSA::Backup::Ver0600toXML;
####################################################
require 5.000;
require Exporter;

#use BU::PSA::Logger;
use BU::PSA::Const;
use BU::PSA::CommonFunc;
use BU::PSA::ActionLog;
use BU::PSA::Logger;
use BU::PSA::Map;
use BU::PSA::DB;
use BU::PSA::MIME::Add2Stream;
use BU::MIME::Container;
use BU::Semaphore;
use BU::XML::File2xml;
use BU::XML::CommonFunc;

use MIME::Base64 qw(encode_base64 decode_base64);

@ISA = qw(Exporter);
#@EXPORT = qw(createUser createCard createServices createEmail createHostRecord dumpDNS dumpDNS_Template domainName domainDir readFPpw dumpAdminUser dumpServer dumpClients dumpDomains dumpPSA);
@EXPORT = qw(dumpPSA dumpSingleDomain);


my $doc;
my $psa_node;
my $psa = '<?xml version="1.0"?><psa version="'.$conf{PSA_VERSION}.'"></psa>';

my $sqlWhereForClients;
my $sqlWhereForDomains;
my %domains = ();
my %hosting = ();

my $MYSQL_USER_PRIVILEGES = 'Host, User, Password, Select_priv, Insert_priv, Update_priv'
							. ', Delete_priv, Create_priv, Drop_priv, Reload_priv, Shutdown_priv'
							. ', Process_priv, File_priv, Grant_priv, References_priv'
							. ', Index_priv, Alter_priv';

my $MYSQL_DB_PRIVILEGES = 'Host, Db, User, Select_priv, Insert_priv, Update_priv'
						. ', Delete_priv, Create_priv, Drop_priv, Grant_priv, References_priv'
						. ', Index_priv, Alter_priv';

if (ver2int($const{MYSQL_VERSION}) >= ver2int('4.0.0')) {
	$MYSQL_USER_PRIVILEGES .= ', Show_db_priv, Super_priv, Create_tmp_table_priv, Lock_tables_priv, Execute_priv, Repl_slave_priv, Repl_client_priv';
	$MYSQL_DB_PRIVILEGES .= ', Create_tmp_table_priv, Lock_tables_priv';
}


sub createUser {
	my ($doc, $type, $level, $login, $passwd, $href) = @_;
	
	my $node = $doc->createElement("user");
	my $sn;
	
	$node->setAttribute('type', $type);
	$node->setAttribute('level', $level);
	if ("x$login" ne "x") {
#		$node->setAttribute('login', $login) ;
		$sn = $doc->createElement("login");
		$sn->setAttribute('name', a2u($login)) ;
		if( exists($href->{real}) and ($href->{real} eq 'true') ) {
			my ($uid,$pw) = (getPwData ("$login"))[2,1];
			print_stdout ("name = $login, uid = $uid\n") if $debug > 1;
			$href->{uid}=$uid;
#			unless( $passwd or ($href->{PW_TYPE} ne 'empty') ) {
#				if( $pw !~ /^\!\!/ && $pw !~ /[*]/ && $pw ne 'x' ) {
#					print_stdout ("name = $login, pw = $pw\n") if $debug > 1;
#					$passwd=$pw;
#					$href->{PW_TYPE}='crypt';
#				}
#				elsif ( $pw eq  'x' ) {
#					print_stderr("System password for user $login is empty!\n");
#					$href->{PW_TYPE}='empty';
#					$passwd = '';
#				}
#			}
		}
		print_stdout ("createUser: $login\n") if $debug > 1;

		if( $passwd ) {
			$sn->setAttribute('password', a2u($passwd)) ;
		} else {
			$href->{PW_TYPE} = 'empty';
		}

		foreach my $k (keys %{$href}) {
			print_stdout ("$k = ".$href->{$k}."\n") if $debug > 1;
			$sn->setAttribute($k, a2u($href->{$k}));
		}

		$node->appendChild( $sn );
	}
#	$node->setAttribute('password', $passwd) if $passwd;

	return $node;
}

sub createCard {
	my ($doc, $ref, $map_fields) = @_;
	my $k=0;
	my $i;
	my $card = $doc->createElement('card');

	$map_fields = \%card_fields unless $map_fields;
	
	foreach $i (keys %{$map_fields}) {
#	foreach $i (keys %card_fields) {
		next unless $ref->{$i};
		print_stdout ($map_fields->{$i}," == $i\n") if $debug > 3;
		my $sn=$doc->createElement( $map_fields->{$i} );
		$sn->appendChild($doc->createTextNode(a2u($ref->{$i})));
		$card->appendChild($sn); 
		$k=1;
	}
	return $card if $k;
	return 0;
}

sub createServices {
	my ($doc,$ref,@servs)=@_;
	my $node=$doc->createElement('services');
	
	foreach my $i (@servs) {
		$node->setAttribute($i, a2u($ref->{$i}));
	}

	return $node;
}


sub createEmail {
	my $doc = shift;
	my $addr = shift;
	my %attr = @_;

    my $node=$doc->createElement( 'email' );
	$node->appendChild($doc->createTextNode($addr));
	foreach my $i (keys %attr) {
		$node->setAttribute($i,a2u($attr{$i}));
	}
	return $node;
}

sub createHostRecord {
	my ($type, $host, $val, $opt, $time_stamp, $ip_address_id) = @_;

	my $node = $doc->createElement('host');
	$node->setAttribute('type',$type);
	if( $type eq 'PTR' ) {
		if ($ip_address_id) {
			$node->setAttribute('ip_address_id',$ip_address_id);
		} else {
			$node->setAttribute('ip', $host);
		}

		$node->setAttribute('name', a2u($val));
		$node->setAttribute('netmask',$opt);
	} elsif( $type eq 'A' ) {
		$node->setAttribute('name', a2u($host));

		if ($ip_address_id) {
			$node->setAttribute('ip_address_id',$ip_address_id);
		} else {
			$node->setAttribute('ip',$val);
		}
	} elsif( $type eq 'NS' ) {
		$node->setAttribute('name', a2u($host));
		$node->setAttribute('ns', a2u($val));
	} elsif( $type eq 'CNAME' ) {
		$node->setAttribute('name', a2u($host));
		$node->setAttribute('cname', a2u($val));
	} elsif( $type eq 'MX' ) {
		$node->setAttribute('name', a2u($host));
		$node->setAttribute('mx', a2u($val));
		$node->setAttribute('priority',$opt);
	} elsif( $type eq 'TXT' ) {
		$node->setAttribute('name', a2u($host));
		$node->setAttribute('text', a2u($val));
	} else {
		print_stderr("Unknow type \'$type\' DNS in record.\n");
	}
	$node->setAttribute('time_stamp',$time_stamp);
	return $node;
}


# Get DNS records.
sub dumpDNS {
	my ($doc) = @_;

	my %dns = ();

	$query = dbDoQuery('SELECT dom_id, dns.id, dns.type, host, val, opt, time_stamp, ipa.id FROM dns_recs dns LEFT JOIN IP_Addresses ipa ON (dns.type="A" AND dns.val=ipa.ip_address) OR (dns.type="PTR" AND dns.host=ipa.ip_address)');
	while ( my ($dom_id, $id, $type, $host, $val, $opt, $time_stamp, $ip_address_id) = $query->fetchrow_array ) {
		unless( exists $domains{$dom_id} ) {
			next;
		}

		unless( exists $dns{$dom_id}) {
			$dns{$dom_id} = $doc->createElement('dns');
			$domains{$dom_id}->appendChild($dns{$dom_id});
		}

		if (!$val || !$host || !$type) {
			printf_stderr("Warning: incorrect dns record is detected for domain '%s'(dom_id:$dom_id) (host:'$host', val:'$val', type:'$type')\n", $domains{$dom_id}->getAttribute('name'));
			next;
		}

		my $node = createHostRecord ($type, $host, $val, $opt, $time_stamp, $ip_address_id);
		$node->setAttribute('id', $id);
		$dns{$dom_id}->appendChild($node) if $node;
	} 

	$query = dbDoQuery('SELECT dom_id, id, ip_address FROM dns_masters WHERE dom_id <> 0 ORDER BY dom_id');
	my $curr_dom_id = -1;
	my $node = undef;
	while ( my ($dom_id, $id, $ip) = $query->fetchrow_array ) {
		next unless exists $domains{$dom_id};

		if ($curr_dom_id != $dom_id) {
			$node = $doc->createElement('dns_masters');
			$domains{$dom_id}->appendChild($node);
			$curr_dom_id = $dom_id;
		}

		my $sn = $doc->createElement('record');
		$sn->setAttribute('id', $id);
		$sn->setAttribute('ip', $ip);

		$node->appendChild($sn);
	}
}# dumpDNS()


sub dumpDNS_Template {
	my ($doc) = @_;
	my $dns = $doc->createElement('dns');
	$dns->setAttribute('template',1);
	
	$query = dbDoQuery("SELECT  type, host, val, opt, time_stamp  FROM dns_recs_t;");
	while ( my ($type, $host, $val, $opt, $time_stamp) = $query->fetchrow_array ) {
#		print_stdout ("$type, $host, $val, $opt, $time_stamp");
		my $node = createHostRecord ($type, $host, $val, $opt, $time_stamp);
		$dns->appendChild($node) if $node;
	}
	return $dns;
} #dumpDNS_Template()


sub domainName {
	my ($id) = @_;
	
	return '' unless exists $domains{$id};

	my $name = $domains{$id}->getAttribute('name');
	print_stdout ("domain name is $name\n") if $debug > 3;
	return $name;
}


sub domainDir {
	my ($id) = @_;
	return $conf{HTTPD_VHOSTS_D} . '/'.domainName($id);
}

# need read /home/httpd/vhosts/make-5.nsk.bsgdesign.com/httpdocs/_vti_pvt/service.pwd
# exp:
## -FrontPage-
#fp_user:a16RxG/Jr0zys
## -FrontPage-
#fp_user:0Gco.FauZltj6

sub readFPpw {
	my ($dom_dir) = @_;
	my $f="$dom_dir/httpdocs/_vti_pvt/service.pwd";
	my %fp_pw = ();

#	print_stdout ("readFPpw: file $f\n");
#   need read /home/httpd/vhosts/<domain_name>/httpdocs/_vti_pvt/service.pwd
	unless (open(FPPW, $f)) {
#		print_stderr("Unable to open file '$f': $!\n");
#	the situation when service.pwd is not exist is normal, and for restoration will
#	be used only fp_admin user and passwd (which was created throught psa control panel).
		return %fp_pw;
	}

	while(<FPPW>) {
		chomp;
		s/#.*$//;
		next if /^$/;
		my @p = split  ':';
		$fp_pw{$p[0]}=$p[1];
		print_stdout ("$p[0] = $p[1]\n") if $debug > 9;
	}
	close FPPW || print_stderr("Unable to close file '$f': $!\n");
	return %fp_pw;
}

sub createDatabaseNode($ $ $)
{
	my ($doc, $name, $type) = @_;

	my $node = $doc->createElement('data_bases');
	$node->setAttribute('name', a2u($name));
	$node->setAttribute('type', $type);

	if ($type eq 'mysql') {
		addMysql2Stream ($node, $name);

		my $query=dbDoQuery("SELECT $MYSQL_DB_PRIVILEGES FROM mysql.db WHERE db=" . dbCorrectDBName($name));
		while (my $hash = $query->fetchrow_hashref) {
			delete($hash->{'Db'});
			$node->appendChild(createNodeFromHash($doc, 'access_rights', $hash));
		}
	} elsif ($type eq 'postgresql') {
		addPGAdminToDB($name, $conf{PG_ADMIN_LOGIN}, 'add') if ($conf{PG_ADMIN_LOGIN});
		addPostgres2Stream($node, $name);
		addPGAdminToDB($name, $conf{PG_ADMIN_LOGIN}, 'del') if ($conf{PG_ADMIN_LOGIN});
	}

	return $node;
}


#
# MAKE admin user
#
sub dumpAdminUser {
	my ($doc) = @_;
	my @adm_params_attr = ('lines_per_page','dom_page','cl_sort', 'dom_sort');

	my $node=createUser($doc, 'admin', '0', 'admin', '', {'PW_TYPE' => 'empty'});
	my $sn=createCard($doc,\%misc,\%adm_card_fields);
	$node->appendChild( $sn ) if $sn;

	### dump actions
	if (my $sn = dumpActionLog($doc, 0, 'admin')) {
		$node->appendChild($sn);
	}

	if (my $sn = dumpCustomReports($doc, 0, $const{'IS_ADMIN'})) {
		$node->appendChild($sn);
	}
	if (my $sn = dumpCustomButtons($doc, 0, $const{'IS_ADMIN'})) {
		$node->appendChild($sn);
	}

	$psn = $doc->createElement('params');
	foreach my $i (@adm_params_attr) {
		$psn->setAttribute($i, $misc{$i});
	}

	#psa3.0 DUMPING Logos
	$query= dbDoQuery('SELECT l.name, l.fake, l.url, l.showMode FROM Logos l, misc m WHERE m.param="logo_id" and m.val=l.id');
	if (my ($file_name, $fake_file_name, $url, $showMode) = $query->fetchrow_array) {
		my $sn = $doc->createElement('logo');
		$sn->setAttribute('file', a2u($file_name));
		$sn->setAttribute('url', $url) if $url;
		$sn->setAttribute('show_mode', a2u($showMode)) if $showMode;
		addLogo2Stream($sn, $fake_file_name) if $fake_file_name;

		$psn->appendChild($sn);
	}

	$node->appendChild($psn);

	return $node;
}


sub dumpEventHandlers
{
	my ($doc) = @_;

	$node = $doc->createElement('event_handlers_list');

	$query = dbDoQuery('SELECT * FROM event_handlers');
	while (my $hash = $query->fetchrow_hashref) {
		$sn = createNodeFromHash($doc, 'event_handler', $hash);
		$node->appendChild($sn);
	}

	return $node;
}

sub dumpBadMailFromList
{
	my ($doc) = @_;
	my $node = $doc->createElement('badmailfrom');

	my $query = dbDoQuery('select * from badmailfrom');
	unless ($query) {
		printf_stderr("Unable to fetch badmailfrom list from db\n");
		return $node;
	}
	while (my $hash = $query->fetchrow_hashref) {
		$sn = $doc->createElement('domain');
		$sn->setAttribute('name', $$hash{'domain'});
		$node->appendChild($sn);
	}

	return $node;
}


#
# MAKE 'server'
#

sub dumpServer {
	my ($doc) = @_;
	my @server_params_attr = (
		'mbox_quota',
		'def_locale',
		'login_timeout', 'lock_interval', 'lock_attempts', 'lock_time',
		'relay', 'disable_smtp_auth', 'disable_pop_auth', 'poplock_time',
		'slave_request_timeout',
		'restart_apache_interval',
		'allow_short_pop3_names',
		'slave_services_list_with_non_obligatory_status_checking',
		'master_services_list_for_obligatory_status_checking',
		'send_announce',
		'include_mailboxes',
		'include_databases',
		'include_logs',
		'include_maillists',
		'include_servlets',
		'include_webapps',
		'include_domaindumps',
		'actionlog_rot_type', 'actionlog_rot_period',
		'actionlog_rot_num_periods', 'actionlog_rot_num_records',
		'postgresql_admin_login', 'postgresql_admin_passwd',
        'access_policy',
		'use_vocabulary', 'stat_ttl', 'traffic_accounting',
		'exp_warn_time', 'multiply_login'
# WHAT TODO ???
#def_skin_id
#default_certificate_id
#cert_rep_id
#stat_timestamp	 -      (  BillingSystem)
		);

	my @extras_arr = ('extras', 'cert_purchasing', 'domain_registration');

	my $node=$doc->createElement('server');
	my $psn = $doc->createElement('params');

	foreach my $i (@server_params_attr) {
		$psn->setAttribute($i, a2u($misc{$i}));
	}
	$node->appendChild($psn) if $psn;;

	my $sn = $doc->createElement('psa_conf');
	$sn->setAttribute('HTTPD_VHOSTS_D', $conf{HTTPD_VHOSTS_D});
	$sn->setAttribute('MAILMAN_ROOT_D', $conf{MAILMAN_ROOT_D});
	$psn->appendChild($sn);

	$sn = $doc->createElement('ip');
	$sn->setAttribute('tagname','FullHostName');
	$sn->setAttribute('host',$misc{'FullHostName'});
	$psn->appendChild($sn);

	if ($backup_filter{'SERVER'}) {
		$sn = $doc->createElement('vhost_skeleton');
		$sn->setAttribute('admin', 'true');
		addDir2Stream($sn, 0, '.', $conf{HTTPD_VHOSTS_D} . '/.skel/0');
		$node->appendChild($sn);

		$sn = $doc->createElement('domain');
		$sn->setAttribute('name','default');
		addDir2Stream($sn, 0, '.', $conf{HTTPD_VHOSTS_D} . '/default');
		$node->appendChild($sn);

		if (open (fd, $const{'PSA_KEY'})) {
			binmode fd;
			my $buff = undef;
			if (sysread(fd, $buff, $const{'PSA_KEY_BUFF_SIZE'})) {
				$sn = $doc->createElement('psa_key');
				$sn->appendChild($doc->createTextNode(encode_base64($buff)));
				$node->appendChild($sn);
			}
			close fd;
		} else {
			printf_stderr("Unable to open '%s' file: %s\n", $const{'PSA_KEY'}, $!);
		}

		if( $sn=dumpDNS_Template($doc) ) {
			$sn->setAttribute('dns_zone_status', ($misc{'dns_zone_status'} eq 'false') ? 'off' : 'on');
			$node->appendChild($sn);
		}

		# store extras
		my $esn = $doc->createElement('additional_services');
		foreach $i (@extras_arr) {
			my $en = $doc->createElement($i);
			$en->setAttribute('enabled', $misc{$i} ? $misc{$i} : 'true');
			$esn->appendChild($en);
		}
		$node->appendChild($esn);

		# store event handlers
		$node->appendChild(dumpEventHandlers($doc));

		# store NTP service
		if ($misc{'ntp_server'}) {
			$sn = $doc->createElement('ntp_service');
			$sn->setAttribute('enabled',($misc{'ntp_enabled'} eq 'true') ? 'true' : 'false');
			$sn->setAttribute('server',$misc{'ntp_server'});
			$node->appendChild($sn);
		}
	}

	if ($backup_filter{'SERVER'}) {
		# store rbl service (spam protection)
		if ($misc{'rbl_server'}) {
			$sn = $doc->createElement('rbl_server');
			$sn->setAttribute('status',($misc{'rbl'} eq 'true') ? 'true' : 'false');
			$sn->setAttribute('server',$misc{'rbl_server'});
			$node->appendChild($sn);
		}

		# store spamassassin server configuration
		{
			my $query = dbDoQuery('SELECT * FROM sa_conf WHERE isnull(mailname) or mailname=""');
			unless ($query) {
				print_stderr("Unable to retrive sa_conf server configuration from database.\n");
			} else {
				while (my $hashref = $query->fetchrow_hashref()) {
					my $sn = dumpSpamAssassinConf($doc, '', $hashref);
					$node->appendChild($sn);
				}
			}
		}

		# backup bad mail from domains list
		if (my $sn = dumpMailWhiteList($doc)) {
			$node->appendChild($sn);
		}

		# backup bad mail from domains list
		if (my $sn = dumpBadMailFromList($doc)) {
			$node->appendChild($sn);
		}
	}

	# store admin apache certificate
	$sn = $doc->createElement('certificate');
	addAdminCert2Stream($sn);
	$node->appendChild($sn);

	# store admin apache rootchain certificate
	if (exists($misc{'rootchain'})) {
		$sn = $doc->createElement('rootchain_certificate');
		$sn->setAttribute('status', $misc{'rootchain'});
		addAdminRootChainCert2Stream($sn);
		$node->appendChild($sn);
	}

	if ($sn = dumpIPAddresses($doc)) {
		$node->appendChild($sn);
	}

	
	if ($backup_filter{'SERVER'}) {
		##############################
		#	Server-wide system DNS settings
		##############################
		$node->appendChild(dumpDNSAllowTransfer($doc));

		$node->appendChild(dumpCPAccess($doc));
		$node->appendChild(dumpCertificates($doc));
	}

	$node->appendChild(dumpServerCertsRepository($doc));

	if ($backup_filter{'SERVER'}) {
		$node->appendChild(dumpSiteAppPackages($doc));
		$node->appendChild(dumpActionLogPrefs($doc));
	}

	return $node;
}


sub dumpCPAccess
{
	my ($doc) = @_;

	my $node = $doc->createElement('cp_access');

	my $query = dbDoQuery('SELECT type, netaddr, netmask FROM cp_access');
	while (my ($type, $ip, $mask) = $query->fetchrow_array()) {
		my $sn = $doc->createElement('rule');
		$sn->setAttribute('type', $type);
		$sn->setAttribute('ip', $ip);
		$sn->setAttribute('mask', $mask);
		$node->appendChild($sn);
	}

	return $node;
}


sub dumpServerCertsRepository
{
	my ($doc) = @_;

	my $cert_rep_id = getMiscParam('cert_rep_id');
	return 1 unless $cert_rep_id;

	my $node = $doc->createElement('admin_certificates_list');

	$query = dbDoQuery('SELECT cert.id FROM certificates cert, Repository r WHERE cert.id=r.component_id AND r.rep_id=' . $cert_rep_id);
	while ( my ($cert_id) = $query->fetchrow_array() ) {
		my $sn = $doc->createElement('certificate');
		$sn->setAttribute('id', $cert_id);
		$node->appendChild($sn);
	}

	return $node;
}


sub dumpSiteAppPackages
{
	my ($doc) = @_;

	my $node = $doc->createElement('site_app_packages');

	my $query = dbDoQuery('SELECT * FROM SiteAppPackages');
	unless ($query) {
		print_stderr("Unable to fetch site app packages from db.\n");
		return undef;
	}

	while (my $hash = $query->fetchrow_hashref()) {
		my $sn = createNodeFromHash($doc, 'site_app', $hash);
		$node->appendChild($sn);
	}

	return $node;
}


###############################################
#	Dumping DNS_Allow_Transfer% from psa.misc
#	<DNS_Allow_Transfer>
#		<transfer ip='...'/>
###############################################
sub dumpDNSAllowTransfer($) {

	my ($doc) = @_;
	my $node = $doc->createElement('DNS_Allow_Transfer');

	my $query = dbDoQuery('SELECT val FROM misc WHERE param LIKE "DNS_Allow_Transfer%"');

	while (my ($at) = $query->fetchrow_array()) {

		my $sn = $doc->createElement('transfer');
		$sn->setAttribute('ip', $at);
		$node->appendChild($sn);
	}

	return $node;
}

sub dumpNotes($)
{
	my ($doc) = @_;
	my $notes_node = $doc->createElement('notifications');
	my %notes = ();

	$query = dbDoQuery('SELECT * FROM Notes');
	while (my ($id, $text) = $query->fetchrow_array) {
		my $sn = $doc->createElement('message');
		$notes{$id} = $sn;
		$sn->setAttribute('id', $id);
		$sn->appendChild($doc->createTextNode(a2u($text)));
		$notes_node->appendChild($sn);
	}

	$query = dbDoQuery('SELECT * FROM Notifications');
	while (my $ref = $query->fetchrow_hashref) {
		my $note_id = $ref->{'note_id'};
		delete $ref->{'note_id'};
#		delete $ref->{'id'};	# fix of bug #26745

		next unless exists ($notes{$note_id});

		my $sn = $doc->createElement('desc');

		$sn->appendChild($doc->createTextNode(a2u($ref->{subj})));
		delete $ref->{'subj'};

		while (my ($field, $value) = each (%$ref)) {
			$sn->setAttribute($field, $value);
		}

		$notes{$note_id}->appendChild($sn);
	}

	return $notes_node;
}


sub dumpSkins($)
{
	my ($doc) = @_;

	my $node = $doc->createElement('skins');

	my $server_default_skin_id = getMiscParam('def_skin_id');
	my $admin_skin_id = getMiscParam('admin_skin_id');

	$query = dbDoQuery('SELECT id, name, place FROM Skins');
	while (my ($id, $name, $path) = $query->fetchrow_array) {
		$path =~ m/^((.+)\/)?([^\/]+)(\/|)$/;
		$path = $1;
		$skin_dir = $3;

		my $sn = $doc->createElement('skin');
		$sn->setAttribute('name', a2u($name));
		$sn->setAttribute('path', a2u($path));
		$sn->setAttribute('dir', a2u($skin_dir));

		# mark default skin
		$sn->setAttribute('server_default_skin', 'true') if ($server_default_skin_id && ($server_default_skin_id eq $id));

		# mark admin skin
		$sn->setAttribute('admin_skin', 'true') if ($admin_skin_id && ($admin_skin_id eq $id));

		if ($path =~ m/^[^\/]/) {
			$path = "/$path";
		}

		addDir2Stream($sn, 0, $skin_dir, $conf{PRODUCT_ROOT_D} . "/admin/htdocs/skins/$path" );

		$node->appendChild($sn);
	}

	return $node;
}


sub dumpUpgradeHistory($)
{
	my ($doc) = @_;

	my $node = $doc->createElement('upgrade_history');

	$query = dbDoQuery('SELECT * FROM upgrade_history');
	while (my $ref = $query->fetchrow_hashref) {
		my $sn = $doc->createElement('upgrade');

		delete $ref->{id};

		while (my ($field, $value) = each (%$ref)) {
			$sn->setAttribute($field, $value);
		}
		$node->appendChild($sn);
	}

	return $node;
}


sub dumpSlaves($)
{
	my ($doc) = @_;
	my %slave_params = ();

	my $node = $doc->createElement('slave_servers');

	$query = dbDoQuery('SELECT * FROM slaves');
	while (my ($id, $hostname) = $query->fetchrow_array) {
		my $sn = $doc->createElement('server');
		$sn->setAttribute('hostname', $hostname);

		my $sn2 = $doc->createElement('params');
		$sn->appendChild($sn2);
		$slave_params{$id} = $sn2;

		$node->appendChild($sn);
	}

	$query = dbDoQuery('SELECT slave_id, parameter, value FROM slave_params WHERE parameter!="cert_id" ORDER BY slave_id');
	while (my ($slave_id, $parameter, $value) = $query->fetchrow_array) {
		next unless exists $slave_params{$slave_id};
		my $sn = $doc->createElement($parameter);
		$sn->setAttribute('value', a2u($value));
		$slave_params{$slave_id}->appendChild($sn);
	}

	$query = dbDoQuery('SELECT * FROM certificates c, slave_params sp WHERE sp.parameter="cert_id" AND sp.value=c.id');
	while (my $hash = $query->fetchrow_hashref ) {
		my $node = $doc->createElement('certificate');
		$node->setAttribute('name', a2u($hash->{name}));
		$node->setAttribute('time_stamp', $hash->{time_stamp});

		my $sn = $doc->createElement('pub_key');
		addTextCertKey2Stream($sn, $hash->{cert});
		$node->appendChild($sn);

		if (exists($hash->{pvt_key}) && $hash->{pvt_key}) {
			my $sn = $doc->createElement('pvt_key');
			addTextCertKey2Stream($sn, $hash->{pvt_key});
			$node->appendChild($sn);
		}

		if (exists($hash->{csr}) && $hash->{csr}) {
			my $sn = $doc->createElement('csr');
			addTextCertKey2Stream($sn, $hash->{csr});
			$node->appendChild($sn);
		}

		if (exists($hash->{ca_cert}) && $hash->{ca_cert}) {
			my $sn = $doc->createElement('ca_cert');
			addTextCertKey2Stream($sn, $hash->{ca_cert});
			$node->appendChild($sn);
		}

		$slave_params{$hash->{slave_id}}->appendChild($node);
	}

	return $node;
}


sub dumpDomainTemplates($)
{
	my ($doc) = @_;

	my %clients_ids = ();
	$query = dbDoQuery('SELECT id, tmpl_id FROM clients WHERE tmpl_id > 0');
	while (my ($cl_id, $rep_id) = $query->fetchrow_array) {
		$clients_ids{$rep_id} = $cl_id;
	}

	my $common_node = $doc->createElement('domain_templates');
	my $node = undef;
	my %clients_tpls = ();
	my $common_tmpl_rep_id = getMiscParam('dom_tmpl_list_id');

	$query = dbDoQuery('SELECT r.rep_id, tpl.id, tpl.name, tpl.note_id FROM Templates tpl, Repository r WHERE tpl.id=r.component_id');
	while (my ($rep_id, $id, $name, $note_id) = $query->fetchrow_array) {
		if (exists $clients_ids{$rep_id}) {
			my $cl_id = $clients_ids{$rep_id};

			if (exists $clients_tpls{$cl_id}) {
				$node = $clients_tpls{$cl_id};
			} elsif (exists $clients{$cl_id}) {
				$node = $doc->createElement('domain_templates');
				$clients{$cl_id}->appendChild($node);
				$clients_tpls{$cl_id} = $node;
			} else {
				### such client with $cl_id is not exist
				next;
			}
		} elsif ($common_tmpl_rep_id && ($common_tmpl_rep_id == $rep_id)) {
			$node = $common_node;
		} else {
			$node = undef;
		}

		if ($node) {
			my $tpl_node = $doc->createElement('domain');
			$tpl_node->setAttribute('name', a2u($name));
			$tpl_node->setAttribute('note_id', $note_id);

			my $query2 = dbDoQuery('SELECT element, value FROM TmplData WHERE tmpl_id=' . $id);
			while (my ($field, $value) = $query2->fetchrow_array) {
				my $sn = undef;
				if ($field eq 'logrotation_id') {
					my $query3 = dbDoQuery('SELECT * FROM log_rotation WHERE id=' . dbQuote($value));
					if (my $ref = $query3->fetchrow_hashref()) {
						$sn = $doc->createElement('log_rotation');
						while (my ($field, $value) = each (%$ref)) {
							$sn->setAttribute($field, a2u($value));
						}	
					}
				} else {
					$sn = $doc->createElement(a2u($field));
					$sn->setAttribute('value', a2u($value));
				}
				$tpl_node->appendChild($sn);
			}
			$node->appendChild($tpl_node);
		}
	}

	return $common_node;
}


sub dumpClientTemplates ($) 
{
	my ($doc) = @_;

	my $ct_node = $doc->createElement('client_templates');
	
	my $query = dbDoQuery("SELECT t.id, t.name, t.note_id FROM Templates t, Repository r, misc m WHERE t.id=r.component_id AND r.rep_id=m.val AND m.param='cl_tmpl_list_id'");
	while (my ($t_id, $t_name, $t_note_id) = $query->fetchrow_array) {
		my $t_node = $doc->createElement('client');
		$t_node->setAttribute('name', a2u($t_name));
		$t_node->setAttribute('note_id', $t_note_id);
		my $query2 = dbDoQuery("SELECT t.element, t.value from TmplData t WHERE t.tmpl_id='$t_id'");
		while (my ($d_element, $d_value) = $query2->fetchrow_array) {
			if ($d_element ne 'tmpl_pool_id') {
				my $param = $doc->createElement(a2u($d_element));
				$param->setAttribute('value', a2u($d_value));
				$t_node->appendChild($param);
			} else {
				# as we don't dump repositories as a single object, we are to
				# dump template pool here
				my $pool = $doc->createElement('ip_pool');
				my $query_ip = dbDoQuery("SELECT r.component_id FROM Repository r WHERE r.rep_id='$d_value'");
				while (my ($ip_id) = $query_ip->fetchrow_array) {
					my $ip_node = $doc->createElement('ip_address');
					$ip_node->setAttribute('id', $ip_id);
					$pool->appendChild($ip_node);
				}
				$t_node->appendChild($pool);
			}
		}
		$ct_node->appendChild($t_node);
	}
	return $ct_node;
}

########################################
#
# Dump crontab of system user
#
# parameter server means dumping server-wide crontab
#
########################################
sub dumpCrontab ($ $; $)
{
	my ($doc, $user, $server) = @_;

	my $crontab = $doc->createElement('crontab');

	# set attribute username for server-wide users
	if ($server eq 'server') {
		$crontab->setAttribute('user', $user);
	}

	addCrontab2Stream($crontab, $user);

	return $crontab;
}

sub dumpCustomButtons
{
	my ($doc, $level_id, $level) = @_;

	my $buttons_node = $doc->createElement('custom_buttons');

	my $query = dbDoQuery('SELECT * FROM custom_buttons WHERE level_id="' . $level_id. '" AND level="' . $level . '"');
	while (my $ref = $query->fetchrow_hashref) {
		delete $$ref{'id'};
		delete $$ref{'level_id'};
		delete $$ref{'level'};
		my $sn = createNodeFromHash($doc, 'button', $ref);
		my $path = $conf{PRODUCT_ROOT_D} . '/admin/htdocs/images/custom_buttons';
		my $path = $conf{'PRODUCT_ROOT_D'} . '/admin/htdocs/images/custom_buttons/' . $file;
		my $file = $$ref{'file'};
		addFile2Stream($sn, 'custom_button', "$path/$file", 'image/x-button') if ($file && -e "$path/$file");
		$buttons_node->appendChild($sn);
	}

	return $buttons_node;
}

sub dumpMailerQueries($)
{
	my ($doc) = @_;

	my $node = $doc->createElement('mailer_queries');
	my $query = dbDoQuery('SELECT * FROM mailer_queries');
	while (my $ref = $query->fetchrow_hashref) {
		my $sn = $doc->createElement('query');

		delete $ref->{id};

		while (my ($field, $value) = each (%$ref)) {
			$sn->setAttribute($field, a2u($value));
		}

		$node->appendChild($sn);
	}

	return $node;
}


#
# Save system ip addresses list.
#
sub dumpIPAddresses($)
{
	my ($doc) = @_;

	my $node = $doc->createElement('ip_addresses_list');

	my $query = dbDoQuery('SELECT * FROM IP_Addresses');
	while (my $hash = $query->fetchrow_hashref) {
		my $sn = createNodeFromHash($doc, 'ip_address', $hash);
		$node->appendChild($sn);
		if (my $sn2 = dumpActionLog($doc, $$hash{'id'}, 'ip_address')) {
			$sn->appendChild($sn2);
		}
	}

	return $node;
}


sub dumpCertificates
{
	my ($doc, $certs_hash) = @_;

	my $default_certificate_id = getMiscParam('default_certificate_id');

	$base_node = $doc->createElement('certificates_list');

	############################################################
	#	Get id of the certificate, which has been used by BU
	############################################################
	my  $query       = dbDoQuery('SELECT val FROM misc WHERE param="bu_cert_id"');
	my ($bu_cert_id) = $query->fetchrow_array;

	$query = dbDoQuery('SELECT * FROM certificates');

	while ( my $hash = $query->fetchrow_hashref) {
		if ($certs_hash && !$$certs_hash{$$hash{id}}) {
			next;
		}

		my $node = $doc->createElement('certificate');

		$node->setAttribute('id', $hash->{id});
		$node->setAttribute('default', 'true') if ($$hash{'id'} == $default_certificate_id);

		######################################
		#	Dump BU certificate distinctly.
		######################################
		if ($bu_cert_id eq $hash->{id}) {
			######################################################
			#	Thats BU cert, add him an appropriate attribute.
			#	It has neither name nor time_stamp.
			######################################################
			$node->setAttribute('bu_cert', 'true');

		} else {

			$node->setAttribute('name', a2u($hash->{name}));
			$node->setAttribute('time_stamp', $hash->{time_stamp});
		}
			
		if (exists($hash->{cert}) && $hash->{cert}) {
			my $sn = $doc->createElement('pub_key');
			addTextCertKey2Stream($sn, $hash->{cert});
			$node->appendChild($sn);
		}

		if (exists($hash->{pvt_key}) && $hash->{pvt_key}) {
			my $sn = $doc->createElement('pvt_key');
			addTextCertKey2Stream($sn, $hash->{pvt_key});
			$node->appendChild($sn);
		}

		if (exists($hash->{csr}) && $hash->{csr}) {
			my $sn = $doc->createElement('csr');
			addTextCertKey2Stream($sn, $hash->{csr});
			$node->appendChild($sn);
		}

		if (exists($hash->{ca_cert}) && $hash->{ca_cert}) {
			my $sn = $doc->createElement('ca_cert');
			addTextCertKey2Stream($sn, $hash->{ca_cert});
			$node->appendChild($sn);
		}

		$base_node->appendChild($node);
	}

	return $base_node;
}


#
# Save mail white list.
#
sub dumpMailWhiteList($)
{
	my ($doc) = @_;

	my $query = dbDoQuery("SELECT ip_address, ip_mask FROM smtp_poplocks WHERE lock_time IS NULL;");

	my $node = $doc->createElement('white_list');

	while (my ($ip, $mask) = $query->fetchrow_array) {
		my $sn = $doc->createElement('address');
		$sn->setAttribute('ip', $ip);
		$sn->setAttribute('mask', $mask);
		$node->appendChild($sn);
	}

	return $node;
}

sub dumpClients {
	my ($doc, $dumpingBy) = @_;
#
# Parse 'clients' table.
#

my $WHERE = '';
if ($sqlWhereForClients) {
	$WHERE = 'where ' . $sqlWhereForClients;
} elsif ($sqlWhereForDomains) {
	$WHERE = ', domains d where d.cl_id=c.id and (' . $sqlWhereForDomains . ')';
}


$query = dbDoQuery("SELECT DISTINCT c.*, a.type, a.password FROM clients c LEFT JOIN accounts a on a.id=c.account_id $WHERE;");
while ( my $ref = $query->fetchrow_hashref ) {
	my $id=$ref->{id};

	if ($ref->{login} eq '') {
		die_log "Unable to store client with id=%s (pname: '%s'): login is empty.\n", "$id", $ref->{pname};
	}

	if ($dumpingBy) {
		if (exists($dumpingBy->{'clients_ids'})) {
			delete($dumpingBy->{'clients_ids'}->{$id});
		} elsif (exists($dumpingBy->{'clients_logins'})) {
			delete($dumpingBy->{'clients_logins'}->{$ref->{login}});
		}
	}

# $node	- main, $sn - subnode
	my $node=createUser($doc, 'client', '1', $$ref{'login'}, $$ref{'password'},
						{	'PW_TYPE' => $$ref{'type'}, 
							'display_login' => $$ref{'display_login'},
							'login_text' => $$ref{'login_text'}
						});
	$node->setAttribute('id', $id);

#
# make address subnode
#	
	my $sn=createCard($doc,$ref);
	$node->appendChild( $sn ) if $sn;

	$node->setAttribute('status',$ref->{status} );
	$node->setAttribute('create_date',$ref->{cr_date} );

	### dump action log
	if (my $sn = dumpActionLog($doc, $id, 'client')) {
		$node->appendChild($sn);
	}

	if (my $sn = dumpCustomReports($doc, $id, $const{'IS_CLIENT'})) {
		$node->appendChild($sn);
	}
	if (my $sn = dumpCustomButtons($doc, $id, $const{'IS_CLIENT'})) {
		$node->appendChild($sn);
	}

	# dump client's hosting skeleton
	if ($id) {
		my $path = $conf{HTTPD_VHOSTS_D} . '/.skel/'. $id;
		if (-d $path) {
			my $sn = $doc->createElement('vhost_skeleton');
			addDir2Stream($sn, 0, '.', $path);
			$node->appendChild($sn);
		}
	}

	$clients{$id}=$node;
	$psa_node->appendChild($node); 
}


if ($dumpingBy) {
	my $clientsHashIsEmpty = 1; # it defines that all clients are dumped (if it = 1)

	my $clientsHash;
	my $fieldName;
	if (exists($dumpingBy->{'clients_logins'})) {
		$clientsHash = $dumpingBy->{'clients_logins'};
		$fieldName = 'login';
	} elsif (exists($dumpingBy->{'clients_ids'})) {
		$clientsHash = $dumpingBy->{'clients_ids'};
		$fieldName = 'id';
	}

	while (my ($key, $value) = each (%$clientsHash)) {
		next unless $key;
		$clientsHashIsEmpty = undef;
		print_stderr "Client with $fieldName '$key' from list is not exist in PSA.\n";
	}
	die_log "Dumping error: found in list not existing clients.\n" unless $clientsHashIsEmpty;
}


# Get param for 'client'
$curr_cl_id = -1;
$query= dbDoQuery('SELECT cl_id, param, val FROM cl_param ORDER BY cl_id');
while (my ($cl_id, $param, $value) = $query->fetchrow_array) {
	if ($cl_id != $curr_cl_id) {
		next unless exists $clients{$cl_id};
		$node = $doc->createElement('params');
		$clients{$cl_id}->appendChild($node);
		$curr_cl_id = $cl_id;
	}
#	$sn = $doc->createElement($perm_name);
#	$sn->setAttribute('value', $perm_value);
#	$node->appendChild($sn);

	$node->setAttribute($param, a2u($value));
}


# DUMPING Clients permissions
$curr_cl_id = -1;
$query= dbDoQuery('SELECT c.id, p.permission, p.value FROM Permissions p, clients c WHERE c.perm_id=p.id ORDER BY c.id');
while (my ($cl_id, $perm_name, $perm_value) = $query->fetchrow_array) {
	if ($cl_id != $curr_cl_id) {
		next unless exists $clients{$cl_id};
		$node = $doc->createElement('permissions');
		$clients{$cl_id}->appendChild($node);
		$curr_cl_id = $cl_id;
		### dump action log
		if (my $sn = dumpActionLog($doc, $cl_id, 'client_permissions')) {
			$node->appendChild($sn);
		}
	}
#	$sn = $doc->createElement($perm_name);
#	$sn->setAttribute('value', $perm_value);
#	$node->appendChild($sn);

	$node->setAttribute($perm_name, $perm_value);
}

# DUMPING Clients limits
$curr_cl_id = -1;
$query= dbDoQuery('SELECT c.id, l.limit_name, l.value FROM Limits l, clients c WHERE c.limits_id=l.id ORDER BY c.id');
while (my ($cl_id, $limit_name, $limit_value) = $query->fetchrow_array) {
	if ($cl_id != $curr_cl_id) {
		next unless exists $clients{$cl_id};
		$node = $doc->createElement('limits');
		$clients{$cl_id}->appendChild($node);
		$curr_cl_id = $cl_id;
		### dump action log
		if (my $sn = dumpActionLog($doc, $cl_id, 'client_limits')) {
			$node->appendChild($sn);
		}
	}
#	$sn = $doc->createElement($limit_name);
#	$sn->setAttribute('value', $limit_value);
#	$node->appendChild($sn);
	$node->setAttribute($limit_name, $limit_value);
}


# Dump traffic statistic
{
my $curr_cl_id = -1;
my $node = undef;
$query = dbDoQuery('SELECT * FROM ClientsTraffic ORDER BY cl_id');
while (my $hash = $query->fetchrow_hashref) {
	my $cl_id = $hash->{cl_id};
	delete($hash->{cl_id});
	next unless exists $clients{$cl_id};

	if ($curr_cl_id != $cl_id) {
		$node = $doc->createElement('traffic_statistic');
		$clients{$cl_id}->appendChild($node);
		$curr_cl_id = $cl_id;
	}

	my $sn = createNodeFromHash($doc, 'record', $hash);
	$node->appendChild($sn);
}
}

#DUMPING Clients logos
$curr_cl_id = -1;
$query= dbDoQuery('SELECT c.id, l.name, l.fake, l.url, l.showMode FROM Logos l, clients c WHERE c.logo_id=l.id ORDER BY c.id');
while (my ($cl_id, $file_name, $fake_file_name, $url, $showMode) = $query->fetchrow_array) {
	if ($cl_id == $curr_cl_id) {
		print_stderr("Client with id=$cl_id has more then one the logo screens.\n");
		next;
	}
	$curr_cl_id = $cl_id;

	next unless exists $clients{$cl_id};

	$sn = $doc->createElement('logo');
	$sn->setAttribute('file', a2u($file_name));
	$sn->setAttribute('url', $url) if $url;
	$sn->setAttribute('show_mode', a2u($showMode)) if $showMode;
	addLogo2Stream($sn, $fake_file_name) if $fake_file_name;
	$clients{$cl_id}->appendChild($sn);
}


#Dumping clients ip pool
{
my $node = undef;
$curr_cl_id = -1;
my $query = dbDoQuery('SELECT c.id, c.pool_id, r.component_id FROM clients c, Repository r WHERE c.pool_id=r.rep_id ORDER BY c.id');
while (my ($cl_id, $ip_pool_id, $ip_address_id) = $query->fetchrow_array) {
	if ($cl_id != $curr_cl_id) {
		next unless exists $clients{$cl_id};
		$node = $doc->createElement('ip_pool');
		$clients{$cl_id}->appendChild($node);
		$curr_cl_id = $cl_id;

		### dump action log
		if (my $sn = dumpActionLog($doc, $cl_id, 'client_ip_pool')) {
			$node->appendChild($sn);
		}
	}

	my $sn = $doc->createElement('ip_address');
	$sn->setAttribute('id', $ip_address_id);
	$node->appendChild($sn);
}
}

#Dumping clients site app pkgs pool
{
my $node = undef;
$curr_cl_id = -1;
my $query = dbDoQuery('SELECT c.id, r.component_id FROM clients c, Repository r WHERE c.sapp_pool_id=r.rep_id ORDER BY c.id');
while (my ($cl_id, $sapp_pkg_id) = $query->fetchrow_array) {
	if ($cl_id != $curr_cl_id) {
		next unless exists $clients{$cl_id};
		$node = $doc->createElement('site_app_packages_pool');
		$clients{$cl_id}->appendChild($node);
		$curr_cl_id = $cl_id;
	}

	my $sn = $doc->createElement('site_app_package');
	$sn->setAttribute('id', $sapp_pkg_id);
	$node->appendChild($sn);
}
}

} # dumpClients()


sub dumpSubdomains($ $ $)
{
	my ($doc, $dom_id, $dom_name) = @_;

	$node = $doc->createElement('subdomains');
	return $node unless $dom_id;

	my $query = dbDoQuery('SELECT sd.* FROM subdomains sd WHERE sd.dom_id=' . $dom_id);
	while (my $hash = $query->fetchrow_hashref()) {
		my $sys_user_id = $$hash{sys_user_id};
		delete($$hash{sys_user_id});
		my $sn = createNodeFromHash($doc, 'subdomain', $hash);

		if (my $sn2 = dumpActionLog($doc, $dom_id, 'subdomain')) {
			$sn->appendChild($sn2);
		}

		### retrive system user info for subdomain if the system user differ from hosting system user
		my $query2 = dbDoQuery('SELECT h.sys_user_id!=su.id as private_user, su.*, a.* FROM hosting h, sys_users su LEFT JOIN accounts a ON (a.id=su.account_id) WHERE h.dom_id=' . $dom_id . ' AND su.id=' . $sys_user_id);
		unless ($query2) {
			printf_stderr("Unable to fetch subdomain system user info from database.\n");
			return '';
		}

		my $ref = $query2->fetchrow_hashref;

		if ($$ref{'private_user'}) {
			my $su_node = createUser($doc, 'subdomain', '4', $$ref{'login'}, $$ref{'password'},
							{'real' => 'true', 'PW_TYPE' => $$ref{'type'}, 'shell' => a2u($$ref{'shell'}),
							 'quota' => $$ref{'quota'}, 'home' => a2u($$ref{'home'})});

			$sn->appendChild($su_node);
		}

		my $uid = $$ref{'private_user'} ? 0 : getpwnam $$ref{'login'};
		addSubDomain2Stream($sn, $uid, $dom_name, $$hash{'name'});
		my $sys_data_node = $doc->createElement('system_data');
		$sn->appendChild($sys_data_node);
		addSubDomain2Stream($sys_data_node, 0, $dom_name, $$hash{'name'});

		if (my $site_apps_node = dumpDomainSiteApps($doc, 'subdomain', $$hash{'id'})) {
			$sn->appendChild($site_apps_node);
		}

		$node->appendChild($sn);
	}

	return $node;
}


sub dumpDomainSiteApps
{
	my ($doc, $dom_type, $dom_id) = @_;

	my $query = dbDoQuery('SELECT * FROM SiteApps WHERE dom_type=\'' . $dom_type . '\' AND dom_id=\'' . $dom_id . '\'');

	unless ($query) {
		print_stderr("Unable to query site applications list for domain '$domain_name'\n");
		return undef;
	}

	my $node = undef;

	while (my $hash = $query->fetchrow_hashref()) {
		unless ($node) {
			$node = $doc->createElement('site_apps');
		}
		my $app_id = $$hash{'id'};
		my $params_id = $$hash{'params_id'};

		delete $$hash{'dom_id'};
		delete $$hash{'params_id'};

		my $sn = createNodeFromHash($doc, 'site_app', $hash);
		$node->appendChild($sn);

		# backup PARAMETERS
		{
			my $query2 = dbDoQuery('SELECT parameter, value FROM Parameters WHERE id=\'' . $params_id . '\'');
			unless ($query2) {
				print_stderr("Unable to fetch params for site application with id=$app_id\n");
				next;
			}
			my $params_node = $doc->createElement('params');
			$sn->appendChild($params_node);
			while (my ($name, $value) = $query2->fetchrow_array()) {
				my $sn2 = $doc->createElement('parameter');
				$sn2->setAttribute('name', a2u($name));
				$sn2->setAttribute('value', a2u($value));
				$params_node->appendChild($sn2);
			}
			$sn->appendChild($params_node);
		}

		# basckup RESOURCES
		{
			my $query2 = dbDoQuery('SELECT type, res_id FROM SiteAppResources WHERE app_id=\'' . $app_id . '\'');
			unless ($query2) {
				print_stderr("Unable to fetch resources info for site application with id=$app_id\n");
				next;
			}
			my $resources_node = $doc->createElement('resources');
			while (my ($type, $res_id) = $query2->fetchrow_array()) {
				my $sn2 = $doc->createElement('resource');
				$sn2->setAttribute('type', a2u($type));
				$sn2->setAttribute('res_id', a2u($res_id));
				$resources_node->appendChild($sn2);
			}
			$sn->appendChild($resources_node);
		}
	}

	return $node;
}


sub dumpDomains {
	my ($doc, $dumpingBy) = @_;
	my @dom_attr = ('name', 'displayName', 'cr_date', 'dns_zone', 'status', 'htype', 'dns_type');


my $WHERE = '';
if ($sqlWhereForClients) {
	$WHERE = ', clients c where d.cl_id=c.id and (' . $sqlWhereForClients . ')';
} elsif ($sqlWhereForDomains) {
	$WHERE = 'where ' . $sqlWhereForDomains;
}

$query = dbDoQuery("SELECT d.* FROM domains d $WHERE");

while ( my $ref = $query->fetchrow_hashref ) {
	my $id=$ref->{id};
	my $clientId=$ref->{cl_id};

	unless( exists $clients{$clientId} ) {
		#print_stderr("Found unref for id = $clientId\n");
		next;
	}

        ######################################################
        #   For server dump:
        #   Determine case when given domain is currently in
        #   dump/restore state.
        #   If it's so - skip it to mantain dump integrity.
        #   Domain dump skips this step
        ######################################################
        if (not exists $flags{SINGLE_DOMAIN_MODE} or not $flags{SINGLE_DOMAIN_MODE}) {
        
         my $sq = dbDoQuery("SELECT status FROM domains WHERE id='$id'");
         my ($domain_status) = $sq->fetchrow_array;

         if ($domain_status & 4) {
            ################################################################
            #   Domain was turned off due to the backup/restore procedure.
            #   We can't include it into server dump because it can be
            #   inconsistent currently.
            ################################################################
            my $sq = dbDoQuery("SELECT name FROM domains WHERE id='$id'");
            my ($domain_name) = $sq->fetchrow_array;

            addNotDumpedDomain($domain_name);
            next;
         }
        }
	
        if ($dumpingBy) {
		if (exists($dumpingBy->{'domains_ids'})) {
			delete($dumpingBy->{'domains_ids'}->{$id});
		} elsif (exists($dumpingBy->{'domains_names'})) {
			delete($dumpingBy->{'domains_names'}->{$ref->{name}});
		}
	}

	my $domain = $doc->createElement('domain');
	$domain->setAttribute('id', $id);

	foreach my $i (@dom_attr) {
		$domain->setAttribute($i, a2u($$ref{$i}));
	}

	my $sn = $doc->createElement('size');
	$sn->setAttribute( 'real',$ref->{real_size});
	$domain->appendChild($sn);

	if (my $sn = dumpActionLog($doc, $id, 'domain')) {
		$domain->appendChild($sn);
	}
	if (my $sn = dumpCustomReports($doc, $id, $const{'IS_DOMAIN_OWNER'})) {
		$domain->appendChild($sn);
	}
	if (my $sn = dumpCustomButtons($doc, $id, $const{'IS_DOMAIN_OWNER'})) {
		$domain->appendChild($sn);
	}

	$domains{$id}=$domain;

#
# If domain first time, we make new node 'sites' for this client.
#
	unless( exists $sites{$clientId}) {
		$sites{$clientId} = $doc->createElement('sites');
		$clients{$clientId}->appendChild($sites{$clientId});
	}
	$sites{$clientId}->appendChild($domain);
}	


if ($dumpingBy) {
	my $domainsHashIsEmpty = 1;

	my $domainsHash;
	my $fieldName;
	if (exists($dumpingBy->{'domains_names'})) {
		$domainsHash = $dumpingBy->{'domains_names'};
		$fieldName = 'name';
	} elsif (exists($dumpingBy->{'domains_ids'})) {
		$domainsHash = $dumpingBy->{'domains_ids'};
		$fieldName = 'id';
	}

	while (my ($key, $value) = each (%$domainsHash)) {
		next unless $key;
		$domainsHashIsEmpty = undef;
		print_stderr "Domain with $fieldName '$key' from list is not exist in PSA.\n";
	}
	die_log "Dumping error: found in list not existing domains.\n" unless $domainsHashIsEmpty;
}


# Dumping certificates repository for domain
{
my $curr_dom_id = -1;
my $node = undef;
$query = dbDoQuery('SELECT d.id, r.component_id FROM domains d, Repository r, certificates cert WHERE d.cert_rep_id=r.rep_id AND cert.id=r.component_id ORDER by d.id');
while ( my ($dom_id, $cert_id) = $query->fetchrow_array ) {
	next unless exists $domains{$dom_id};

	if ($curr_dom_id != $dom_id) {
		$node = $doc->createElement('certificates_list');
		$domains{$dom_id}->appendChild($node);
		$curr_dom_id = $dom_id;
	}

	my $sn = $doc->createElement('certificate');
	$sn->setAttribute('id', $cert_id);
	$node->appendChild($sn);
}
}

# Dumping limits for domains
my $curr_dom_id = -1;
my $limits_node = undef;
$query = dbDoQuery('SELECT d.id, l.limit_name, l.value FROM domains d, Limits l WHERE d.limits_id=l.id ORDER by d.id');
while ( my ($dom_id, $name, $value) = $query->fetchrow_array ) {
	next unless exists $domains{$dom_id};

	if ($curr_dom_id != $dom_id) {
		$limits_node = $doc->createElement('limits');
		$domains{$dom_id}->appendChild($limits_node);
		$curr_dom_id = $dom_id;
		if (my $sn = dumpActionLog($doc, $dom_id, 'domain_limits')) {
			$limits_node->appendChild($sn);
		}
	}

	$limits_node->setAttribute($name, $value);
}


# Dumping Parameters for domains
my $curr_dom_id = -1;
my $params_node = undef;
$query = dbDoQuery('SELECT d.id, p.parameter, p.value FROM domains d, Parameters p WHERE d.params_id=p.id ORDER BY d.id');
while ( my ($dom_id, $name, $value) = $query->fetchrow_array ) {
	next unless exists $domains{$dom_id};

	if ($curr_dom_id != $dom_id) {
		$params_node = $doc->createElement('ext_params');
		$domains{$dom_id}->appendChild($params_node);
		$curr_dom_id = $dom_id;
	}

	$params_node->setAttribute($name, a2u($value));
}


if ($backup_filter{'HOSTING'} || $backup_filter{'MAIL'} || $backup_filter{'DATABASES'}) {
	# Get disk usage counting parameters
	$query = dbDoQuery('SELECT * FROM disk_usage');
	while ( my $ref = $query->fetchrow_hashref ) {
		my $dom_id = $ref->{dom_id};
		next unless exists $domains{$dom_id};
		delete $ref->{dom_id};

		my $du = $doc->createElement('disk_usage');
		while (my ($field, $value) = each (%$ref)) {
			$du->setAttribute($field, $value);
		}

		$domains{$dom_id}->appendChild($du);
	}
}


# Dumping domain services
{
my $curr_dom_id = -1;
my $node = undef;
my %dsMailList = ();
my %domainServices = ();

$query = dbDoQuery('SELECT * FROM DomainServices ORDER BY dom_id');
while (my $hash = $query->fetchrow_hashref) {
	my $dom_id = $$hash{dom_id};
	delete($$hash{dom_id});

	next if (
		!$backup_filter{'HOSTING'} && ($hash->{type} eq 'tomcat') ||
		!$backup_filter{'MAIL'} && ($hash->{type} eq 'mail') ||
		!$backup_filter{'MAILLISTS'} && ($hash->{type} eq 'maillists')
	);

	if ($curr_dom_id != $dom_id) {
		unless (exists $domains{$dom_id}) {
			$node = undef;
			next;
		}
		$node = $doc->createElement('domain_services');
		$domains{$dom_id}->appendChild($node);
		$domainServices{$dom_id} = $node;
		$curr_dom_id = $dom_id;
	} else {
		next unless $node;
	}

	my $ds_params_id = $$hash{'parameters_id'};
	delete($$hash{'parameters_id'});

	my $sn = createNodeFromHash($doc, 'domain_service', $hash);
	$node->appendChild($sn);

	if ($ds_params_id) {
		my $query = dbDoQuery('SELECT * FROM Parameters WHERE id=' . dbQuote($ds_params_id));
		my $sn2 = $doc->createElement('params');
		$sn->appendChild($sn2);
		while (my $href = $query->fetchrow_hashref()) {
			$sn2->setAttribute($$href{'parameter'}, a2u($$href{'value'}));
		}
	}

	if ($hash->{type} eq 'tomcat') {
		next if (($flags{BACKUP_TYPE} ne 'FULL'));
		# Dumping domain web applications
		my $query = dbDoQuery('SELECT * FROM WebApps wa WHERE wa.domain_service_id=' . $$hash{id});
		while (my $hash = $query->fetchrow_hashref) {
			delete($hash->{domain_service_id});
			my $webAppNode = createNodeFromHash($doc, 'web_app', $hash);
			$sn->appendChild($webAppNode);

			###############################
			#	Web application file name
			###############################
			my $filename = $$hash{name} . '.war';
			my $path = $conf{CATALINA_HOME} . '/psa-wars/' . $domains{$dom_id}->getAttribute('name') . '/' . $filename;
			addFile2Stream($webAppNode, $$hash{name}, $path, 'application/x-war-webapp', 0);
		}
	} elsif ($hash->{type} eq 'mail') {
		next unless ($backup_filter{'MAIL'});

	} elsif ($hash->{type} eq 'maillists') {
		next unless ($backup_filter{'MAILLISTS'});

	    $dsMailList{$dom_id} = $sn;
	}
} # end while



########################################################################
### dump maillists
### --------------
### such strange dumping method - because DsomainService maillists 
### can be absent but maillists can be exists at the same time
########################################################################
if ($backup_filter{'MAILLISTS'}) {
	$curr_dom_id = -1;
	my $query = dbDoQuery('SELECT id, dom_id, name, status FROM MailLists ORDER BY dom_id');
	while (my ($id, $dom_id, $ml_name, $ml_status) = $query->fetchrow_array()) {
	    next unless $domains{$dom_id};

	    my $sn = undef;
	    if (exists($dsMailList{$dom_id})) {
		$sn = $dsMailList{$dom_id};
	    } else{
		my $node = undef;
		if (exists($domainServices{$dom_id})) {
		    $node = $domainServices{$dom_id};
		} else {
	    	    $node = $doc->createElement('domain_services');
		    $domains{$dom_id}->appendChild($node);
		}
		$sn = $doc->createElement('domain_service');
		$sn->setAttribute('id', $id);
		$sn->setAttribute('type', 'maillists');
		$sn->setAttribute('status', '1');
		$node->appendChild($sn);
	    }
	    $sn->appendChild(dumpMailList($doc, $ml_name, $ml_status));
	}
}

} ### end of dumping domain services




# Dump domain traffic
if ($backup_filter{'HOSTING'}) {
my $curr_dom_id = -1;
my $node = undef;
$query = dbDoQuery('SELECT * FROM DomainsTraffic ORDER BY dom_id');
while (my $hash = $query->fetchrow_hashref) {
	my $dom_id = $hash->{dom_id};
	delete($hash->{dom_id});
	next unless exists $domains{$dom_id};

	if ($curr_dom_id != $dom_id) {
		$node = $doc->createElement('traffic_statistic');
		$domains{$dom_id}->appendChild($node);
		$curr_dom_id = $dom_id;
	}

	my $sn = createNodeFromHash($doc, 'record', $hash);
	$node->appendChild($sn);
}
}


# Store mysq databases which managed by PSA
if ($backup_filter{'DATABASES'}) {
	my $node = undef;
	my %existsDatabases = ();
	$query = dbDoQuery("show databases");

	while (my ($name) = $query->fetchrow_array) {
		$existsDatabases{$name} = 1;
	}

	$query = dbDoQuery("SELECT dom_id, id, name, type FROM data_bases");

	while ( my ($id, $db_id, $name, $type) = $query->fetchrow_array ) {
		unless (exists($domains{$id})) {
#			print_stderr("Can't store database '$name': found unreferenced record for domains id = $id\n");
			next;
		}

		if ($type eq 'mysql') {
			if (!exists($existsDatabases{$name})) {
				print_stderr("Database '$name' backup is skipped: database is not exist.\n");
				next;
			}
		} elsif ($type eq 'postgresql') {
			my $res = checkPostgresDBExisting($name);
			if ($res < 0) {
				printf_stderr("Database %s backup is skipped: error during checking database existing.\n", $name);
				next;
			} elsif ($res == 0) {
				printf_stderr("Database %s backup is skipped: database is not exist.\n", $name);
				next;
			}
		}

		$node = createDatabaseNode($doc, $name, $type);
		$dbs{$db_id} = $node;
		$domains{$id}->appendChild($node);

		#set database id for SiteApp related resources checking
		$node->setAttribute('id', $db_id);
	}

my $cur_db_id = undef;
$query = dbDoQuery("SELECT dbu.id, dbu.db_id, dbu.login, a.type, a.password FROM db_users dbu LEFT JOIN accounts a on dbu.account_id=a.id");
while ( my ( $id, $db_id, $login, $password_type, $password) = $query->fetchrow_array ) {
	unless (exists $dbs{$db_id}) {
#		print_stderr("Database user '$login' is not stored because her database is not exists.\n");
		next;
	}
	my $db_node = $dbs{$db_id};

#	print "Dumping data base user ($login)\n";

	$node=createUser($doc, 'db_users', '5', $login, a2u($password), {'id' => $id, 'PW_TYPE' => $password_type} );
	my $db_type = $dbs{$db_id}->getAttribute('type');
	if ($db_type eq 'mysql') {
		my $query2=dbDoQuery("SELECT $MYSQL_USER_PRIVILEGES FROM mysql.user WHERE User=". dbQuote($login));
		while (my $hash = $query2->fetchrow_hashref) {
			delete($hash->{'User'});
			$node->appendChild(createNodeFromHash($doc, 'access_rights', $hash));
		}
	} elsif ($db_type eq 'postgresql') {
	}

	$db_node->appendChild($node);
}
}



if ($backup_filter{'HOSTING'}) {
	my @hu_services=('ssi', 'php', 'cgi', 'perl', 'asp', 'ssl', 'webstat', 'python', 'coldfusion');
	#$query = dbDoQuery('SELECT h.*, su.*, a.password, a.type, l.value FROM hosting h, domains d LEFT JOIN Limits l on (d.limits_id=l.id and l.limit_name="max_traffic"), sys_users su LEFT JOIN accounts a on (su.account_id=a.id) WHERE  d.id=h.dom_id and h.sys_user_id=su.id');
	$query = dbDoQuery('SELECT h.*, su.*, a.password, a.type FROM hosting h, domains d, sys_users su LEFT JOIN accounts a on (su.account_id=a.id) WHERE  d.id=h.dom_id and h.sys_user_id=su.id');
	while ( my $ref = $query->fetchrow_hashref ) {
		unless( exists $domains{$ref->{dom_id}} ) {
#			print_stderr("Found unref for domains id = $ref->{dom_id}\n");
			next;
		}

		my $domainNode = $domains{$ref->{dom_id}};
		my $domainName = $domainNode->getAttribute('name');

		my $node=createUser($doc, 'hosting', '3', $ref->{login}, $ref->{password},
							{'real' => 'true', 'PW_TYPE' => $ref->{type}, 'shell' => a2u($ref->{shell}),
							 'quota' => $$ref{quota}, 'home' => a2u($$ref{home})});

		if (($flags{BACKUP_TYPE} eq 'FULL') && ($domains_backup_dirs{$domainNode->getAttribute('htype')})) {
			my $uid = getpwnam($$ref{'login'});
			addDomain2Stream($node, $uid, $domainName);
			my $sys_data_node = $doc->createElement('system_data');
			$node->appendChild($sys_data_node);
			addDomain2Stream($sys_data_node, 0, $domainName);
			# dump crontab for system user
			$node->appendChild(dumpCrontab($doc, $ref->{login}));
		}

		$node->setAttribute('at_domains', $$ref{'at_domains'});

		my $sn = $doc->createElement('ip_address');
		$sn->setAttribute('id', $ref->{ip_address_id});
		$node->appendChild($sn);

		my $sn = createServices($doc,$ref,@hu_services);
		domainDir($ref->{dom_id});

		if ($ref->{'fp_adm'}) {
			my $sn2 = $doc->createElement('fp');
			$sn2->setAttribute('fp_auth',$ref->{fp_enable});
			$sn2->setAttribute('ssl', (($ref->{fp_ssl} eq 'true') ? 'true' : 'false'));
			$sn2->setAttribute('not_ssl', (($ref->{fp} eq 'true') ? 'true' : 'false'));

			$sn2->appendChild(createUser($doc, 'fp', '7', $ref->{'fp_adm'}, $ref->{'fp_pass'},
								{'PW_TYPE' => 'plain', 'ADMIN' => 'true'}));

			if (($ref->{fp} eq 'true') || ($ref->{fp_ssl} eq 'true')) {
				### store additional fp users only if fp service is turned on
				my %pw = readFPpw(domainDir($ref->{dom_id}));
				delete($pw{$ref->{'fp_adm'}});

				foreach my $i (keys %pw) {
					$sn2->appendChild(createUser($doc, 'fp', '7', $i, $pw{$i},
						{"PW_TYPE" => "des"} ));
				}
			}
			$sn->appendChild($sn2);
		}
		$node->appendChild($sn);


		### dump actions
		if (my $sn = dumpActionLog($doc, $$ref{dom_id}, 'phys_hosting')) {
			$node->appendChild($sn);
		}

#	 traffic => traffic.limit, real_traffic => traffic.real
		my $sn = $doc->createElement('traffic');
		$sn->setAttribute('real',$ref->{real_traffic});
#		$sn->setAttribute('limit', (!$ref->{value} || ($ref->{value} < 0)) ? 'unlimited' : $ref->{value});

		$node->appendChild($sn);

		if (exists($ref->{'rootchain'})) {
			$sn = $doc->createElement('rootchain_certificate');
			$sn->setAttribute('status', $ref->{'rootchain'});

			addDomainRootChainCert2Stream($sn, $domains{$ref->{dom_id}}->getAttribute('name')) if
				(($ref->{'rootchain'} eq 'true') && ($flags{BACKUP_TYPE} eq 'CONF'));

			$node->appendChild($sn);
		}

		$domains{$$ref{dom_id}}->appendChild($node);
		$hosting{$$ref{dom_id}} = $node;
		if (my $subdomains_node = dumpSubdomains($doc, $$ref{dom_id}, $domainName)) {
			$node->appendChild($subdomains_node);
		}

		if (my $site_apps_node = dumpDomainSiteApps($doc, 'domain', $$ref{dom_id})) {
			$node->appendChild($site_apps_node);
		}
	}


	my @wu_services=('ssi', 'php', 'cgi', 'perl', 'asp', 'python');
	# SELECT id, dom_id, username,  @wu_services FROM web_users;
	$query = dbDoQuery('SELECT wu.*, su.*, a.password, a.type FROM web_users wu, sys_users su LEFT JOIN accounts a on (su.account_id=a.id) WHERE wu.sys_user_id=su.id');
	while ( my $ref = $query->fetchrow_hashref ) {
		unless( exists $hosting{$$ref{dom_id}} ) {
#			print_stderr("Found unref for domains id = $ref->{dom_id}\n");
			next;
		}

		my $shell = undef;
		if (exists($ref->{shell}) && $ref->{shell}) {
			$shell = $ref->{shell};
		} else {
			my @pw_info = getpwnam($ref->{login});
			$shell = $pw_info[8];
		}

		my $node=createUser($doc, 'web_users', '3', $$ref{'login'}, $$ref{'password'},
							{'real' => 'true',
							 'PW_TYPE' => $$ref{'password'} ? $$ref{'type'} : 'empty',
							 'shell' => a2u($shell),
							 'quota' => $ref->{quota}, 'home' => a2u($ref->{home})});

		my $domainName = $domains{$$ref{dom_id}}->getAttribute('name');

#		my $uid = getpwnam $$ref{'login'};
		addDir2Stream($node, 0, $$ref{'login'}, $conf{HTTPD_VHOSTS_D} . "/$domainName/web_users");

		$node->appendChild(createServices($doc,$ref,@wu_services));

		# dump crontab for webuser
		$node->appendChild(dumpCrontab($doc, $ref->{login}));

#		$domains{$ref->{dom_id}}->appendChild($node);
		$hosting{$$ref{dom_id}}->appendChild($node);

		if (my $sn = dumpActionLog($doc, $$ref{'id'}, 'webuser')) {
			$node->appendChild($sn);
		}
	}

	### dump log_rotation prefs
	$query = dbDoQuery('SELECT dp.dom_id, lr.* FROM dom_param dp, log_rotation lr WHERE dp.param="logrotation_id" AND dp.val=lr.id');
	while (my $ref = $query->fetchrow_hashref) {
		my $dom_id = $ref->{dom_id};
		delete $ref->{dom_id};
		delete $ref->{id};
		next unless exists $hosting{$dom_id};

		my $node = $doc->createElement('log_rotation');
		while (my ($field, $value) = each (%$ref)) {
			$node->setAttribute($field, a2u($value));
		}

		$domains{$dom_id}->appendChild($node);
	}
}

$query = dbDoQuery('SELECT dom_id, param, val FROM dom_param WHERE NOT (param LIKE "%_id") ORDER BY dom_id');
my $node = undef;
my $curr_dom_id = -1;

while (my ($dom_id, $param,  $value) = $query->fetchrow_array) {
	if ($curr_dom_id != $dom_id) {
		next unless exists $domains{$dom_id};
		$node = $doc->createElement('params');
		$domains{$dom_id}->appendChild($node);
		$curr_dom_id = $dom_id;
	}

	$node->setAttribute($param, $value);
}



# Get forwarding for 'domains'
if ($backup_filter{'HOSTING'}) {
	$query = dbDoQuery("SELECT dom_id, redirect, ip_address_id FROM forwarding ORDER by dom_id;");
	while ( my ($id, $redirect, $ip_address_id) = $query->fetchrow_array ) {
		unless( exists $domains{$id} ) {
#			print_stderr("Found unref for domains id = $id in 'forwarding' db\n");
			next;
		}
		my $node = $doc->createElement('forwarding');
		$node->setAttribute('redirect',$redirect);
		$node->setAttribute('ip_address_id',$ip_address_id);
		$domains{$id}->appendChild($node);
	}
}

if ($backup_filter{'MAIL'}) {
	my @mail_services=('postbox','mail_group','redirect','autoresponder');
	$query = dbDoQuery('SELECT m.*, a.type, a.password FROM mail m LEFT JOIN accounts a ON (m.account_id=a.id)');
	while ( my $ref = $query->fetchrow_hashref() ) {

		unless( exists $domains{$ref->{dom_id}} ) {
#			print_stderr("Found unref for domains id = $ref->{dom_id}\n");
			next;
		}
		my $id = $ref->{id};

		my $node=createUser($doc, 'mail', '4', $$ref{mail_name}, $$ref{password},
							{'PW_TYPE' => ($$ref{type} ? $$ref{type} : 'empty'), 'cp_access' => $ref->{cp_access}});
		$mails{$id}=$node;
		$domains{$$ref{dom_id}}->appendChild($node);

		my $sn=createServices($doc,$ref,@mail_services);
		my $sn2=$doc->createElement('forwarding');
		$sn2->setAttribute('redirect', $ref->{redirect});
		$sn2->appendChild(createEmail($doc,$ref->{redir_addr})) if $ref->{redir_addr};
		$sn->appendChild($sn2);
		my $sn2=$doc->createElement('forwarding');
		$sn2->setAttribute('mailgroup', $ref->{mail_group});
		$forwards{$id}=$sn2;
		$sn->appendChild($sn2);
		$node->appendChild($sn);

		### save mn_param
		## drweb preferences is stored in params
		my $sn=$doc->createElement('params');
		my $has_params = undef;
		unless( $ref->{mbox_quota} ==  -1 ) {
			$sn->setAttribute('mbox_quota', $ref->{mbox_quota});
			$has_params = 1;
		}
		my $sub_query = dbDoQuery('SELECT param, val FROM mn_param WHERE mn_id=' . $id);
		while (my ($param, $value) = $sub_query->fetchrow_array) {
			$has_params = 1;
			my $sn2 = $doc->createElement('parameter');
			$sn2->setAttribute('name', a2u($param));
			$sn2->setAttribute('value', a2u($value));
			$sn->appendChild($sn2);
		}

		if ($has_params) {
			$node->appendChild($sn);
		}
		### end of mn_param saving

		my $domainName = $domains{$$ref{dom_id}}->getAttribute('name');
		if ($flags{BACKUP_TYPE}	eq 'FULL') {
			addMailbox2Stream($node,"$domainName/".$ref->{mail_name});
		}
	}

	my $curr_mn_id = -1;
	my $node = undef;
	$query = dbDoQuery('SELECT mn_id, alias FROM mail_aliases ORDER BY mn_id');
	while (my ($mn_id, $alias) = $query->fetchrow_array) {
		if ($curr_mn_id != $mn_id) {
			next unless $mails{$mn_id};
			$node = $doc->createElement('aliases');
			$mails{$mn_id}->appendChild($node);
			$curr_mn_id = $mn_id;
		}
		my $sn = $doc->createElement('alias');
		$sn->setAttribute('name', a2u($alias));
		$node->appendChild($sn);
	}


	$query = dbDoQuery("SELECT mn_id, address FROM mail_redir;");
	while (my ($id, $email) = $query->fetchrow_array) {
		unless( exists $forwards{$id} ) {
#			print_stderr("Found unref for mail id = $id\n");
			next;
		}
		$forwards{$id}->appendChild(createEmail($doc,$email));
	}

	# dump attachment repositories
	foreach my $mail (values %mails) 
	{
		# get list or files in repositary for specified mail
		my $path = &getPathToAttachRepos ($mail);
		my @files_at_repository=();
		if (-d $path) {
			unless (open ( FD , "ls $path |" )) {
				print_stderr("Unable to retrieve the list of files in repository");
			} else {
				while (<FD>) {
					my $fname=$_;
					chomp $fname;
					print_stderr ("Processing file $fname\n") if ($debug>3);
					next unless ($fname);
					print_stderr("Unable to find file $path/$fname\n") unless (-e "$path/$fname");
					push ( @files_at_repository, $fname);
				}
				close FD or 
					warn ("Some problems occured during an attempt to close the pipe while getting files list from the repository");
					print_stderr ("Files in repository dir'$path': ", join (", ", @files_at_repository), "\n") if ($debug>2);
			}
		}

		# 
		my $node = $doc->createElement('repository');
		$mail->appendChild($node);
	
		foreach my $filename (@files_at_repository)
		{
			my $file = $doc->createElement('file');
			$file->setAttribute('name', a2u($filename));
			$node->appendChild($file);
			addAttach2Stream($file, $filename);
		}
	}

	my %mails_resp;
	$query = dbDoQuery("SELECT * FROM mail_resp;");
	while ( my $ref = $query->fetchrow_hashref ) {

		unless( exists $mails{$ref->{mn_id}} ) {
#			print_stderr("Found unreferenced record for mail id = $ref->{mn_id}\n");
			next;
		}
		my $id = $ref->{id};
	
		my $node = $doc->createElement('autoresponder');
		$node->setAttribute('name',$ref->{resp_name});
		$node->setAttribute('turn',$ref->{resp_on});
#		$node->setAttribute(,$ref->{});

		my $sn = $doc->createElement('params');
		$sn->setAttribute('ans_freq',$ref->{ans_freq});
		$sn->setAttribute('mem_limit',$ref->{mem_limit});
		$sn->setAttribute('reply_to',$ref->{reply_to});
		$node->appendChild($sn);

		my $sn = $doc->createElement('keystr');
		$sn->setAttribute('where',$ref->{key_where});
		$sn->appendChild($doc->createTextNode(a2u($ref->{keystr})));
		$node->appendChild($sn);
	
		my $sn = $doc->createElement('message');
		$sn->setAttribute('subject',a2u($ref->{subject}));
		$sn->appendChild($doc->createTextNode(a2u($ref->{text})));
		$node->appendChild($sn);

		$mails_resp{$id}=$node;

		$mails{$ref->{mn_id}}->appendChild($node);
	}

	$query = dbDoQuery("SELECT rn_id, address FROM resp_forward;");
	while ( my ($id, $email) = $query->fetchrow_array ) {
		unless( exists $mails_resp{$id} ) {
#			print_stderr("Found unreferenced record for mail_resp id = $id\n");
			next;
		}
		my $node = $doc->createElement('forwarding');
		if ($email) {
			$node->setAttribute('autoresponder', 'true');
			$node->appendChild(createEmail($doc,$email));
		} else {
			$node->setAttribute('autoresponder', 'false');
		}
		$mails_resp{$id}->appendChild($node);
	}

	$query = dbDoQuery('SELECT rn_id, filename FROM resp_attach');
	while ( my ($id, $file) = $query->fetchrow_array ) {
		unless( exists $mails_resp{$id} ) {
#			print_stderr("Found unreferenced record for mail_resp id = $id\n");
			next;
		}
		my $node = $doc->createElement('attach');
		$node->setAttribute('file', a2u($file));
		$mails_resp{$id}->appendChild($node);
	}
}

sub dumpSpamAssassinConf
{
	my ($doc, $mailname, $hashref) = @_;

	my $node = createNodeFromHash($doc, 'spamassassin', $hashref);
	my $query2 = dbDoQuery('SELECT * FROM sa_list WHERE mailname=' . dbQuote($mailname));
	unless ($query2) {
		print_stderr("Unable to retrive sa_list database table.\n");
	} else {
		while (my $hashref2 = $query2->fetchrow_hashref()) {
			delete $$hashref2{'mailname'};
			my $sn = createNodeFromHash($doc, 'record', $hashref2);
			$node->appendChild($sn);
		}
	}

	return $node;
}

####################################
# DUMP SPAMASSASSIN PREFERENCES
####################################
if ($backup_filter{'MAIL'}) {
	### dump sa_conf, dump only if accordingly mailname is dumped
	my $query = dbDoQuery('SELECT m.id, sa.* FROM sa_conf sa, mail m, domains d WHERE d.id=m.dom_id AND CONCAT(m.mail_name, "@", d.name)=sa.mailname ORDER BY sa.mailname');
	unless ($query) {
		print_stderr("Unable to retrive sa_conf database table.\n");
	} else {
		while (my $hashref = $query->fetchrow_hashref()) {
			my $mn_id = $$hashref{'id'};
			next unless exists($mails{$mn_id});
			delete $$hashref{'id'};

			my $mailname = $$hashref{'mailname'};
			delete $$hashref{'mailname'};

			my $node = dumpSpamAssassinConf($doc, $mailname, $hashref);
			$mails{$mn_id}->appendChild($node);
		}
	}
}


if ($backup_filter{'HOSTING'}) {
	my %ppds=();
	$query = dbDoQuery("SELECT * FROM protected_dirs;");
	while ( my $ref = $query->fetchrow_hashref ) {
		unless( exists $hosting{$ref->{dom_id}} ) {
#			print_stderr("Found unreferenced record for domains id = $ref->{dom_id}\n");
			next;
		}

		my $id = $ref->{id};
		my $node = $doc->createElement('protected_dir');
		$node->setAttribute('path', $ref->{path} );

		my $sn = $doc->createElement('realm');
		$sn->appendChild($doc->createTextNode(a2u($ref->{realm})));
		$node->appendChild($sn);

		if( $sn=createServices($doc, $ref, 'ssl', 'non_ssl') ) {
			$node->appendChild($sn);
		}
		$ppds{$id}=$node;

		$hosting{$ref->{dom_id}}->appendChild($node);
	}

	$query = dbDoQuery("SELECT pd.pd_id, pd.login, a.type, a.password FROM pd_users pd, accounts a WHERE pd.account_id=a.id");
	while ( my ($id, $login, $pw_type, $password) = $query->fetchrow_array ) {
		unless( exists $ppds{$id} ) {
#			print_stderr("Found unreferenced record for protected_dirs id = $id\n");
			next;
		}
		my $node=createUser($doc, 'pd_users', '4', $login, $password, {'PW_TYPE' => $pw_type} );
		$ppds{$id}->appendChild($node);
	}
}

$query = dbDoQuery('SELECT d.id, r.component_id FROM domains d, Repository r WHERE d.cert_rep_id=r.rep_id');
while ( my ($dom_id, $cert_id) = $query->fetchrow_array ) {
	unless( exists $domains{$dom_id} ) {
#		print_stderr("Found unreferenced record for domains id = $id\n");
		next;
	}
	my $node = $doc->createElement('certificate');
	$node->setAttribute('id', $cert_id);
	$domains{$dom_id}->appendChild($node);
}


if ($backup_filter{'DOMAIN_OWNER'}) {
$query = dbDoQuery('SELECT dlu.*, a.type, a.password FROM dom_level_usrs dlu LEFT JOIN accounts a on a.id=dlu.account_id');
	while (my $ref = $query->fetchrow_hashref) {
		my $dom_id = $$ref{'dom_id'};

		unless( exists $domains{$dom_id} ) {
#			print_stderr("Found unreferenced record for domains id = $id\n");
			next;
		}
		my $name=domainName($dom_id);

		my $node=createUser($doc, 'domain_level', '5', $name, $$ref{password}, {'PW_TYPE' => $$ref{type}});
		$node->setAttribute('status', $ref->{state});

		### dump actions
		if (my $sn = dumpActionLog($doc, $dom_id, 'dl_user')) {
			$node->appendChild($sn);
		}

		if (my $query2 = dbDoQuery('SELECT * FROM Cards crd WHERE crd.id=' . dbQuote($$ref{card_id}))) {
			if (my $ref2 = $query2->fetchrow_hashref) {
				%new_card_fields = ( 
					'personalName' => 'pname', 'companyName' => 'cname', 'phone' => 'phone', 'fax' => 'fax',
					'email' => 'email', 'address' => 'address', 'city' => 'city', 'state' => 'state',
					'zip' => 'zip', 'country' => 'country'
				);

				if (my $sn = createCard($doc, $ref2, \%new_card_fields)) {
					$node->appendChild($sn);
				}
			}
		}

		{ # backup domain level user permissions
			my $sn = $doc->createElement('permissions');
			my $query2 = dbDoQuery('SELECT p.permission, p.value FROM Permissions p, dom_level_usrs du WHERE du.perm_id=p.id AND du.dom_id=\'' . $dom_id . '\'');
			while (my ($name, $value) = $query2->fetchrow_array()) {
				$sn->setAttribute($name, ($value eq 'true') ? 'true' : 'false');
			}
			$node->appendChild($sn);
		}

		$domains{$dom_id}->appendChild($node);
	}
}


if ($backup_filter{'HOSTING'}) {
	my @anon_ftp_params=('status', 'incoming', 'max_conn', 'bandwidth',
						'incoming', 'incoming_readable', 'incoming_subdirs',
						'display_login', 'login_text');
	$query = dbDoQuery("SELECT * FROM anon_ftp;");
	while ( my $ref = $query->fetchrow_hashref ) {
		unless( exists $domains{$ref->{dom_id}} ) {
#			print_stderr("Found unreferenced record for domains id = $ref->{dom_id}\n");
			next;
		}

		my $node = $doc->createElement('anon_ftp');
		foreach my $param (@anon_ftp_params) {
			$node->setAttribute($param, a2u($$ref{$param}));
		}

		my $sn = $doc->createElement('size');
		$sn->setAttribute('limit',$ref->{'quota'});
		$node->appendChild($sn);

#		$domains{$ref->{dom_id}}->appendChild($node);
		$hosting{$ref->{dom_id}}->appendChild($node);
	}
}

	dumpDNS($doc, );
} # dumpDomains()


sub dumpMailList
{
	my ($doc, $name, $status) = @_;

	my $node = $doc->createElement('mail_list');
	$node->setAttribute('name', $name);
	$node->setAttribute('status', $status);

	my $sn = undef;
	my $path = undef;

	$path = $conf{MAILMAN_ROOT_D} . "/lists/$name";
	if (-d $path) {
		$sn = $doc->createElement('config_dir');
		addDir2Stream($sn, 0, '.', $path);
		$node->appendChild($sn);
	}

	$path = $conf{MAILMAN_ROOT_D} . "/archives/private/$name";
	if (-d $path) {
		$sn = $doc->createElement('archive_private_dir');
		addDir2Stream($sn, 0, '.', $path);
		$node->appendChild($sn);
	}

	$path = $conf{MAILMAN_ROOT_D} . "/archives/private/$name.mbox";
	if (-d $path) {
		$sn = $doc->createElement('archive_private_mbox_dir');
		addDir2Stream($sn, 0, '.', $path);
		$node->appendChild($sn);
	}

	return $node;
}


sub dumpSystemDB
{
	my ($doc, $node) = @_;

	{ # dumping not psa databases
		my %excludedDBList = ();
		my $query = dbDoQuery('SELECT name FROM data_bases');
		return '' unless $query;
		while (my ($name) = $query->fetchrow_array) {
			$excludedDBList{$name} = 1;
		}
		$excludedDBList{$const{PSA_DB_NAME}} = 1;
		$excludedDBList{$const{WEBMAIL_DB_NAME}} = 1;
		$excludedDBList{'mysql'} = 1;
		$excludedDBList{'test\_%'} = 1;

		$query = dbDoQuery('show databases');
		return '' unless $query;

		while (my $hash = $query->fetchrow_hashref) {
			my $name = $hash->{'Database'};
			next if ($name eq '');

			unless (exists($excludedDBList{$name})) {
				$node->appendChild(createDatabaseNode($doc, $name, 'mysql'));
			}
		}
	}

	{ # dumping not psa databse users
		my %excludedUsersList = ();
		my $query = dbDoQuery('SELECT login FROM db_users');
		return '' unless $query;
		while (my ($login) = $query->fetchrow_array) {
			$excludedUsersList{$login} = 1;
		}
		$excludedUsersList{$const{PSA_ADMIN_NAME}} = 1;
		$excludedUsersList{$const{WEBMAIL_ADMIN_NAME}} = 1;
		
		$query = dbDoQuery("SELECT $MYSQL_USER_PRIVILEGES FROM mysql.user GROUP BY User");
		return '' unless $query;
		while (my $hash = $query->fetchrow_hashref) {
			my $login = $hash->{'User'};
			next if ($login eq '');
			delete($hash->{'User'});

			unless (exists($excludedUsersList{$login})) {
				$sn = createUser($doc, 'db_users', 5, $login, $hash->{'Password'}, {'PW_TYPE'=>'crypt'});
				$sn->appendChild(createNodeFromHash($doc, 'access_rights', $hash));

				$sn->setAttribute('not_psa_user', 'true');
				$node->appendChild($sn);
			}
		}
	}

	return 1;
}


sub dumpDBTable
{
	my ($doc, $db_name, $table_name) = @_;

	my $table_node = $doc->createElement('table');

	$table_node->setAttribute('name', $table_name);

	my $query = dbDoQuery("SELECT * FROM $db_name.$table_name");
	return '' unless $query;

	while (my $hash = $query->fetchrow_hashref) {
		$ns_rec = $doc->createElement('record');
		$table_node->appendChild($ns_rec);
		while (my ($key, $value) = each %$hash) {
			my $ns_field = $doc->createElement(a2u($key));
			$ns_field->appendChild($doc->createTextNode(a2u($value)));
			$ns_rec->appendChild($ns_field); 
		}
	}

	return $table_node;
}


sub dumpIMPHordeDB
{  # dumping IMP HORDE users preferences
	my ($doc) = @_;

	my $db_node = $doc->createElement('data_bases');
	$db_node->setAttribute('name', 'horde');
	$db_node->setAttribute('type', 'mysql');
	addMysql2Stream($db_node, 'horde', {'force' => 1});

#	$db_node->appendChild(dumpDBTable($doc, 'horde', 'horde_categories'));
#	$db_node->appendChild(dumpDBTable($doc, 'horde', 'horde_categories_categories'));
#	$db_node->appendChild(dumpDBTable($doc, 'horde', 'horde_prefs'));
#	$db_node->appendChild(dumpDBTable($doc, 'horde', 'horde_users'));
#	$db_node->appendChild(dumpDBTable($doc, 'horde', 'turba_objects'));

	return $db_node;
}


#sub dumpSystem {
#	my ($doc, $node) = @_;
#
#} # dumpSystem()


sub getSQLWhereStrByField($ $)
{
	my ($fieldName, $valuesHash) = @_;

	my $sqlWhereStr = '';

	foreach $fieldValue (keys %$valuesHash) {
		if ($fieldValue) {
			$sqlWhereStr .= ($sqlWhereStr ? ' or ' : '') . "$fieldName='$fieldValue'";
		}
	}

	return $sqlWhereStr;
}

sub dumpSingleDomain($)
{
	my ($domain_name) = @_;

	$query = dbDoQuery("SELECT param, val FROM misc;");
	while ( my ($param, $val) = $query->fetchrow_array ) {
		$misc{$param}=$val;
	}

	$conf{PG_ADMIN_LOGIN} = $misc{'postgresql_admin_login'};
	$conf{PG_ADMIN_PASSWORD} = $misc{'postgresql_admin_passwd'};

	my $parser = new XML::DOM::Parser;
	$doc = $parser->parse($psa);

	$psa_node=$doc->getChildNodes()->item(0);

	$psa_node->setAttribute('sqlVer', $misc{version});

	if ($conf{DESCRIPTION}) {
		my $descNode = $doc->createElement('description');
		$descNode->appendChild($doc->createTextNode(a2u($conf{DESCRIPTION})));
		$psa_node->appendChild($descNode);
	}

	$server_node = $doc->createElement('server');
	$psa_node->appendChild($server_node);

	my $psn = $doc->createElement('params');
	$server_node->appendChild($psn);
	my $sn = $doc->createElement('psa_conf');
	$sn->setAttribute('HTTPD_VHOSTS_D', $conf{HTTPD_VHOSTS_D});
	$sn->setAttribute('MAILMAN_ROOT_D', $conf{MAILMAN_ROOT_D});
	$psn->appendChild($sn);

	$sn = $doc->createElement('ip_addresses_list');
	$server_node->appendChild($sn);
	my $cert_id_for_ip = undef;
	{
		my $query = dbDoQuery('SELECT ipa.* FROM IP_Addresses ipa, domains d, hosting h WHERE d.id=h.dom_id AND ipa.id=h.ip_address_id AND d.name=' . dbQuote($domain_name));
		if (my $hash = $query->fetchrow_hashref()) {
			$sn->appendChild(createNodeFromHash($doc, 'ip_address', $hash));
			$cert_id_for_ip = $$hash{ssl_certificate_id};
		} else {
			my $query = dbDoQuery('SELECT ipa.* FROM IP_Addresses ipa, domains d, dns_recs dns WHERE d.id=dns.dom_id AND dns.type="A" AND dns.val=ipa.ip_address AND (dns.host=CONCAT(d.name, ".") OR dns.host=CONCAT(d.displayName, ".")) AND d.name=' . dbQuote($domain_name));
			if (my $hash = $query->fetchrow_hashref()) {
				$sn->appendChild(createNodeFromHash($doc, 'ip_address', $hash));
				$cert_id_for_ip = $$hash{ssl_certificate_id};
			} else {
				print_stderr("Unable to define ip address.\n");
			}
		}
	}

	{ # dump required certificates
		%certs_hash = ();
		$certs_hash{$cert_id_for_ip} = 1;
		$query = dbDoQuery('SELECT r.component_id FROM domains d, Repository r, certificates cert WHERE d.cert_rep_id=r.rep_id AND cert.id=r.component_id AND d.name=' . dbQuote($domain_name) . ' ORDER by d.id');
		while (my ($cert_id) = $query->fetchrow_array) {
			$certs_hash{$cert_id} = 1;
		}
		$server_node->appendChild(dumpCertificates($doc, \%certs_hash));
	}

	$server_node->appendChild(dumpSiteAppPackages($doc));

	my $query = dbDoQuery('SELECT cl_id FROM domains WHERE name=' . dbQuote($domain_name));
	if (my ($id) = $query->fetchrow_array()) {
		$clients{$id} = $psa_node;
	}

	my %domains_names = ($domain_name => 1);
	my %dumpingBy = ('domains_names' => \%domains_names);
	$sqlWhereForDomains = getSQLWhereStrByField('d.name', $dumpingBy{'domains_names'});

	dumpDomains($doc, \%dumpingBy);

	return $doc;
}

sub dumpPSA($) {
	my ($dumpingBy) = @_;

# and read first table:
	$query = dbDoQuery("SELECT param, val FROM misc;");
	while ( my ($param, $val) = $query->fetchrow_array ) {
		$misc{$param}=$val;
	}

	$conf{PG_ADMIN_LOGIN} = $misc{'postgresql_admin_login'};
	$conf{PG_ADMIN_PASSWORD} = $misc{'postgresql_admin_passwd'};

	my $parser = new XML::DOM::Parser;
	$doc = $parser->parse($psa);

	$psa_node=$doc->getChildNodes()->item(0);

	$psa_node->setAttribute('sqlVer', $misc{version});
	$psa_node->setAttribute('creating_timestamp', time());

	if ($conf{DESCRIPTION}) {
		my $descNode = $doc->createElement('description');
		$descNode->appendChild($doc->createTextNode(a2u($conf{DESCRIPTION})));
		$psa_node->appendChild($descNode);
	}

	if (exists($dumpingBy->{'clients_logins'})) {
		$sqlWhereForClients = getSQLWhereStrByField('c.login', $dumpingBy->{'clients_logins'});
	} elsif (exists($dumpingBy->{'clients_ids'})) {
		$sqlWhereForClients = getSQLWhereStrByField('c.id', $dumpingBy->{'clients_ids'});
	} elsif (exists($dumpingBy->{'domains_names'})) {
		$sqlWhereForDomains = getSQLWhereStrByField('d.name', $dumpingBy->{'domains_names'});
	} elsif (exists($dumpingBy->{'domains_ids'})) {
		$sqlWhereForDomains = getSQLWhereStrByField('d.id', $dumpingBy->{'domains_ids'});
	} else {
		$sqlWhereForClients = '';
		$sqlWhereForDomains = '';
	}

###   {
	$server_node = dumpServer($doc);
	$psa_node->appendChild($server_node);
	if ($backup_filter{'SERVER'}) {
		$server_node->appendChild(dumpUpgradeHistory($doc));
		$psa_node->appendChild(dumpAdminUser($doc));
		$server_node->appendChild(dumpNotes($doc));
		$server_node->appendChild(dumpSkins($doc));
		$server_node->appendChild(dumpSlaves($doc));
		$server_node->appendChild(dumpMailerQueries($doc));
	}

	dumpClients($doc, $dumpingBy);

	if ($backup_filter{'SERVER'}) {
		$server_node->appendChild(dumpDomainTemplates($doc));
		$server_node->appendChild(dumpClientTemplates($doc));
	}
### }  

	############################################
	# Dumping crontabs of the server page
	my @serverUsers = ($const{'ADM_USER'});
	# determine if the mailman installed
	if (defined $conf{MAILMAN_ROOT_D} && -d $conf{MAILMAN_ROOT_D}) {
		$serverUsers[$#serverUsers] = "mailman";
	}
	if ($backup_filter{'SERVER'}) {
		foreach my $user (@serverUsers) {
			$server_node->appendChild(dumpCrontab($doc, $user, 'server'));
		}
	}

	if ($backup_filter{'DATABASES'}) {
		my ($by_what, $tmp) = each(%$dumpingBy);
		# for fast work not PSA databases will be dumped first
		unless ($by_what) {
			my $systemNode = $doc->createElement('system');
			dumpSystemDB($doc, $systemNode);
			$systemNode->appendChild(dumpIMPHordeDB($doc));
			$psa_node->appendChild($systemNode);
		}
	}

	dumpDomains($doc, $dumpingBy);

#	unless ($by_what) {
#		dumpSystem($doc, $systemNode);
#	}

	return $doc;
}


sub addPGAdminToDB
{
	my ($db_name, $db_username, $add_del) = @_;

	return 1 if ($flags{'BACKUP_TYPE'} ne 'FULL');

	my $ignore_errors = 0;

#	my $path =  $conf{PIPE_UTIL}
#			. ' ' . semGetKey()
#			. ' "' . getProcessingLogFileName() . '"'
#			. ' "' . ($add_del eq 'add' ? 'add' : 'remove') . " permissions for postgresql admin to '$db_name' \" "
#			. " $ignore_errors "
#			. " \"\" "
#			. ' "umask 0022; ' . $conf{UTILS_DIR} . '/pg_manage ' . (($add_del eq 'add') ? 'add_db_usr' : 'rm_db_usr')
#			. ' ' . dbQuote($db_name) . ' ' . dbQuote($db_username) . '" |';

	my $path = pipe_util_execstr(
			($add_del eq 'add' ? 'add' : 'remove') . " permissions for postgresql admin to '$db_name'",
			$ignore_errors, 0,
			'',
			'umask 0022; ' . $conf{UTILS_DIR} . '/pg_manage ' . (($add_del eq 'add') ? 'add_db_usr' : 'rm_db_usr')
			. ' ' . shellArgQuote($db_name) . ' ' . shellArgQuote($db_username)
	);

	addFakeNode2Stream($path);

	return 1;
}

sub dumpActionLogPrefs
{
	my ($doc) = @_;

	my $node = $doc->createElement('action_log_prefs');

	my $query = dbDoQuery('SELECT id, name, descr, enabled FROM actions');
	while (my ($id, $name, $desc, $enabled) = $query->fetchrow_array()) {
		my $sn = $doc->createElement('action');
		$sn->setAttribute('id', $id);
		$sn->setAttribute('name', $name);
		$sn->setAttribute('enabled', ($enabled eq 'true') ? 'true' : 'false');
		$sn->setAttribute('desc', a2u($desc));
		$node->appendChild($sn);
	}

	return $node;
}

sub dumpCustomReports
{
	my ($doc, $user_id, $user_type) = @_;

	my $query = dbDoQuery("SELECT * FROM report WHERE user_id='$user_id' AND user_type='$user_type'");

	my $node = undef;
	while (my $hashref = $query->fetchrow_hashref()) {
		unless ($node) {
			$node = $doc->createElement('custom_reports');
		}
		my $report_id = $$hashref{'id'};
		delete $$hashref{'id'};
		delete $$hashref{'user_id'};
		delete $$hashref{'user_type'};
		my $sn = createNodeFromHash($doc, 'report', $hashref);
		$node->appendChild($sn);
		{
			my $query = dbDoQuery('SELECT * FROM report_section WHERE report_id=' . dbQuote($report_id));
			next unless $query;
			while (my $hashref = $query->fetchrow_hashref()) {		
				delete $$hashref{'id'};
				delete $$hashref{'report_id'};
				my $sn2 = createNodeFromHash($doc, 'section', $hashref);
				$sn->appendChild($sn2);
			}
		}
		{
			my $query = dbDoQuery('SELECT * FROM report_auto WHERE report_id=' . dbQuote($report_id));
			next unless $query;
			while (my $hashref = $query->fetchrow_hashref()) {		
				delete $$hashref{'id'};
				delete $$hashref{'report_id'};
				my $sn2 = createNodeFromHash($doc, 'auto', $hashref);
				$sn->appendChild($sn2);
			}
		}
	}

	return $node;
}


sub dumpActionLog($$$)
{
	my ($doc, $object_id, $object_type) = @_;

	my $actions_arr = getActionsByObjectType($object_id, $object_type);
	my $actions_list = join '","', @$actions_arr;
	$actions_list = '"'. $actions_list . '"';

	my $node = undef;

	my $query = dbDoQuery('SELECT a.name, la.id, la.user, la.ip_address, la.date FROM actions a, log_actions la WHERE a.id=la.action_id AND (a.name in ('.$actions_list.')) AND la.object_id="' . $object_id . '"');

	unless ($query) {
		print_stderr("Error dumping actions($actions_list) for object with id=$object_id\n");
	} else {
		while (my $hashref = $query->fetchrow_hashref()) {
			unless ($node) {
				$node = $doc->createElement('action_log'); ### group element
			}

			my $sn = $doc->createElement('action'); ### subgroup element

			my $logged_action_id = $$hashref{'id'};
			delete $$hashref{'id'};

			while (my ($attr, $value) = each(%$hashref)) {
				$sn->setAttribute($attr, a2u($value));
			}

			my $query2 = dbDoQuery('SELECT component, old_value, new_value FROM log_components  WHERE action_id=' . dbQuote($logged_action_id));
			while (my ($attr, $old_value, $new_value) = $query2->fetchrow_array()) {
				my $sn2 = $doc->createElement('component'); ### simple element
				$sn2->setAttribute('name', a2u($attr));
				$sn2->setAttribute('old_value', a2u($old_value));
				$sn2->setAttribute('new_value', a2u($new_value));
				$sn->appendChild($sn2);
			}
			$node->appendChild($sn);
		}
	}

	return $node;
}


1;
__END__

=head1 NAME

BU::PSA::PSA2xml - The PSA modules for ... .

=head1 SYNOPSIS

A short usage summary for programs and functions, which may someday be deemed mandatory. 

=head1 DESCRIPTION

Long drawn out discussion of the program. It's a good idea to break this up into subsections using the =head2 directives, like 

=head2 A Sample Subection

=head2 Yet Another Sample Subection

=head1 OPTIONS

Some people make this separate from the description. 

=head1 RETURN VALUE

What the program or function returns if successful. 

=head1 ERRORS

Exceptions, return codes, exit stati, and errno settings. 

=head1 EXAMPLES

Give some example uses of the program. 

=head1 ENVIRONMENT

Envariables this program might care about. 

=head1 FILES

All files used by the program. You should probably use the F<> for these. 

=head1 SEE ALSO

Other man pages to check out, like man(1), man(7), makewhatis(8), or catman(8). 

=head1 NOTES

Miscellaneous commentary. 

=head1 CAVEATS

Things to take special care with; sometimes called WARNINGS. 

=cut DIAGNOSTICS

All possible messages the program can print out--and what they mean. 

=head1 BUGS

