###################################################################
package BU::PSA::CommonFunc;
###################################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::Logger;
use BU::PSA::DB;
use BU::FileFunc;
use MIME::Base64 qw(encode_base64 decode_base64);
use Digest::MD5 qw(md5_hex);

use BU::CommonFunc;
@ISA = qw(BU::CommonFunc);


@EXPORT = qw(
getMiscParam putMiscParam
createDefaultDNSZone
$sys_no_change
denyFtpAccessByList
getSysIPsList
isValidIP
setPasswd getPwData
askInteractive
checkInstalledPSAVersion 
ver2int
a2u u2a
genSaltForCrypt
util_exec 
util_exec_env 
util_exec_errormsg
util_exec_error
hard_disk_quota_is_supported
lock_domain unlock_domain
lock_bu_process unlock_bu_process is_bu_process_locked
url_encode
url_decode
checkPostgresRunning
initPostgres
checkPostgresDBExisting
checkPostgresConnecting
mapSysUserHomeDir
shellArgQuote
escapeFileNameSpecChar
getShellsList
writeShellsListToMapFile
readShellsListFromMapFile
correctFreeBSDIPMasks
checkPackagesInstallation
hashXMLNodeByCID
);

$sys_no_change = '';

@errormsg_array = ();

local %misc = ();
local $misc_is_loaded = undef;

sub util_exec_env
{
	my ($util, $env_vars, @params) = @_;

	my $exec_str = '';

	foreach $var (keys %$env_vars) {
		next unless $var;
		my $var_val = $$env_vars{$var};
		$var_val =~ s/'/\\'/g;
		$exec_str .= "export $var='$var_val'; ";
	}

	$exec_str .= $conf{UTILS_DIR} . '/' . $util;

	foreach $i (@params) {
		$exec_str .= ' ' . $i;
	}

	unless ($sys_no_change) {
		print_stdout ("REAL EXEC: ", $exec_str, "\n") if ($debug > 0);

		unless (open (fd, $exec_str . ' 2>/dev/null |')) {
			return undef;
		}

		@errormsg_array = ();

		while (<fd>) {
			next unless $_;
			push(@errormsg_array, $_);
		}

		$result_code = close fd ? '0' : '1';

		my $last = $#errormsg_array;

		if ($last < 0) {
			return 1;
		}

#		my $result_code = pop(@errormsg_array);
		if ($result_code =~ m/\s*0\s*/) {
			my $result_code = pop(@errormsg_array);
			if ($result_code =~ m/\s*[0-9]+\s*/) { # is integer (used second code)
				if ($result_code =~ m/\s*0\s*/) {
					return 1;
				} else {
					return undef;
				}
			}
			return 1;
		}

		return undef;
	}

	print_stdout ("EMULATE EXEC: ", $exec_str, "\n") if ($debug > 0);

	if ($exec_str =~ 'exist') {
		return '';
	}

	return 1;
}


sub util_exec
{
	my ($util, @params) = @_;

	return util_exec_env($util, undef, @params);
}


sub util_exec_error()
{
	return ($#errormsg_array >= 0);
}


sub util_exec_errormsg()
{
	my $errormsg = '';

	foreach $i (@errormsg_array) {
		$errormsg .= $i;
	}

	return $errormsg;
}


sub getMiscParam(;$)
{
	my ($param) = @_;

	if (!$misc_is_loaded || $param && !exists($misc{$param})) {
		### load params from misc table
		my $query = dbDoQuery('SELECT param, val FROM misc');
		while ( my ($p, $v) = $query->fetchrow_array ) {
			$misc{$p} = $v;
		}
		$misc_is_loaded = 1;
	}

	return \%misc unless ($param);
	return $misc{$param};
}


sub putMiscParam($ $)
{
	my ($param, $value) = @_;

	return dbDoQuery('REPLACE INTO misc(param, val) '
					.'values('.dbQuote($param).', '.dbQuote($value).')');
}


sub insert_dns_rec($ $ $ $; $)
{
	my ($domainId, $host, $type, $val, $opt) = @_;

	$DNSRec = new PSA::DNSRec($domainId, $host, $type, $val, $opt);
	
	$DNSRec->Store();
	
	return 1;
}


sub createDefaultDNSZone($ $ $)
{
	my ($domainId, $domainName, $wwwPrefix) = @_;


	# delete old DNS records
	unless (dbDoQuery('DELETE FROM dns_recs WHERE dom_id=' . $domainId)) {
#		$self->setError();
		return '';
	}

#	$self->setField('dns_zone', not (get_param('dns_zone_status') eq 'false'));

	my $query = dbDoQuery("SELECT type, host, val, opt FROM dns_recs_t");

	while (my ($type, $host, $val, $opt) = $query->fetchrow_array) {
		$host =~ s/<ip>/$ip/gi;
		$host =~ s/<domain>/$domainName/gi;
		$val =~ s/<ip>/$ip/gi;
		$val =~ s/<domain>/$domainName/gi;

		unless (insert_dns_rec($domainId, $host, $type, $val, $opt)) {
			return '';
		}
	}

	if ($wwwPrefix) {
		unless (insert_dns_rec($domainId, "www.$domainName.", 'CNAME', "$domainName.")) {
			return '';
		}
	}

	unless (util_exec('dnsmng', 'update ' . $domainName)) {
		dbDoQuery('DELETE FROM dns_recs WHERE dom_id=' . $domainId);
		return '';
	}

	return 1;
}


sub denyFtpAccessByList(@)
{
	my (@usersList) = @_;

	# read existing file /etc/ftpusers
	unless (open FD, '/etc/ftpusers') {
#		print_stderr("Unable to deny FTP access for users by list: can't to open file /etc/ftpusers\n");
		return '';
	}

	%ftpUsers = ();
	while (<FD>) {
		next if m/^\s*#/;

		if (m/^\s*([^\s]+)\s*$/) {
			$ftpUsers{$1} = 1;
		}
	}

	unless (close FD ) {
#		print_stderr("Error deny FTP access for users by list: reading error from file /etc/ftpusers\n");
		return '';
	}

	# add new users
	foreach $newFtpUser (@usersList) {
		$ftpUsers{$newFtpUser} = 1;
	}

	# write tmp file /etc/ftpusers.tmp
	my $tmp_file = randFileName('/etc/ftpusers');
	unless (open FD2, ">$tmp_file") {
#		printf_stderr("Unable to deny FTP access for users by list: can't to open temporary file %s\n", $tmp_file);
		return '';
	}

	foreach $user (keys %ftpUsers) {
		print FD2 "$user\n";
	}

	unless (close FD2 ) {
#		printf_stderr("Unable to deny FTP access for users by list: can't to open temporary file %s\n", $tmp_file);
		return '';
	}

	unless (system("mv -f $tmp_file /etc/ftpusers >/dev/null 2>&1")) {
		return '';
	}

	return 1;
}


sub getSysIPsList()
{
	my %sysIPList = ();

	# get all ip addresses from database
	my $query = dbDoQuery('SELECT id, iface, ip_address, type, mask FROM IP_Addresses');
	while (($ip_id, $db_ip_iface, $ip_address, $db_ip_type, $db_ip_mask) = $query->fetchrow_array()) {
		my $client_login = '';
		if ($db_ip_type eq 'exclusive') {
			my $query = dbDoQuery('select login from clients c, Repository r where c.pool_id=r.rep_id and r.component_id=' . dbQuote($ip_id));
			($client_login) = $query->fetchrow_array();
		}
		my @tmp_arr = ($db_ip_iface, $db_ip_mask, $db_ip_type, $client_login);
		$sysIPList{$ip_address} = \@tmp_arr;
		printf_stderr ("get FROM DB iface: %s, (ip: %s, mask: %s, type: %s, owner: %s)\n", $db_ip_iface, $ip_address, $db_ip_mask, $db_ip_type, $client_login) if ($debug > 0);
	}


	# get network interfaces list
	unless (open (fd_ifconfig, $conf{UTILS_DIR}."/ifmng -a|")) {
		print_stderr("Unable to retrieve network interfaces list: unable to exec 'interface -a'.\n");
		return '';
	}

	my %ifaces = ();

	while (!eof(fd_ifconfig)) {
		$_ = <fd_ifconfig>;
		m/^\s*([a-zA-Z0-9_]+)\s*$/;
		$ifaces{$1} = 1 if $1;
		print_stderr ("iface: $1\n") if ($debug > 0);
	}

	unless (close fd_ifconfig ) {
		print_stderr("Unable to retrive network interfaces list.\n");
		return '';
	}

	# get system ip addresses list from all interfaces
	foreach my $iface (keys %ifaces) {
		unless (open (fd_ifconfig, $conf{UTILS_DIR}."/ifmng -i $iface|")) {
			printf_stderr("Unable to retrieve ip addresses for '%s' interface.\n", $iface);
			return '';
		}

		binmode fd_config;

		while ($_ = <fd_ifconfig>) {
			chomp($_);
			m/^\s*([0-9\.]+)\s+([0-9\.]+)\s*[0-9]{0,3}\s*$/;
			my $ip_address = $1;
			my $ip_mask = $2;

			if (exists($sysIPList{$ip_address})) {
				next;
			}

			my @tmp_arr = ($iface, $ip_mask, '', '');
			$sysIPList{$ip_address} = \@tmp_arr;

			printf_stderr ("get FROM SYSTEM iface: %s, (ip: %s, mask: %s)\n", $iface, $ip_address, $ip_mask) if ($debug > 0);
		}

		unless (close fd_ifconfig ) {
			printf_stderr("Unable to retrieve ip addresses for %s interface.\n", $iface);
			return '';
		}
	}

	return (\%ifaces, \%sysIPList);
}


sub applyIPMask($ $)
{
	my ($ip, $mask) = @_;

	my @ip_arr = split /\./, $ip;
	my @mask_arr = split /\./, $mask;

	for (my $i=0; $i<4; $i++) {
		$ip_arr[$i] = ($ip_arr[$i] + 0) & ($mask_arr[$i] + 0);
	}

	print "applyMask:$ip,$mask=>(" . join('.', @ip_arr) . ")\n" if ($debug > 5);
	return join('.', @ip_arr);
}


sub correctFreeBSDIPMasks($)
{
	my ($ip_list) = @_;
	my %ip_basis = ();

	while (my ($ip, $ip_info) = each(%$ip_list)) {
		my ($tmp, $mask, $tmp, $tmp) = @$ip_info;
		print "ip:($ip) mask:($mask)\n" if ($debug > 5);
		if ($mask ne '255.255.255.255') {
			$ip_basis{$ip} = applyIPMask($ip, $mask);
			print "basis ip:($ip) mask:($mask)\n" if ($debug > 5);
		}
	}

	while (my ($ip, $ip_info) = each(%$ip_list)) {
		next if exists($ip_basis{$ip});

		while (my ($ip_b, $ip_template) = each(%ip_basis)) {
			my $ip_b_info = $$ip_list{$ip_b};
			my ($tmp, $mask_b, $tmp, $tmp) = @$ip_b_info;
			print "ip_b:($ip_b) mask:($mask_b) template($ip_template)\n" if ($debug > 5);
			if (applyIPMask($ip, $mask_b) eq $ip_template) {
				print "change mask: ($mask_b)\n" if ($debug > 5);
				$$ip_info[1] = $mask_b;
			}
		}
	}
}


sub isValidIP($)
{
	my ($ip) = @_;

	return '' unless ($ip =~ m/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/);

	for ($i=1; $i<=4; $i++) {
		if ((${$i} < 0) || (${$i} > 255)) { return ''; }
	}

	return 1;
}


sub setPasswd($$)
{
	my ($name,$crypt_passwd) = @_;

	return system($const{chpasswd} . ' -p ' . dbQuote($crypt_passwd) . " $name >/dev/null 2>&1");
}

sub checkInstalledPSAVersion
{
	if ((ver2int($conf{PSA_VERSION}) < ver2int($const{MINOR_SUPPORTED_PSA_VERSION})) or 
		(ver2int($conf{PSA_VERSION}) > ver2int($const{MAJOR_SUPPORTED_PSA_VERSION})))
	{
		die_log "You are running PSA version %s . This script will work properly with PSA versions %s - %s only.\n",
			$conf{PSA_VERSION}."\n",
			$const{MINOR_SUPPORTED_PSA_VERSION}."\n",
			$const{MAJOR_SUPPORTED_PSA_VERSION}."\n";
	}
}


sub hard_disk_quota_is_supported()
{
	$ret = util_exec('usermng', ('--isquotable-quiet'));

	if (util_exec_error()) {
		printf_stderr('Error hard disk quota support checking: ' . util_exec_errormsg());
		return undef;
	}

	return $ret;
}


sub lock_domain($)
{
	my ($domainName) = @_;

	return 1 if $flags{'NO_BLOCK'};

	$lock_domain_struct = lock_file($conf{DUMP_D} . "/$domainName/" . $const{LOCK_FILE_NAME}, 1);

	unless ($lock_domain_struct) {
		printf_stderr("Unable to lock domain '%s': error in lock_file() function.\n", $domainName);
		return '';
	}

	if ($$lock_domain_struct{error}) {
		if ($$lock_domain_struct{error_message} =~ m/esource.+temporarily.+unavailable/) {
			printf_stderr("Unable to lock domain '%s': psadump or psarestore is already running now.\n", $domainName);
		} else {
			printf_stderr("Unable to lock domain '%s': %s.\n", $domainName, $$lock_domain_struct{error_message});
		}
		return '';
	}

	my $query = dbDoQuery('SELECT status FROM domains WHERE name=' . dbQuote($domainName));
	unless ($query) {
		printf_stderr("Unble to define status for domain '%s'\n", $domainName);
		unlock_file($lock_domain_struct);
		return '';
	}

	my ($status) = $query->fetchrow_array();
	$status += 0;
	if ($status & 4) { # set backup/restore domain status
		return $lock_domain_struct;
	}

	dbDoQuery(sprintf('UPDATE domains SET status=%s WHERE name=%s', ($status | 4), dbQuote($domainName)));
	unless (util_exec('domainmng', '--turn-off', "--domain-name='$domainName'")) {
		printf_stderr("Unable to turn domain '%s' off: %s\n", $domainName, util_exec_errormsg());
		unlock_file($lock_domain_struct);
		if ($status == 0) {
			util_exec('domainmng', '--turn-on', "--domain-name='$domainName'");
		}
		dbDoQuery('UPDATE domains SET status=' . $status . ' WHERE name=' . dbQuote($domainName));
		return '';
	}

	return $lock_domain_struct;
}


sub unlock_domain($ $)
{
	my ($domainName, $lock_domain_struct) = @_;

	return 1 if $flags{'NO_BLOCK'};
	return '' unless $lock_domain_struct;

	my $query = dbDoQuery('SELECT status FROM domains WHERE name=' . dbQuote($domainName));
	unless ($query) {
		printf_stderr("Unable to define status for domain '%s'\n", $domainName);
		return '';
	}

	my ($status) = $query->fetchrow_array();
	$status += 0;
	if ($status == 0) {
		return 1;
	}

	unlock_file($lock_domain_struct);

	if ($$lock_domain_struct{error}) {
		my $lock_file = $conf{DUMP_D} . "/$domainName/" . $const{LOCK_FILE_NAME};
		printf_stderr("Unable to unlock file '%s': %s\n", $lock_file, $$lock_domain_struct{error_message});
#		return '';
	}

	dbDoQuery(sprintf('UPDATE domains SET status=%s WHERE name=%s', ($status & ~4), dbQuote($domainName)));

	if (($status & ~4) == 0) {
		unless (util_exec('domainmng', '--turn-on', "--domain-name='$domainName'")) {
			dbDoQuery(sprintf('UPDATE domains SET status="%s" WHERE name=%s', ($status & ~4) | $const{'STATUS_STATUS'}, dbQuote($domainName)));
			printf_stderr("Unable to turn domain '%s' on: %s\n", $domainName, util_exec_errormsg());
			return '';
		}
	}


	return 1;
}


local $lock_bu_struct = undef;


sub lock_bu_process()
{
	return 1 if $flags{'NO_BLOCK'};

	my $lock_file = $conf{DUMP_D} . '/' . $const{LOCK_FILE_NAME};
	$lock_bu_struct = lock_file($lock_file, 1);

	unless ($lock_bu_struct) {
		printf_stderr("Unable to create lock file '%s': error in lock_file() function\n", $lock_file);
		return '';
	}

	if ($$lock_bu_struct{error}) {
		if ($$lock_bu_struct{error_message} =~ m/esource\s+temporarily\s+unavailable/) {
			printf_stderr("Error: unable to continue processing: psadump or psarestore is already running now.\n");
		} else {
			printf_stderr("Error: unable to continue processing: %s\n", $$lock_bu_struct{error_message});
		}
		return '';
	}

	return 1;
}

##########################################################################
#  @desc Check if server-wide backup is currently performed on host
#  @return 1 if there is backup currently on server, othervise returns 0
##########################################################################
sub is_bu_process_locked() {

   ###############################################
   #  If this file is locked - backup is running
   ###############################################
   my $filename = $conf{DUMP_D}.'/'.$const{LOCK_FILE_NAME};

   my $lock = lock_file($filename);

   unless ($lock) {
      ###############################################
      #  Failed to lock file - means backup running
      ###############################################
      return 1;
   }

   #########################################
   #  Fine, no backup currently running
   #  There is a race condition however!!!
   #########################################
   unlock_file($lock);

   return 0;
}

sub unlock_bu_process()
{
	return 1 if $flags{'NO_BLOCK'};

	unlock_file($lock_bu_struct);

	if ($$lock_bu_struct{error}) {
		my $lock_file = $conf{DUMP_D} . '/' . $const{LOCK_FILE_NAME};
		printf_stderr("Unable to unlock file '%s': %s\n", $lock_file, $$lock_bu_struct{error_message});
		return '';
	}

	return 1;
}


sub checkPostgresRunning()
{
#	my $res = system(sprintf('PGDATA=%s %s/pg_ctl status 2>&1 >/dev/null', $conf{'PGSQL_DATA_D'}, $conf{'PGSQL_BIN_D'}));
	my $res = system($conf{PGSQL_SCRIPT} . ' status 2>&1 >/dev/null');
	return ($res == 0);
}


sub initPostgres
{
	my ($psaNode) = @_;
	my $res = 1;

	if (!$conf{'postgresql_admin_login'} || !$conf{'postgresql_admin_passwd'}) {
		$res = 2; ## need postgres restart
		printf_stderr("Postgres is not configured on this server:\n");

		if ($psaNode) {
			if (my ($node) = $psaNode->xql('./server/params')) {
				printf_stderr("Fetch postgres admin login from dump.\n");
				$conf{'postgresql_admin_login'} = u2a($node->getAttribute('postgresql_admin_login')) unless ($conf{'postgresql_admin_login'});
				printf_stderr("Fetch postgres admin password from dump.\n");
				$conf{'postgresql_admin_passwd'} = u2a($node->getAttribute('postgresql_admin_passwd')) unless ($conf{'postgresql_admin_passwd'});
			}
		}

		unless ($conf{'postgresql_admin_login'}) {
			$conf{'postgresql_admin_login'} = substr(md5_hex(rand() * 9999), 0, 8);
			printf_stderr("Generate postgres admin login: %s\n", $conf{'postgresql_admin_login'});
		}
		unless ($conf{'postgresql_admin_passwd'}) {
			$conf{'postgresql_admin_passwd'} = substr(md5_hex(rand() * 9999), 0, 8);
			printf_stderr("Generate postgres admin password: %s\n", $conf{'postgresql_admin_passwd'});
		}

		printf_stderr("Create postgres admin user.\n");
		my $buff = sprintf('umask 0022; PSA_PASSWORD=%s %s/pg_manage create_user %s 2>&1 >/dev/null',
							shellArgQuote($conf{'postgresql_admin_passwd'}), $conf{UTILS_DIR},
							$conf{'postgresql_admin_login'});

		if (system($buff) == 0) {
			putMiscParam('postgresql_admin_passwd', $conf{'postgresql_admin_passwd'});
			putMiscParam('postgresql_admin_login', $conf{'postgresql_admin_login'});
		} else {
			printf_stderr("Error during postgres database user '%s' creating.\n", $conf{'postgresql_admin_login'});
			$conf{'postgresql_admin_login'} = '';
			$conf{'postgresql_admin_passwd'} = '';
			$res = 0; ## error occured
		}
	}

	return $res;
}


sub checkPostgresConnecting
{
	return '' if (!$conf{'postgresql_admin_login'} || !$conf{'postgresql_admin_passwd'});

	my $exec_str = sprintf('PGPASSWORD=%s %s/psql -U%s -tc "" template1 2>&1 >/dev/null; echo $?',
							shellArgQuote($conf{'postgresql_admin_passwd'}), $conf{PGSQL_BIN_D},
							$conf{'postgresql_admin_login'});

	my $res = `$exec_str`;
	print "($exec_str):($res)\n" if $debug > 0;

	chomp($res); chomp($res); chomp($res);
	$res =~ s/\s//g;

	print "checkPostgresConnecting:($res)\n" if $debug > 0;

	if (($res !~ m/^[0-9]+$/) || ($res+0 > 0)) {
		return '';
	}

	return 1;
}


sub checkPostgresDBExisting
{
	my ($dbName) = @_;

	return -1 unless $conf{PG_IS_RUNNING};
	return -1 if (!$conf{'postgresql_admin_login'} || !$conf{'postgresql_admin_passwd'});

	my $query_str = 'select count(*) from pg_database where lower(datname)=lower(' . dbQuote($dbName) . ')';
	my $exec_str = sprintf('PGPASSWORD=%s %s/psql -U%s -tc "%s" template1 2>/dev/null',
							shellArgQuote($conf{'postgresql_admin_passwd'}), $conf{PGSQL_BIN_D},
							$conf{'postgresql_admin_login'}, $query_str);

	my $res = `$exec_str`; ### get count of databases with name $dbName
	print "($exec_str):($res)\n" if $debug > 0;

	chomp($res); chomp($res); chomp($res);
	$res =~ s/\s//g;

	if ($res !~ m/^[0-9]+$/) { ### if res = count(*) is not numeric value then error
		return -1;
	}

	return (($res + 0) > 0) ? 1 : 0;
}


sub mapSysUserHomeDir
{
	my ($home) = @_;

#	print "[" . $conf{OLD_HTTPD_VHOSTS_D} . "/" . $conf{HTTPD_VHOSTS_D} . "] = ($home) -> ";

	$home =~ s/$conf{OLD_HTTPD_VHOSTS_D}/$conf{HTTPD_VHOSTS_D}/e;

#	print "($home)\n";

	return $home;
}


sub writeShellsListToMapFile($ $)
{
	my ($shells_map_file, $shells_list) = @_;

	my $shells_from_system = &getShellsList();

	unless (open (fd, ">$shells_map_file")) {
		print_stderr("Unale to open for writing file $shells_map_file: $!\n");
		return undef;
	}

	while (my ($file, $name) = each(%$shells_list)) {
		print fd "$file => ";
		print fd $shells_from_system->{$name} if (exists($shells_from_system->{$name}));
		print fd "\n";
	}

	close fd;
}


sub readShellsListFromMapFile($ $)
{
	my ($shells_map_file, $shells_from_system) = @_;

	unless (open fd, $shells_map_file) {
		print_sterr("Unable to open shells map file: $!\n");
		return undef;
	}

	my %shells_map = ();
	my $line = 1;

	while (<fd>) {
		chomp($_);
		s/#.*$//;
		next if m/^[ \t]*$/;
		unless (m/^[ \t]*([^ \t]+)[ \t]*=>[ \t]*([^ \t]+)[ \t]*$/) {
			print_stderr("Syntax error detected in '$shells_map_file' on line $line.\n");
			return undef;
		}

		unless (exists($shells_from_system->{$2})) {
			print_stderr("Target shell '$2' which is specified in map file for source shell '$1' is absend in system.\n");
			return undef;
		}

		$shells_map{$1} = $2;
		$line++;
	}

	close fd;

	return \%shells_map;
}

sub getShellsList
{
	my ($format__SHELLFILE_SHELLNAME) = @_;

	my %shells_list = ();

	if ( -e '/etc/shells') {
		if (open (fd, '/etc/shells')) {
			while (<fd>) {
				chomp($_);
				next if m/^[ \t]#/;
				next unless m/^[ \t]*([^ \t]+\/([^\/ \t]+))[ \t]*$/;

				if ($format__SHELLFILE_SHELLNAME) {
					$shells_list{$1} = $2;
				} else {
					$shells_list{$2} = $1;
				}
			}
			close fd;
			return \%shells_list;
		}
	}

	@search_list = ('sh', 'csh', 'tcsh'. 'bash', 'bash2', 'login', 'false', 'rbash', 'ash');
	foreach $shell (@search_list) {
		$_ = `whereis -b $shell`;
		next unless m/^[^:]+:[ \t]*([^ \t]+\/([^\/ \t]+))[ \t]*$/;

		if ($format__SHELLFILE_SHELLNAME) {
			$shells_list{$1} = $2;
		} else {
			$shells_list{$2} = $1;
		}
	}

	return \%shells_list;
}


sub checkPackagesInstallation()
{
	{ ### checking ColdFusion package installation
		my $cf_type = getMiscParam('cf_instance_type');
		my $cf_path = getMiscParam('cf_instance_path');
		if ($cf_type && $cf_path) {
			$conf{'COLDFUSION_IS_INSTALLED'} = (-d $cf_path);
		} else {
			$conf{'COLDFUSION_IS_INSTALLED'} = undef;
		}
	}

	my %pkg_list = (
		'frontpage'		=> 'FRONTPAGE_IS_INSTALLED',
		'tomcat'		=> 'TOMCAT_IS_INSTALLED',
		'mailman'		=> 'MAILMAN_IS_INSTALLED',
		'spamassassin'	=> 'SPAMASSASSIN_IS_INSTALLED',
		'drweb'			=> 'DRWEB_IS_INSTALLED'
	);

	if ($const{OSid} eq 'freebsd') {
		$conf{PSA_SCRIPT_STARTALL}		= $conf{PSA_SCRIPT} . ' start';
		$conf{PSA_SCRIPT_STOPALL}		= $conf{PSA_SCRIPT} . ' stop';
		$conf{DISTRIB_TYPE} = STANDARD;

		foreach my $pkg_flag (values %pkg_list) {
			$conf{$pkg_flag} = 1;
		}
	} elsif ($const{OSid} eq 'linux') {
		$pkg_list{'psa'} = 'PSARPM_IS_INSTALLED';

		### refresh Components table
		system($conf{UTILS_DIR}.'/packagemng');

		### check installed for components
		my $query = dbDoQuery('SELECT name, (version!="not_installed") FROM Components');
		while (my ($pkg, $installed) = $query->fetchrow_array()) {
			if (exists($pkg_list{$pkg})) {
				$conf{$pkg_list{$pkg}} = $installed;
			}
		}

		if ($conf{PSARPM_IS_INSTALLED}) { ### psa rpm is present - it means rpm distribution
			$conf{PSA_SCRIPT_STOPALL}		= $conf{PSA_SCRIPT} . ' stopall';
			$conf{DISTRIB_TYPE} = RPM;
		} else { ### psa rpm is not present - it means standard distribution
			$conf{PSA_SCRIPT_STOPALL}		= $conf{PSA_SCRIPT} . ' stop';
			$conf{DISTRIB_TYPE} = STANDARD;
		}
		$conf{PSA_SCRIPT_STARTALL}		= $conf{PSA_SCRIPT} . ' start';

		if ($conf{'SPAMASSASSIN_IS_INSTALLED'}) {
			$conf{'SPAMASSASSIN_IS_INSTALLED'} = (-e $conf{UTILS_DIR} . '/spammng');
		}
	} else {
		die_log "Error: unsupported OS is detected.\n";
	}
}

sub hashXMLNodeByCID($)
{
	my ($node) = @_;

	return '' unless $node;

	$cid = $node->getAttribute('src');

	return '' unless $cid;

	$cid =~ s/^\s*cid\s*:\s*//;

	$globIndex{$cid} = $node;

	return 1;
}


BEGIN {
	srand time;
}

return 1;

