###################################################################
package BU::PSA::Const;
###################################################################

require 5.004;
require Exporter;

use BU::Init;
use BU::CommonFunc;
use BU::Logger;

use POSIX qw(geteuid strftime);

@ISA = qw(Exporter);
@EXPORT = qw(
$psaNode
%conf %flags %const
%metod_const %domains_backup_dirs 
read_conf save_conf
FULL CONF BASE64 RPM STANDARD
read_PSA_limits
printPASTitle
%backup_filter

$debug
$psaNode
%globIndex
$ProcessName
$psa_xml_template
);

$psaNode= undef;
%globIndex = ();
$ProcessName = undef;

# global constants
%const = (%const, (
'STATUS_STATUS'				=> 1,
'STATUS_WITH_PARENT'		=> 2,
'STATUS_BACKUP_RESTORE'		=> 4,
'STATUS_ADMIN'				=> 16,
'STATUS_RESELLER'			=> 32,
'STATUS_CLIENT'				=> 64,

'IS_ADMIN'					=> 1,
'IS_RESELLER'				=> 2,
'IS_CLIENT'					=> 4,
'IS_DOMAIN_OWNER'			=> 8,
'IS_MAIL_USER'				=> 16,
'IS_UNDEFINED'				=> 1024,

PSA_VERSION					=> '7.5.x',
MINOR_SUPPORTED_PSA_VERSION	=> '7.5.0',
MAJOR_SUPPORTED_PSA_VERSION	=> '7.5.x',
BU_NAME						=> 'Backup Utilities',
BUILD_NAME					=> 'psa_v7.5.x_Backup_Utilities_build060322.10_os_Mandrake_10.0_i686',
SQL_VERSION_STR				=> 'sqlVer',
PSA_CONF_D					=> '/etc/psa',
PSA_CONF_FILE				=> '/etc/psa/psa.conf',
PSA_SHADOW_FILE				=> '/etc/psa/.psa.shadow',
PSA_KEY						=> '/etc/psa/psa.key',
PSA_KEY_BUFF_SIZE			=> 100000,
PSA_ADMIN_NAME				=> 'admin',
PSA_DB_NAME					=> 'psa',
WEBMAIL_ADMIN_NAME			=> 'horde',
WEBMAIL_DB_NAME				=> 'horde',
PRODUCT_NAME				=> 'Plesk',
FTP_USER					=> 'psaftp',
FTP_GROUP					=> 'psaftp',
ADM_USER					=> 'psaadm',
ADM_GROUP					=> 'psaadm',
CLN_GROUP					=> 'psacln',
APACHE_GROUP				=> 'psaserv',
FILE_MASK					=> 0600,
FILE_UMASK					=> 077,
LINE						=> "\n".'==============================================================================='."\n\n",
LOCK_FILE_NAME				=> 'backup_restore.lock',
DOMAIN_BU_LOG_FILE			=> 'backup_restore.log',
DOMAIN_BU_SYS_LOG_FILE		=> 'backup_restore_sys.log',
BU_PUBKEY_FILE				=> '/admin/htdocs/bu_pubkey.txt'
));

# global flags
%flags = (
BACKUP_TYPE		=> 'CONF',  # valid values: FULL, CONF
RESTORE_MODE	=> 'CHECK' # valid values: CHECK, RESTORE
);

## services list for backup
%backup_filter = (
SERVER			=> 1,
MAIL			=> 1,
DATABASES		=> 1,
HOSTING			=> 1,
MAILLISTS		=> 1,
DOMAIN_OWNER	=> 1
);

# directories list for each domain which must be dumped in backup
%domains_backup_dirs = (
none => 0, 
vrt_hst => 1,
std_fwd => 0, 
frm_fwd => 0
);

%conf = ();

sub read_conf
{
	my $cf = $const{PSA_CONF_FILE};

	unless (-r $cf ) {
		die_log "Undefined PSA configuration file name (%s).\n","$cf\n";
	}

	open FCONF, "<$cf" || die_log "Unable to read %s. The cause: %!\n" , "$cf\n";

	while (<FCONF>) {
		s/\#.*$//;
		m/^\s*(\w+)\s+(.+?)\s*$/;
		next unless $1;
		print_stdout ("conf var: $1 => $2\n") if ($debug > 0);
		$conf{$1} = $2;
	}

	close FCONF;

	$conf{PIPE_UTIL}					= $conf{PRODUCT_ROOT_D} . '/bin/pipe_util';

	$conf{UTILS_DIR}					= $conf{PRODUCT_ROOT_D} . '/admin/bin';
	$conf{ADMIN_ROOTCHAIN_CERTIFICATE}	= $conf{PRODUCT_ROOT_D} . '/admin/conf/rootchain.pem';
	$conf{OPENSSL_CNF}					= $conf{PRODUCT_ROOT_D} . '/admin/conf/openssl.cnf';
	$conf{QMAIL_CONTROL_D}				= $conf{QMAIL_ROOT_D}	. '/control';
	$conf{CERTIFICATES_D}				= $conf{PRODUCT_ROOT_D}	. '/var/certificates';

	$conf{ADMIN_USER}					= 'admin';
	$conf{PSA_SCRIPT}					= $conf{PRODUCT_RC_D} . '/psa';

	if (-e $conf{PRODUCT_RC_D} . '/postgresql') {
	    $conf{PGSQL_SCRIPT}	= $conf{PRODUCT_RC_D} . '/postgresql';
	} else {
	    $conf{PGSQL_SCRIPT}	= $conf{PRODUCT_RC_D} . '/rhdb';
	}

	$conf{PGSQL_IS_INSTALLED}			= (-e $conf{PGSQL_SCRIPT});

	if (($const{OSid} eq 'linux') && (($const{linux_distr} eq 'mandrake') || ($const{linux_distr} eq 'suse') || ($const{linux_distr} eq 'debian'))) {
		$conf{MYSQL_SCRIPT}	= $conf{PRODUCT_RC_D} . '/mysql';
	} else {
		$conf{MYSQL_SCRIPT}	= $conf{PRODUCT_RC_D} . '/mysqld';
	}


	$conf{LOGOS_REPOSITORY_D} = $conf{PRODUCT_ROOT_D} . '/admin/htdocs/images/logos';

#	$flags{BACKUP_FILENAME} = strftime "psa%Y%m%d.%H.archive", localtime;
	$flags{BACKUP_DATE} = strftime "%Y%m%d.%H", localtime;
}

sub define_component_versions
{
	### define mysql version
	my $mysql_version_str;

	if ($const{OSid} eq 'freebsd') {
		$mysql_version_str=`$conf{PRODUCT_ROOT_D}/mysql/bin/mysql --version`;
	} else {
		$mysql_version_str=`mysql --version`;
	}

	chomp($mysql_version_str);
	$mysql_version_str =~ m/[Vv][Ee][Rr]\s+[0-9\.]+\s+Distrib\s+([0-9\.]+([a-z])?)[\s,]/;
	my $mysql_version = $1;
	if ($mysql_version) {
		$const{MYSQL_VERSION} = $mysql_version;


		if ($const{MYSQL_VERSION} !~ m/^3/) {
			### TEMPORARY FIX OF PROBLEM:
			### mysql has 4.x.x version but mysql database has format of 3.x.x version
			### BEGIN (additional check for mysql permissions format)
			my $mysql_passwd = shellArgQuote(read_psa_passwd());

			my $result = 0;
			if ($const{OSid} eq 'freebsd') {
				$result=`$conf{PRODUCT_ROOT_D}/mysql/bin/mysql -uadmin -p$mysql_passwd -e 'select Lock_tables_priv from mysql.db where 1=0' >/dev/null 2>&1; echo \$?`
			} else {
				$result=`mysql -uadmin -p$mysql_passwd -e 'select Lock_tables_priv from mysql.db where 1=0' >/dev/null 2>&1; echo \$?`
			}
			$const{MYSQL_VERSION} = ($result =~ m/0/) ? '4.0.0' : '3.0.0';
			### END
		}
	} else {
		print_stderr("Unable to define mysql version, use mysql 3.x.x privileges syntax\n");
		$const{MYSQL_VERSION} = '3.0.0';
	}
}

sub read_PSA_limits
{
	my %params = (
		'lim_cl'		=> 'CLIENTS',
		'lim_dom'		=> 'DOMAINS',
		'plesk_key_id'	=> 'PRODUCT_ID',
		'lim_mn'		=> 'MAILNAMES',
		'lim_wu'		=> 'WEBUSERS'
	);

	unless (open (fd, $conf{'PRODUCT_ROOT_D'}.'/bin/keyinfo.sh -l |')) {
		printf_stderr("Unable to execute '%s': %s\n", $conf{'PRODUCT_ROOT_D'}.'/bin/keyinfo.sh', $!);
		return '';
	}

	while (<fd>) {
		chomp($_);
		my ($key_param, $key_param_value) = split /:\s*/, $_;
		next unless exists $params{$key_param};
		if ($params{$key_param} ne 'PRODUCT_ID') {
			$conf{$params{$key_param} . '_LIMIT'} = ($key_param_value + 0);
			$conf{$params{$key_param} . '_USAGE'} = 0;
		} else {
			$conf{$params{$key_param}} = $key_param_value;
		}
	}

	close fd;
}


sub save_conf()
{
	my $tmp_cf = $const{PSA_CONF_FILE} . 'tmp';
	open (fd, ">$tmp_cf") || return '';

	my $cf = $const{PSA_CONF_FILE};
	open (FCONF, "<$cf") || return '';

	while (<FCONF>) {
		if (m/^\s*\#/) {
			print fd $_;
			next;
		}

		if (m/^\s*(\w+)\s+(.+?)\s*(|\#.*)$/) {
			my $new_value = $conf{$1};
			s/^(\s*\w+\s+).+?(\s*(|\#.*))$/\1$new_value\2/;
		}
		print fd $_;
	}

	close FCONF;
	close fd;

	system("mv -f $tmp_cf $cf");
}


sub  read_psa_passwd()
{
	return $conf{ADMIN_PASSWD} if $conf{ADMIN_PASSWD};

	my $passwd = $const{'PSA_SHADOW_FILE'};

	open(PASSWD,"< $passwd") || warn_log "Unable to read shadow file. The cause: %!\n","$passwd\n";
	$_=<PASSWD>;
	chomp;
	$conf{ADMIN_PASSWD} = $_;
	close PASSWD;

	return $conf{ADMIN_PASSWD};
}


sub checkIntegrity()
{
	unless (geteuid() == 0) {
		die_log "You must have a root permissions for execution of this utility.\n";
	}

	unless ( -d $const{PSA_CONF_D}) {
		die_log "PSA is not installed: the directory %s does not exist.\n", $const{PSA_CONF_D}."\n";
	}
}

sub checkPSARootDir()
{
	unless (-d $conf{PRODUCT_ROOT_D}) {
		die_log "PSA is not installed: the directory %s does not exist.\n", $conf{PRODUCT_ROOT_D}."\n";
	}

	my $version_file = $conf{PRODUCT_ROOT_D} . '/version';

	unless (-e $version_file) {
		die_log "Error: unable to read %s file.\n", "$version_file\n";
	}

	open fd, $version_file || die_log "Error: %!\n";
	my $str = <fd>;
	close fd || die_log "An error occured during %s file reading.\n", "$version_file\n";

	if  ($str =~ m/^([0-9]\.[0-9]\.[0-9])\s+[^\s]+\s+[^\s]+\s+([0-9]+\.[0-9]+)$/) {
		$conf{PSA_VERSION} = $1;
		$conf{PSA_BUILD} = $2;
	} else {
		die_log "Error: unable to parse %s file.\n", "$version_file\n";
	}
}


sub printPASTitle()
{
	return 1 unless $flags{SHOW_PROCESSING};

	my $versionStr = '';

	if ($const{MINOR_SUPPORTED_PSA_VERSION} ne $const{MAJOR_SUPPORTED_PSA_VERSION}) {
		$versionStr = $const{MINOR_SUPPORTED_PSA_VERSION} . '-' . $const{MAJOR_SUPPORTED_PSA_VERSION};
	} else {
		$versionStr = $const{PSA_VERSION};
	}

	print_stderr ($const{BU_NAME} . ' for ' . $const{'PRODUCT_NAME'} . ' v' . $versionStr . " (C) SWsoft, Inc.\n");
	print_stderr ("Build: " . $const{'BUILD_NAME'} . "\n");
	print_stderr ("Operating System: " . $const{OS});
	if ($const{OS_VERSION}) {
		print_stderr (' ' . $const{OS_VERSION});
	}
	print_stderr ("\n\n");

	return 1;
}

checkIntegrity();
read_psa_passwd();
read_conf();
define_component_versions();
read_PSA_limits();
checkPSARootDir();

$psa_xml_template = '<?xml version="1.0"?><psa version="'.$conf{PSA_VERSION}.'"></psa>';

return 1;
