###################################################################
package BU::PSA::Objects::ClientLimits;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::DB;

use BU::PSA::Objects::Limits;

@ISA = qw(BU::PSA::Objects::Limits);

sub init
{
	my $self = shift @_;
	my ($clientId, $limitsId) = @_;

	$self->SUPER::init($limitsId);

	$$self{clientId} = $clientId;

	return $self;
}


sub getSummaryDomainsLimit($)
{
	my ($self, $limit) = @_;
	$self->resetError();

	my $ret = 0;

	if ($limit =~ /^(disk_space|max_traffic|max_box|max_maillists|mbox_quota|max_redir|max_mg|max_resp|max_wu|max_db|max_webapps)$/) {
		my $query_str = 'SELECT min(l.value), max(l.value), sum(l.value), count(*) FROM Limits l, domains d'
					.	' WHERE d.limits_id=l.id AND l.limit_name=' . dbQuote($limit)
					.	' AND d.cl_id=' . dbQuote($$self{clientId});

		my $query = dbDoQuery($query_str);
		my ($min, $max, $sum, $count) = $query->fetchrow_array();
		if ($count <= 0) {
			$ret = 0;
		} elsif ($min < 0) {
			$ret = -1;
		} elsif ($limit eq 'mbox_quota') { ### mbox_quota is not additive value
			$ret = $max;
		} else {
			$ret = $sum;
		}
	} elsif ($limit eq 'max_dom') {
		my $query = dbDoQuery('select COUNT(*) from domains where cl_id=' . dbQuote($$self{clientId}));
		unless ($query) {
			$self->setError($self . ': unable to define number of domains.');
			return undef;
		}
		$ret = $query->fetchrow_array();
	} elsif ($limit eq 'expiration') {
		my $query_str = 'select d.id, L.value from domains d left join Limits L on (d.limits_id = L.id and L.limit_name="expiration") where d.cl_id=' . dbQuote($$self{clientId});
		my $query = dbDoQuery($query_str);
		unless ($query) {
			$self->setError('Unable to fetch maximum expiration date.');
			return undef;
		}

		while (my ($dom_id, $exp_date) = $query->fetchrow_array()) {
			if (($exp_date == -1) || ($exp_date == 0)) {
				$ret = -1;
			}
		}

		my $where_clause = ' L.id=d.limits_id and L.limit_name="expiration" and d.cl_id=' . dbQuote($$self{clientId});
		$query_str = 'select MAX(L.value) from Limits L, domains d where ' . $where_clause;
		$query = dbDoQuery($query_str);
		unless ($query) {
			$this->setError('Unable to fetch maximum expiration date.');
			return undef;
		}

		my ($expiration) = $query->fetchrow_array();
		$ret = $expiration;
	} else {
		$ret = undef;
	}

	return $ret;
}

return 1;

