###################################################################
package BU::PSA::Objects::Database;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;

use BU::PSA::Objects::Object;

@ISA = qw(BU::PSA::Objects::Object);


sub init($ $ $)
{
	my $self = shift @_;
	my ($domainId, $name, $type) = @_;

	my $fields = $self->{dbFields};

	defined($domainId) and $fields->{dom_id} = $domainId;# or croak "Argument 'domainId' is undefined.";
	defined($name) and $fields->{name} = $name or croak "Argument 'name' is undefined.";
	defined($type) and $fields->{type} = $type or croak "Argument 'type' is undefined.";

	$self->{dbFieldsNames} = { 'name' => 1, 'type' => '', 'dom_id' => 1 };

	$self->{dbTable} = 'data_bases';
	$self->{Changeable} = '';

	$self->SUPER::init(@_);

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return sprintf('type=%s AND lower(name)=lower(%s)', dbQuote($fields->{type}) , dbQuote($fields->{name}));
}


sub dbExistsCondition()
{
	my ($self) = @_;

	return $self->dbGetIdCondition();
}


sub doCheckExists()
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	if ($$fields{type} eq 'mysql') {
		return existDir($conf{MYSQL_VAR_D} . '/' . $$fields{'name'});
	} elsif ($$fields{type} eq 'postgresql') {
		my $res = checkPostgresDBExisting($$fields{'name'});
		if ($res < 0) {
			$self->setError("Database %s restoration has been skipped: error checking database '%s' existing.\n", $$fields{'name'});
			return '';
		}
		if ($res > 0) {
			return 1;
		}
	}

	return '';
}


sub doCreate
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	$self->resetError();

	if ($$fields{type} eq 'mysql') {
		unless ($self->{LockSysChanges}) {
			return dbDoQuery('create database `' . $$fields{'name'} . '`');
		}
	} elsif (($$fields{type} eq 'postgresql') && $conf{PG_IS_RUNNING}) {
		unless ($self->{LockSysChanges}) {
			my $buff = sprintf('umask 022; export PGPASSWORD=%s; %s/psql -U%s -tc "CREATE DATABASE \"%s\"" template1 2>&1 >/dev/null',
							shellArgQuote($conf{'postgresql_admin_passwd'}), $conf{PGSQL_BIN_D},
							$conf{'postgresql_admin_login'}, $$fields{'name'});

			if (system($buff) != 0) {
				$self->setError("Error during creating database '" . $$fields{'name'} . "'.\n");
				return '';
			}
			$buff = sprintf('umask 0022; %s/pg_manage add_db %s',
							$conf{UTILS_DIR}, dbQuote($$fields{'name'}));
			if (system($buff) != 0) {
				$self->setError('Error during pg_manage executing.');
				return '';
			}
		}
	}

	return 1;
}


return 1;

