###################################################################
package BU::PSA::Objects::Hosting; # psa_3.0
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::DB;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::Restore::IPMapping;
use BU::PSA::Objects::Object;
use BU::PSA::Objects::SysUser;
use BU::PSA::Objects::DNSRec;

@ISA = qw(BU::PSA::Objects::Object);


sub init($ $ $ $; $)
{
	my $self = shift @_;
	my ($domainId, $domainName, $domainDisplayName, $ipAddress, $login, $passwd, $passwd_type, $home, $shell, $quota) = @_;

	my $fields = $self->{dbFields};

	defined($domainId) and $self->{dbId} = $domainId and $fields->{dom_id} = $domainId
						or croak "Argument 'domainId' is undefined.";
	defined($domainName) and $self->{domainName} = $domainName or croak "Argument 'domainName' is undefined.";
	defined($domainDisplayName) and $self->{domainDisplayName} = $domainDisplayName or croak "Argument 'domainDisplayName' is undefined.";
	defined($ipAddress) and $self->{ip_address} = $ipAddress or croak "Unable to create hosting for domain '$domainName': argument 'ipAddress' is undefined.";

	$self->{sysUser} = new BU::PSA::Objects::SysUser($login, $passwd, $passwd_type, $home, $shell, $quota);

	if ($self->{sysUser}->error()) {
		$self->setError($self->{sysUser}->getErrorMsg());
		return undef;
	}

	$self->{dbFieldsNames} = {
								'dom_id' => 1, 'sys_user_id' => 1, 'ip_address_id' => 1,
								'real_traffic' => 0,
								'fp' => 0, 'fp_enable' => 0, 'fp_adm' => 0, 'fp_pass' => 0,
								'ssi' => 0, 'php' => 0, 'cgi' => 0, 'perl' => 0, 'fp_ssl' => 0,
								'asp' => 0, 'python' => 0,
								'ssl' => 0, 'same_ssl' => 0,
								'rootchain' => 0, 'webstat' => 0,
								'coldfusion' => 0,
								'at_domains' => 0
							};

	$$self{action_log_map} = {
					'fp' => COMP_FP_SUPPORT,
					'fp_ssl' => COMP_FP_SSL_SUPPORT,
					'fp_enable' => COMP_FP_AUTHORIZATION,
					'fp_adm' => COMP_FP_ADMIN_LOGIN,
					'fp_pass' => COMP_FP_ADMIN_PASSWD,
					'ssi' => COMP_SSI_SUPPORT,
					'php' => COMP_PHP_SUPPORT,
					'cgi' => COMP_CGI_SUPPORT,
					'perl' => COMP_PERL_SUPPORT,
					'asp' => COMP_ASP_SUPPORT,
					'python' => COMP_PYTHON_SUPPORT,
					'ssl' => COMP_SSL_SUPPORT,
					'coldfusion' => COMP_COLDFUSION_SUPPORT,
				};

	$self->{dbTable} = 'hosting';
	$self->{dbIdName} = 'dom_id';

	$self->{Changeable} = 1;

	$self->SUPER::init(@_);

	return $self;
}


sub getChangedComponents
{
	my $self = shift @_;

	my $diff1 = $self->SUPER::getChangedComponents(@_);
	my $diff2 = $self->{sysUser}->getChangedComponents();

	my %diff = (%$diff1, %$diff2);

#print "DEBUG:Hosting\n";
#while (my ($component, $tmp) = each(%diff)) {
#	print "($component)\n";
#}

	return \%diff;
}


sub getDBId()
{
	return $self->{dbId};
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	return dbExistsCondition();
}


sub dbExistsCondition()
{
	my ($self) = @_;

	return 'dom_id=' . $self->{'dbId'};
}


sub setField($ $ $)
{
	my ($self, $fieldName, $fieldValue) = @_;

	if ($fieldName eq 'dom_id') {
		croak 'The value of field "dom_id" can not be changed because it is a key field.';
	}

	return $self->SUPER::setField($fieldName, $fieldValue);
}


#sub dbLoad()
#{
#	my ($self) = @_;
#	my $fields = $self->{dbFields};
#}


sub doCheckExists
{
	my ($self) = @_;
	my $result = '';

	return $self->utilExecAC('usermng', 'exist', $self->{sysUser}->getField('login'))
			&& $self->ExistHostingDir();
}


sub ExistHostingDir()
{
	my ($self) = @_;

	my $path = $self->{sysUser}->getField('home');

	return (-d "$path/httpdocs");
}


sub Store
{
	my $self = shift @_;

	$self->resetError();

	$self->{sysUser}->Store();

	if ($self->{sysUser}->error()) {
		$self->setError($self->{sysUser}->getErrorMsg());
		return undef;
	}

	$self->setField('sys_user_id', $self->{sysUser}->getDBId());

	$self->SUPER::Store(@_);
}


sub doCreate
{
	my ($self) = @_;

	$self->resetError();

	my $ftpUser = $self->{sysUser}->getField('login');
	unless ($self->utilExec('vhostmng', '--install-vhost', '--vhost-name=' . $self->{domainName},
							'--user-name=' . $ftpUser))  {
		$self->{sysUser}->Delete();
		$self->setError(&util_exec_errormsg());
		return '';
	}

	if ($self->{'ip_address'}) {
		my $dnsRec = new BU::PSA::Objects::DNSRec (	$self->getField('dom_id'),
												$self->{domainDisplayName} . '.',
												'A',
												$self->{'ip_address'}
											);

		$dnsRec->Store();

		if ($dnsRec->error()) {
			$self->setError('Unable to create DNS record: ' . $dnsRec->getErrorMsg());
			return '';
		}

		my $dnsRec = new BU::PSA::Objects::DNSRec (	$self->getField('dom_id'),
												$self->{'ip_address'},
												'PTR',
												$self->{domainDisplayName} . '.',
												24
											);
		$dnsRec->Store();
		if ($dnsRec->error()) {
			$self->setError('Unable to create DNS record: ' . $dnsRec->getErrorMsg());
			return '';
		}

		$self->utilExecAC('dnsmng', ('update', $self->{domainName}));
	}

	my %fp_services = ('fp' => '80', 'fp_ssl' => '443');
	my %fp_errors = ('fp' => undef, 'fp_ssl' => undef);

	while (my ($fp, $port) = each %fp_services) {
		if ($self->getField($fp) eq 'true') {
			my $res = undef;

			my %enviroment = ('PSA_PASSWORD' => $self->getField('fp_pass'));
			$res = $self->utilExecEnvAC('websrvmng', \%enviroment, ('--install-fp',
											'--vhost-name='.$self->{domainName},
											'--user-name=' . $self->{sysUser}->getField('login'),
											'--port='.$port,
											'--admin-name='.$self->getField('fp_adm')
									));
			unless ($res) {
				$fp_errors{$fp} = 'Unable to create FrontPage service: ' . util_exec_errormsg();
				$self->setField($fp, 'false');
				next;
			}

			# ENABLE/DISABLE ATHORIZATION OVER HTTP
			my $authorization = ($self->getField('fp_enable') eq 'true') ? '--enable-fp' : '--disable-fp';
			$res = $self->utilExecAC('websrvmng', ($authorization,
											'--vhost-name='.$self->{domainName},
											'--user-name='.$self->{sysUser}->getField('login'),
											'--port=80'));
			unless ($res) {
				$fp_errors{$fp} = 'Unable to turn '. ($self->getField('fp_enable') eq 'true' ? 'on' : 'off'). ' authorization for FrontPage service (' . $fp . '): ' . util_exec_errormsg();
				$self->setField('fp_enable', ($self->getField('fp_enable') eq 'true' ? 'false' : 'true'));
				next;
			}
		}
	}

	my $err = '';
	while (my ($fp, $port) = each %fp_services) {
		$err .= "$error\n" if $error;
	}
	if ($err) {
		$self->setError($err);
	}

	return 1;
}


sub doDelete
{
	my ($self) = @_;

	my $domId = $self->getField('dom_id');
	my $domName = $self->{domainName};
	my $query = undef;

	$self->resetError();

	$self->{sysUser}->Delete();

	if ($self->{sysUser}->error()) {
		$self->setError($self->{sysUser}->getErrorMsg());
		return undef;
	}

	unless (dbDoQuery('DELETE FROM dom_param WHERE param="apacheErrorDocs" AND dom_id=' . $domId))
	{
		$self->setError("Unable to delete domain parameters from 'dom_param' database table");
		return '';
	}

	my $res1 = dbDoQuery('DELETE FROM dns_recs WHERE type="A" AND dom_id=' . $domId
						. ' AND host="' . $$self{domainDisplayName} . '."');

	my $res2 = dbDoQuery('DELETE FROM dns_recs WHERE type="PTR" AND dom_id=' . $self->getField('dom_id')
						. ' AND val="' . $self->{domainDisplayName} . '."');

	unless ($res1 && $res2) {
		$self->setError('Unable to delete DNS records for domain(' . $self->{domainName} . ').');
		return '';
	}

	my $dnsRec = new BU::PSA::Objects::DNSRec (	$self->getField('dom_id'),
											$self->{domainDisplayName} . '.',
											'A',
											$self->{'ip_address'}
											);
	$dnsRec->Store();
	if ($dnsRec->error()) {
		$self->setError('Unable to create DNS record: ' . $dnsRec->getErrorMsg());
		return '';
	}

	my $dnsRec = new BU::PSA::Objects::DNSRec (	$self->getField('dom_id'),
											$self->{'ip_address'},
											'PTR',
											$self->{domainDisplayName} . '.',
											24
										);
	$dnsRec->Store();
	if ($dnsRec->error()) {
		$self->setError('Unable to create DNS record: ' . $dnsRec->getErrorMsg());
		return '';
	}

	$self->utilExecAC('dnsmng', ('update', $self->{domainName}));

	#delete anonymous ftp domain
#			$ftp_domain = new AnonFtp();
#			if (!($ftp_exists = $ftp_domain->initialize($dom_id, $dom_name))) {
#				$plesk_errormsg = lmsg('cmd_domains__unable_delete_anonymous_ftp', $plesk_errormsg);
#				sem_release($sem_id);
#				return false;
#			}
#			if (($ftp_exists != -1) && !$ftp_domain->destroyAnonFtp()) {
#				$plesk_errormsg = lmsg('cmd_domains__unable_delete_anonymous_ftp', $plesk_errormsg);
#				sem_release($sem_id);
#				return false;
#			}
#			if (!$ftp_domain->putConf()){
#				$plesk_erromsg = lmsg('cmd_domains__unable_delete_anonymous_ftp', $plesk_errormsg);
#				sem_release($sem_id);
#				return false;
#			}


	# delete webusers for this domain
	$query = dbDoQuery('SELECT username FROM web_users WHERE dom_id=' . $self->getField('dom_id'));
	unless ($query) {
		$self->setError('Unable to retrieve the list of webusers: ' . $DBI::error);
		return '';
	}
	while (my ($wuName) = $query->fetchrow_array) {
		my $wu = new BU::PSA::WebUser ($domId, $$self{domainName}, $wuName);
		unless ($wu) {
			$self->setError('Unable to create WebUser object.');
			return '';
		}

		$wu->Delete();

		if ($wu->error()) {
			$self->setError("Unable to delete webuser($wuName) for domain " . $$self{domainDisplayName});
			return '';
		}
	}


	# delete protected direectories for this domain
	$query = dbDoQuery('SELECT path, `ssl` FROM protected_dirs WHERE dom_id=' . $domId);
	unless ($query) {
		$self->setError('Unable to retrieve the list of protected directories: ' . $DBI::error);
		return '';
	}
	while(my ($pdPath, $pdSSL) = $query->fetchrow_array) {
		my $pDir = new BU::PSA::ProtectedDir ($domId, $domName, $pdPath, ($pdSSL eq 'true'));
		unless ($pDir) {
			$self->setError('Unable to create WebUser object.');
			return '';
		}

		$pDir->Delete();

		if ($pDir->error()) {
			$self->setError("Unable to delete protected directory($pdPath) for domain $domName.");
			return '';
		}
	}


	# delete virtual hosting and fp if exists
	$query = dbDoQuery('SELECT login, fp, fp_ssl FROM hosting WHERE dom_id=' . $domId);
	unless ($query) {
		$self->setError('Unable to retrieve FrontPage information: ' . $DBI::error);
		return '';
	}
	if (my ($login, $fp, $fp_ssl) = $query->fetchrow_array) {

		# delete fp
		if ($fp eq 'true') {
			if ($self->utilExec('websrvmng', ('--remove-fp', "--vhost-name=$domName", "--user-name=$login", '--port=80'))) {
				$self->setError('Unable to delete FrontPage from the system.');# . $ExecError);
				return '';
			}
		}
		if ($fp_ssl eq 'true') {
			if ($self->utilExec('websrvmng', ('--remove-fp', "--vhost-name=$domName", "--user-name=$login", '--port=443'))) {
				$self->setError('Unable to delete FrontPage from the system.');# . $ExecError);
				return '';
			}
		}

		# delete virtual hosting
		unless ($self->utilExec('rm_vrt', ($domName, $login))) {
			$self->setError('Unable to delete physical hosting.');
			return '';
		}
	}


	# delete statistics
	unless (dbDoQuery('DELETE FROM stat WHERE dom_id=' . $domId)) {
		$self->setError('Unable to delete statistics: ' . $DBI::error);
		return '';
	}

	unless (dbDoQuery('UPDATE domains SET htype="none" WHERE id=' . $domId)) {
		$self->setError('Unable to set hosting type on domain to "none": ' . $DBI::error);
		return '';
	}

	return 1;
}


return 1;

