###################################################################
package BU::PSA::Objects::MailName;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Logger;
use BU::PSA::Const;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::MailAutoresponder;

use BU::PSA::Objects::Object;
@ISA = qw(BU::PSA::Objects::Object);


sub init($ $ $; $)
{
	my $self = shift @_;
	my ($domainId, $domainName, $mailName, $password_type, $password) = @_;

	my $fields = $self->{dbFields};

	defined($domainId) and $fields->{dom_id} = $domainId or croak "Argument 'domainId' is undefined.";
	defined($domainName) and $self->{domainName} = $domainName or croak "Argument 'domainName' is undefined.";
	defined($mailName) and $fields->{mail_name} = $mailName or croak "Argument 'mailName' is undefined.";

	$self->{dbFieldsNames} = {
								'dom_id' => 1, 'mail_name' => 1, 'account_id' => 0,
								'mail_name' => '', 'postbox' => '', 'cp_access' => '',
								'redirect' => '', 'redir_addr' => '', 'mail_group' => '',
								'autoresponder' => '', 'mbox_quota' => ''
							};

	$self->{dbTable} = 'mail';
	$self->{Changeable} = 1;

	$self->SUPER::init(@_);

	my $account_id = $self->getField('account_id');
	$self->{objAccount} = new BU::PSA::Objects::Account($account_id ? $account_id : 0);
	$self->{objAccount}->setField('type', $password_type);
	$self->{objAccount}->setField('password', $password);

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'mail_name=' . dbQuote($fields->{mail_name})
		.  ' AND dom_id=' . $fields->{dom_id};
}


sub dbExistsCondition()
{
	my ($self) = @_;

	return $self->dbGetIdCondition();
}


sub doCheckExists
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return existDir($conf{QMAIL_ROOT_D} . '/mailnames/' . $self->{domainName} 
					. '/' . $fields->{mail_name} . '/Maildir');
}

sub Store
{
	my $self = shift @_;

	$self->resetError();

	$self->{objAccount}->Store();

	if ($self->{objAccount}->error()) {
		$self->setError($self->{objAccount}->getErrorMsg());
		return undef;
	}

	$self->setField('account_id', $self->{objAccount}->getDBId());
	$self->SUPER::Store(@_);
}

sub doCreate
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	$self->resetError();

	# create mailname
	unless ($self->utilExecAC('mailmng', ('--add-mailname',
										'--domain-name=' . $self->{domainName},
										'--mailname=' . $fields->{mail_name}
										))) {
		$self->setError('Unable to create mailname.');
		return '';
	}

	# create mailbox
	if ($fields->{postbox} eq 'true') {
		my %env = ();
		my $password = $self->{objAccount}->getField('password');
		my $password_type = $self->{objAccount}->getField('type');

		if ($password_type eq 'plain') {
			$env{PSA_PASSWORD} = $password;
		} else {
			$env{PSA_CRYPTED_PASSWORD} = $password;
		}

		unless (util_exec_env('mailmng', \%env, ('--add-mailbox',
										'--domain-name=' . $self->{domainName},
										'--mailname=' . $fields->{mail_name}
										)))
		{
			$self->setError('Unable to create mailbox: ' . $ExecError);
		}
	}

	# create forward
	if ($fields->{redirect} eq 'true') {
		unless ($self->utilExecAC('mailmng', ('--add-redirect',
										'--domain-name=' . $self->{domainName},
										'--mailname=' . $fields->{mail_name},
										'--redirect-email=' . $fields->{redir_addr})))
		{
			$self->setError('Unable to create mailbox: ' . $ExecError);
		}
	}

	# set mailbox quota
	printf_stderr("mbox quota: %s\n", $fields->{mbox_quota}) if $debug > 0;

	if (exists($fields->{mbox_quota}) && $fields->{mbox_quota} && ($fields->{mbox_quota}+0 >= 0)) {
		unless ($self->utilExecAC('mailmng', ('--set-mailbox-disk-quota', 
										'--domain-name=' . $self->{domainName},
										'--mailname=' . $fields->{mail_name},
										'--disk-quota=' . ($fields->{mbox_quota}+0)
										)))
		{
			$self->setError('Unable to set mailbox quota: ' . util_exec_errormsg());
		}
	}

	return 1;
}


sub addAutoResp($ $ $ $ $ $ $ $ $)
{
	my ($self, $respName, $turn_on, $keyString, $keyWhere, $subject, $respText, $ansFreq, $memLimit, $replyTo) = @_;

	unless ($self->dbExists()) {
		$self->setError('Unable to add autoresponder because MailName must be stored previously.');
		return '';
	}

	my $resp = new BU::PSA::Objects::MailAutoresponder($self->getDBId(), $respName);

	unless ($resp) {
		$self->setError('Unable to create object MailAutoresponder(' . $respName . ').');
		return '';
	}

	if ($turn_on) {
		$resp->setField('resp_on', 'true');
	} else {
		$resp->setField('resp_on', 'false');
	}

	$resp->setField('keystr', $keyString);
	$resp->setField('key_where', $keyWhere);
	$resp->setField('subject', $subject);
	$resp->setField('text', $respText);
	$resp->setField('ans_freq', $ansFreq);
	$resp->setField('mem_limit', $memLimit);
	$resp->setField('reply_to', $replyTo);

	if ($resp->error()) {
		$self->setError('Unable to create mail autoresponder(' . $respName . '): ' . $resp->getErrorMsg());
		return '';
	}

	$resp->Store();

	if ($resp->error()) {
		$self->setError('Unable to create mail autoresponder(' . $respName . '): ' . $resp->getErrorMsg());
		return '';
	}

	if ($turn_on && ($self->getField('autoresponder') eq 'true')) {

		my $res = $self->utilExecAC('mailmng', ('--set-autoresponder',
										'--domain-name=' . $self->{domainName},
										'--mailname=' . $self->getField('mail_name')
									));

		unless ($res) {
			$resp->setField('resp_on', 'false');
			$resp->Store();
			$self->setError('Unable to turn autoresponder(' . $respName . ') on.');
			return '';
		}
	}

	return $resp;
}


sub addForward($ $)
{
	my ($self, $address, $on) = @_;

	unless (dbDoQuery('REPLACE INTO mail_redir(mn_id, address) '
					. 'VALUES(' . $self->getDBId() . ', ' . dbQuote($address) . ')')) {

		$self->setError('Unable to set mail forward:\n ' . $DBI::error);
		return '';
	}

	if ($on) {
		unless ($self->utilExecAC('mailmng', ('--add-redirect',
									'--domain-name=' . $self->{domainName},
									'--mailname=' . $self->getField('mail_name'),
									'--redirect-email=' . $address)))
		{
			$self->setError('Unable to create forward: ' . util_exec_errormsg());
		}
	}

	return 1;
}


return 1;

