###################################################################
package BU::PSA::Objects::Object;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Logger;
use BU::PSA::Const;
use BU::PSA::DB;
use BU::PSA::CommonFunc;
use BU::PSA::ActionLog;

#@ISA = qw(Exporter);
@ISA = qw();

%validMODEs = ('PSA' => '', 'BACKUP_RESTORE' => '');
$MODE = undef;


sub new()
{
	unless (defined($MODE)) {
		croak "ERROR Object.pm: undefined MODE value.\n";
	}

	unless (exists($validMODEs{$MODE})) {
		croak "ERROR Object.pm: unknown MODE value $MODE.\n";
	}

	my $class = shift @_;
	my $self = {};

	print_stdout ("NEW Object $self\n") if $debug > 7;

	$self->{error} = '';
	$self->{errorMsg} = '';

	$self->{Changeable} = '';
	$self->{LockSysChanges} = '';

	unless (exists($self->{dbIdName})) {
		$self->{dbIdName} = 'id';
	}

	# do not define this hash variable!  ->        $self->{dbId};
	$self->{dbExists} = '';
	$self->{dbChecked} = '';
	$self->{dbSynced} = '';
	$self->{dbFields} = {};
	$self->{enable} = 1;

	my @RequiredFields = ('dbTable', 'dbFieldsNames');
	$self->{RequiredFields} = \@RequiredFields;

	bless($self, $class);

	$self->init(@_);

	return $self;
}


sub init()
{
	my $self = shift @_;

#print_stdout ("init Object\n");

	my $RequiredFields = $self->{RequiredFields};

	# BEGIN: the next fields must be set
	foreach my $requireField (@$RequiredFields) {
#print_stdout ("check for REQUIRED field: '$requireField'.\n");
		unless (exists($self->{$requireField})) {
			#setError("Field '$requireField' must be set.");
			croak "Error occured during initialization of $self class: field '$requireField' must be set.\n";
			return '';
		}
	}
	# END

	# BEGIN: the is test calls
#	$self->dbGetIdCondition();
#	$self->dbExistsCondition();
	# END

	if ($self->dbExists()) {
		$self->getDBId();
		$self->Load(1);
	}

	my $fields = $$self{dbFields};
	my %old_fields = %$fields;

#print "DEBUG old value saving:\n";

#while (my ($field, $value) = each(%old_fields)) {
#	print "DEBUG: ($field,$value)\n";
#}
	$$self{'dbFieldsOld'} = \%old_fields;

	return $self;
}


sub getChangedComponents($)
{
	my ($self) = @_;

	my %diff = ();

#print "DEBUG (getChangedComponents)\n";

	return undef unless exists($$self{'action_log_map'});

	my $fields = $$self{dbFields};
	my $fields_old = $$self{dbFieldsOld};
	my $action_log_map = $$self{'action_log_map'};
	
	while (my ($field, $action_log_component) = each(%$action_log_map)) {
		my $value = $$fields{$field};
		my $value_old = $$fields_old{$field};
#print "DEBUG (diff): $field, $action_log_component, ($value_old) => ($value) ...";
		if ("$value_old" ne "$value") {
			$diff{$action_log_component} = [($value_old, $value)];
#print " changed";
		}
#print "\n";
	}

	return \%diff;
}


sub logChanges($$)
{
	my ($self, $object_id, $action) = @_;

#print "DEBUG (Object::action): $action\n";

	my $changed_components = $self->getChangedComponents();

	logAction($object_id, $action, $changed_components);
}


sub dbGetIdCondition()
{
	croak ('This method(dbGetIdCondition) must be overwritten in children class!' . "\n");
}


sub dbExistsCondition()
{
	croak ('This method(dbExistsCondition) must be overwritten in children class!' . "\n");
}


sub resetError($)
{
	my ($self) = @_;

	$self->{error} = '';
	$self->{errorMsg} = '';
}


sub setError($)
{
	my ($self, $errorMsg) = @_;

	$self->{error} = 1;
	$self->{errorMsg} = $errorMsg;
}


sub appendErrorMsg($)
{
	my ($self, $errorMsg) = @_;

	$self->{error} = 1;
	$self->{errorMsg} .= "\n" . $errorMsg;
}


sub getErrorMsg()
{
	my ($self) = @_;

	return $self->{errorMsg};
}


sub error()
{
	my ($self) = @_;

	return $self->{error};
}


sub Exists()
{
	my ($self) = @_;

	if ($MODE eq 'PSA') {

		return $self->dbExists() || $self->doCheckExists();

	} elsif ($MODE eq 'BACKUP_RESTORE') {

		return $self->dbExists() && $self->doCheckExists();

	}

	croak "ERROR Exists(): unknown MODE.";
}

sub dbExists()
{
	my ($self) = @_;

	if (exists($self->{dbExists}) && $self->{dbExists}) {
		return 1;
	}

	if ($self->{dbChecked}) {
		return '';
	}

	$self->{dbChecked} = 1;

	my $query = dbDoQuery('SELECT ' . $self->{dbIdName} . ' FROM ' . $self->{dbTable}
							. ' WHERE ' . $self->dbExistsCondition());

	if (my ($dbId) = $query->fetchrow_array) {
		$self->{dbId} = $dbId;
		$self->{dbExists} = 1;
		return 1;
	}

	return '';
}


sub setDBId
{
	my ($self, $dbId) = @_;

	$self->{dbId} = $dbId;

	$self->{dbExists} = undef;
	$self->{dbChecked} = undef;

	if ($self->Exists()) {
		$self->Load();
	}

	return 1;
}


sub getDBId()
{
	my ($self) = @_;

	if (exists($self->{dbId})) {
		return $self->{dbId};
	}

	my $query = dbDoQuery('SELECT ' . $self->{dbIdName} . ' FROM ' . $self->{dbTable} .
							' WHERE ' . $self->dbGetIdCondition());

	if (my ($dbId) = $query->fetchrow_array) {
		$self->{dbId} = $dbId;
		return $dbId;
	}

	return '';
}


sub setField($ $)
{
	my ($self, $fieldName, $fieldValue) = @_;

	printf_stdout("$self ::Object::setField(%s,%s)\n", $fieldName, $fieldValue) if $debug > 7;

	unless (exists($self->{dbFieldsNames}->{$fieldName})) {
		$self->setError('In "' . $self . '" object: attempting to set value for unknown field('
						. $fieldName . ').');
		return '';
	}

	my $fields = $self->{dbFields};

	if (exists($fields->{$fieldName}) && ($fields->{$fieldName} eq $fieldValue)) {
		return 1;
	}

	printf_stdout("$self ::Object::setField(%s,%s) result: changed.\n", $fieldName, $fieldValue) if $debug > 7;

	$fields->{$fieldName} = $fieldValue;
	$self->{dbSynced} = '';
	return 1;
}


sub getField($)
{
	my ($self, $fieldName) = @_;

	unless (exists($self->{dbFieldsNames}->{$fieldName})) {
		$self->setError('In "' . $self . '" object: attempting to get value for unknown field('
						. $fieldName . ').');
		return '';
	}

	my $fields = $self->{dbFields};

#	if (exists($fields->{$fieldName})) {
#		$self->dbLoad();
#	}

	return $fields->{$fieldName};
}


sub Store()
{
	my $self = shift @_;

	$self->resetError();

	my $exists = '';

	if ($MODE eq 'PSA') {

		if ($self->{dbSynced}) {
			return 1;
		}

		$exists = $self->dbExists() || $self->doCheckExists();

		if ($exists && not $self->{Changeable}) {
			$self->setError('This object already exists and cannot be changed.');
			return '';
		}

		unless ($self->dbStore()) {
			return '';
		}

		if (not $exists) {
			unless ($self->doCreate()) {
				$self->dbDelete();
				return '';
			}
		} else {
			$self->resetError();

			unless ($self->doUpdate()) {
				$self->dbStore();
				return '';
			}

			if ($self->error()) {
				$self->dbStore();
			}
		}
	} elsif ($MODE eq 'BACKUP_RESTORE') {

		if ($self->doCheckExists()) {
			$self->LockSysChanges();
		} else {
			$self->UnlockSysChanges();
		}

		printf_stdout ("$self ::Object::Store() {Changeable:%s}\n", $self->{Changeable}) if $debug > 7;

		my $alreadyExists = $self->dbExists();

		if (!$alreadyExists || $self->{Changeable}) {
			$self->dbStore();

			if ($self->error()) {
				return undef;
			}
		}

		unless ($self->doCreate()) {
			$self->setError("Unable to restore object $self.") unless $self->error();

			if (!$alreadyExists && $self->dbExists()) {
				$self->dbDelete();
			}

			return '';
		}

	}

	return 1;
}


sub Delete
{
	my $self = shift @_;

	$self->resetError();

	unless ($self->doDelete()) {
		return '';
	}

	unless ($self->dbDelete()) {
		$self->appendErrorMsg('Unable to delete object from database.');
		return '';
	}

	return 1;
}


sub Load()
{
	my $self = shift @_;

	$self->resetError();

	return $self->dbLoad(@_);
}


sub dbStore()
{
	my ($self) = @_;

	printf_stdout ("$self ::Object::dbStore() {dbSynced:%s}\n", $self->{dbSynced}) if $debug > 7;

	if ($self->{dbSynced}) {
		return 1;
	}

	$self->resetError();

	my $dbFieldsNames = $self->{dbFieldsNames};

	if ($self->dbExists() && exists($dbFieldsNames->{$self->{dbIdName}})) {

		# copy dbFields hash and create a reference on it
		my %dbFieldsNamesCopy = %$dbFieldsNames;
		$dbFieldsNames = \%dbFieldsNamesCopy;

		# it is excluding of the 'dbId' field from the 'dbFieldsnames' list
		# for a 'REPLACE' query because it will be used automatic
		delete($dbFieldsNames->{$self->{dbIdName}});
	}

	my $fields = $self->{dbFields};

	my $fieldsList = '';
	my $valuesList = '';
	my $coma = '';

	foreach my $fieldName (keys %$dbFieldsNames) { 
		if ($dbFieldsNames->{$fieldName} eq 'DATE') {
			if ((not exists($fields->{$fieldName})) || ($fields->{$fieldName} eq '')) {
				$fieldsList .= $coma . (($fieldName eq 'ssl') ? '`ssl`' : $fieldName);
				$valuesList .= $coma . 'CURRENT_DATE';
				$coma = ', ';
			} else {
				$fieldsList .= $coma . (($fieldName eq 'ssl') ? '`ssl`' : $fieldName);
				$valuesList .= $coma . dbQuote($fields->{$fieldName});
				$coma = ', ';
			}
		} elsif (($dbFieldsNames->{$fieldName} eq '1') && not exists($fields->{$fieldName})) {
			$self->setError("Key field '$self::$fieldName' is not specified.");
			return '';
		} elsif (exists($fields->{$fieldName})) {
			$fieldsList .= $coma . (($fieldName eq 'ssl') ? '`ssl`' : $fieldName);
			$valuesList .= $coma . dbQuote($fields->{$fieldName});
			$coma = ', ';
		}
	}

	if ($self->dbExists()) {
		unless (dbDoQuery('REPLACE INTO ' . $self->{dbTable} . '(' . $self->{dbIdName}
						. ", $fieldsList) VALUES(" . $self->{dbId} . ", $valuesList)")) {
			$self->setError('Unable to exec "REPLACE" query. ' . $DBI::errstr);
			return '';
		}
	} else {
		unless (dbDoQuery('INSERT INTO ' . $self->{dbTable} . "($fieldsList) VALUES($valuesList)")) {
			$self->setError('Unable to exec "INSERT" query. ' . $DBI::errstr);
			return '';
		}
#		$self->getDBId();
		$self->{dbId} = dbGetInsertId();
	}

	$self->{dbExists} = 1;
	return 1;
}


sub dbDelete
{
	my ($self, ) = @_;

	unless ($self->dbExists()) {
		return 1;
	}

	unless (dbDoQuery('DELETE FROM ' . $self->{dbTable}
						. ' WHERE ' . $self->{dbIdName} . '=' . $self->getDBId())) {
		#$self->setError('');
		return '';
	}

	$self->{dbSynced} = 1;
	$self->{dbExists} = '';

	return 1;
}

sub dbLoad
{
	my ($self, $merge) = @_;

	if (not $self->dbExists()) {
		return '';
	}

	unless (defined($merge)) { $merge = 1; }

	$query = dbDoQuery('SELECT * FROM ' . $self->{dbTable}
						. ' WHERE ' . $self->{dbIdName} . '=' . $self->{dbId});

	$self->{dbSynced} = 1;

	my %fieldsFromDB = ();
	$fieldsFromDB = $query->fetchrow_hashref;

	my $fields = $self->{dbFields};

	if ($merge) {
		while (my ($key, $value) = each(%$fieldsFromDB)) {
			if (exists($fields->{$key})) {
				unless ($value eq $fields->{$key}) { $self->{dbSynced} = ''; }
			} else {
				$fields->{$key} = $value;
			}
		}
	} else {
		%$fields = $fieldsFromDB;
	}
}


sub doCheckExists
{
	return '';
}


sub doCreate
{
	return 1;
}


sub doUpdate
{
	return 1;
}


sub doDelete
{
	my ($self) = @_;

	if ($self->{Exists}) {
		
	}

	return 1;
}


sub LockSysChanges()
{
	my $self = shift @_;

	$self->{LockSysChanges} = 1;

	print_stdout("$self ::LockSysChanges\n") if $debug > 7;
}


sub UnlockSysChanges()
{
	my $self = shift @_;

	$self->{LockSysChanges} = undef;

	print_stdout("$self ::UnlockSysChanges\n") if $debug > 7;
}


sub utilExec
{
	my $self = shift @_;

	if ($self->{LockSysChanges}) {		
		return 1;
	}

	if (util_exec(@_)) {
		return 1;
	}

	if (util_exec_error()) {
		$self->setError(util_exec_errormsg());
	}

	return undef;
}


sub utilExecEnv
{
	my $self = shift @_;

	if ($self->{LockSysChanges}) {		
		return 1;
	}

	if (util_exec_env(@_)) {
		return 1;
	}

	if (util_exec_error()) {
		$self->setError(util_exec_errormsg());
	}

	return undef;
}


sub utilExecAC
{
	my $self = shift @_;

	return util_exec(@_);
}


sub utilExecEnvAC
{
	my $self = shift @_;

	if (util_exec_env(@_)) {
		return 1;
	}

	if (util_exec_error()) {
		$self->setError(util_exec_errormsg());
	}

	return undef;
}

sub enable()
{
	$self->{enable} = 1;
}


sub disable()
{
	$self->{enable} = '';
}

return 1;

