###################################################################
package BU::PSA::Objects::ProtectedDirUser;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::CommonFunc;
use BU::PSA::DB;

use BU::PSA::Objects::Object;

@ISA = qw(BU::PSA::Objects::Object);


sub init($ $ $ $)
{
	my $self = shift @_;
	my ($domainName, $pdirId, $path, $login, $passwd) = @_;

	my $fields = $self->{dbFields};

	defined($domainName) and $self->{domainName} = $domainName or croak "Argument 'domainName' is not defined.";
	defined($pdirId) and $fields->{pd_id} = $pdirId or croak "Argument 'pdirId' is not defined.";
	$self->{path} = $path; # path can be empty it means '/'
	defined($login) and $fields->{login} = $login or croak "Argument 'login' is not defined.";

	$self->{dbFieldsNames} = { 'pd_id' => 1, 'login' => 1, 'account_id' => 0};

	$self->{dbTable} = 'pd_users';
	$self->{Changeable} = '';

	$self->SUPER::init(@_);

	my $account_id = $self->getField('account_id');
	$self->{objAccount} = new BU::PSA::Objects::Account($account_id ? $account_id : 0);
	$self->{objAccount}->setField('type', 'plain');
	$self->{objAccount}->setField('password', $passwd);

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'login=' . dbQuote($fields->{login})
		.  ' and pd_id=' . $fields->{pd_id};
}


sub dbExistsCondition()
{
	my ($self) = @_;

	return $self->dbGetIdCondition();
}


sub doCheckExists
{
	my ($self) = @_;
	my $pwd_file_name = $self->{path};
	$pwd_file_name =~ s/\//../g;

	my $pwd_file = $conf{HTTPD_VHOSTS_D} . '/' . $self->{domainName} . "/pd/d..$pwd_file_name";

	return '' unless ( -e $pwd_file );

	unless (open fd, $pwd_file) {
		$self->setError("Unable to open pd users password file.\n");
		return '';
	}

	my $user_name = $self->{dbFields}->{login};

	while (<fd>) {
		if (m/^\s*$user_name\s*:/) {
			close fd;
			return 1;
		}
	}

	close fd;
	return '';
}


sub Store
{
	my $self = shift @_;

	$self->resetError();

	$self->{objAccount}->Store();

	if ($self->{objAccount}->error()) {
		$self->setError($self->{objAccount}->getErrorMsg());
		return undef;
	}

	$self->setField('account_id', $self->{objAccount}->getDBId());
	$self->SUPER::Store(@_);
}


sub doCreate
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	$self->resetError();

	my %env = (PSA_PASSWORD => crypt($self->{objAccount}->getField('password'), genSaltForCrypt()));

	unless ($self->utilExecEnv('pdmng', \%env, '--add-user', 
								'--vhost-name=' . $self->{domainName},
								'--directory=/' . $self->{path},
								'--user-name=' . $fields->{login}))
	{
		$self->setError('Unable to create Protected directory user is the system ('
						. $self->{path} . '/' . $$fields{login} . ')'
						. (util_exec_error() ? ': ' . util_exec_errormsg() : '.')
						);
		return '';
	}

	return 1;
}

return 1;

