package BU::PSA::Restore::CommonData;

require 5.004;
require Exporter;

use MIME::Base64 qw(encode_base64 decode_base64);
use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Restore::IPMapping;
use BU::XML::CommonFunc;

@ISA = qw(Exporter);
@EXPORT = qw(
restoreCommonData
restoreIPAddresses
restoreCertificates
checkCertExists
restoreReports
restoreCustomButtons
);


sub restoreCommonData
{
	my ($baseNode) = @_;

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		foreach my $node ($baseNode->xql('./skins/skin')) {
#			my $path = u2a($node->getAttribute('path'));
			$node->setAttribute('destdir', $conf{PRODUCT_ROOT_D} . "/admin/htdocs/skins");
        	hashXMLNodeByCID($node);
		}

		###################################################################
		#	Determine which certificates are belong to admin's repository
		###################################################################
		my %certIdsHash = ();
		foreach my $node ($baseNode->xql('./admin_certificates_list/*')) {
			$certIdsHash{$node->getAttribute('id')} = 1;
		}

		######################################################
		#	Mark admin's certificates with special attribute
		######################################################
		foreach my $node ($baseNode->xql('./certificates_list/*')) {
			if (exists($certIdsHash{$node->getAttribute('id')}) || ($conf{psaIsConfigured} && ($node->getAttribute('bu_cert') eq 'true'))) {
				$node->setAttribute('repository', 'ADMIN');

				# hash certificate keys nodes
				foreach my $keyNode ($node->xql('./*')) {
	        		hashXMLNodeByCID($keyNode);
				}
			}
		}

		# hash nodes for slave server certificates
		foreach my $node ($baseNode->xql('./slave_servers/server/params/certificate/*')) {
			hashXMLNodeByCID($node);
		}		
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {

		dbDoQuery('DELETE FROM Notes');
		dbDoQuery('DELETE FROM Notifications');
		my $notification_index = 1;										# fix of bug #26745
		foreach my $noteNode ($baseNode->xql('./notifications/message')) {
			my $note_id = $noteNode->getAttribute('id');
			my $textNode = $noteNode->getFirstChild();
			next unless $textNode;
			dbDoQuery('REPLACE INTO Notes(id, text) VALUES('.$note_id.', '.dbQuote(u2a($textNode->toString)).')');
	
			foreach my $descNode ($noteNode->xql('./desc')) {
				my $subj = '';
				if (my $subjNode = $descNode->getFirstChild()) {
					$subj = $subjNode->toString();
				}
				if (!$descNode->getAttribute('id')) {					# fix of bug #26745
					$descNode->setAttribute('id', $notification_index);	# fix of bug #26745
				}														# fix of bug #26745
				$notification_index++;									# fix of bug #26745
				my $buff = XMLAttrs2SQL("REPLACE INTO Notifications(note_id, subj, %s) VALUES('$note_id', '$subj', %s)", $descNode->getAttributes());
				dbDoQuery($buff);
			}
		}

		foreach my $node ($baseNode->xql('./skins/skin')) {
			my $name = u2a($node->getAttribute('name'));
			my $path = u2a($node->getAttribute('path'));
			my $dir = u2a($node->getAttribute('dir'));

			unless (-d $node->getAttribute('destdir') . "/$dir") {
				print_stderr("Skin '$name' restoration is skiped because the directory is not exists.\n");
				next;
			}

			my $path_dir = $path ? "$path/$dir" : $dir;
			my $query = dbDoQuery("SELECT id FROM Skins WHERE place=\"$path_dir\"");
			my $skin_id = undef;
			if (($skin_id) = $query->fetchrow_array()) {
				dbDoQuery("REPLACE INTO Skins(id, name, place) VALUES($skin_id, ". dbQuote($name) . ', ' . dbQuote("$path_dir") . ')');
			} else {
				dbDoQuery('INSERT INTO Skins(name, place) VALUES('. dbQuote($name) . ', ' . dbQuote("$path_dir") . ')');
				$skin_id = dbGetInsertId();
			}
			if ($skin_id) {
				if (($node->getAttribute('server_default_skin') eq 'true')  && (!$conf{psaIsConfigured} || $flags{'RESTORE_SERVER'})) {
					putMiscParam('def_skin_id', $skin_id);
				} elsif (($node->getAttribute('admin_skin') eq 'true') && (!$conf{psaIsConfigured} || $flags{'RESTORE_ADMIN'})) {
					putMiscParam('admin_skin_id', $skin_id);
				}
			}
		}

		# restore domain templates
		my $rep_id = getMiscParam('dom_tmpl_list_id');
		foreach my $node ($baseNode->xql('./domain_templates/domain')) {
			my $tpl_id = undef;
			my $query = undef;
			my $name = u2a($node->getAttribute('name'));

			if ($rep_id) {
				$query = dbDoQuery("SELECT tpl.id FROM Templates tpl, Repository r WHERE r.rep_id=$rep_id AND r.component_id=tpl.id AND name=" . dbQuote($name));
				unless (($tpl_id) = $query->fetchrow_array()) {
					$tpl_id = undef;
				}
			}
	
			if ($tpl_id) {
				dbDoQuery("REPLACE INTO Templates(id, name) VALUES($tpl_id, " . dbQuote($name) . ')');
			} else {
				dbDoQuery('INSERT INTO Templates(name) VALUES(' . dbQuote($name) . ')');
				$tpl_id = dbGetInsertId();
			}
	
			foreach my $sn ($node->xql('./*')) {
				my $field = $sn->getNodeName();
				my $value = undef;
				$sn->removeAttribute('id');
				if ($field eq 'log_rotation') {
					$field = 'logrotation_id';
					my $query = dbDoQuery('SELECT value FROM TmplData WHERE element="logrotation_id" AND tmpl_id=' . dbQuote($tpl_id));
					my ($logrot_id) = $query->fetchrow_array();
					if ($logrot_id) {
						my $buff = XMLAttrs2SQL('REPLACE INTO log_rotation(id, %s) VALUES('.$logrot_id.', %s)',
												$sn->getAttributes(), 1);
						dbDoQuery($buff);
						$value = $logrot_id;
					} else {
						my $buff = XMLAttrs2SQL('INSERT INTO log_rotation(%s) VALUES(%s)',
												$sn->getAttributes(), 1);
						dbDoQuery($buff);
						$value = dbGetInsertId();
					}
				} else {
					$value = $sn->getAttribute('value');
				}

				dbDoQuery('REPLACE INTO TmplData(tmpl_id, element, value) VALUES('.
							$tpl_id . ', "'. u2a($field) . '", "' . u2a($value) . '")');
			}
	
			if ($rep_id) {
				dbDoQuery("REPLACE INTO Repository(rep_id, component_id) VALUES($rep_id, $tpl_id)");
			} else {
				dbDoQuery("INSERT INTO Repository(component_id) VALUES($tpl_id)");
				$rep_id = dbGetInsertId();
				putMiscParam('dom_tmpl_list_id', $rep_id);
			}
		}

		#restore client templates
		$rep_id = getMiscParam('cl_tmpl_list_id');
		foreach my $node ($baseNode->xql('./client_templates/client')) {
			my $tpl_id = undef;
			my $query = undef;
			my $name = u2a($node->getAttribute('name'));

			if ($rep_id) {
				$query = dbDoQuery("SELECT tpl.id FROM Templates tpl, Repository r WHERE r.rep_id=$rep_id AND r.component_id=tpl.id AND name=" . dbQuote($name));
				unless (($tpl_id) = $query->fetchrow_array()) {
					$tpl_id = undef;
				}
			}
	
			if ($tpl_id) {
				dbDoQuery("REPLACE INTO Templates(id, name) VALUES($tpl_id, " . dbQuote($name) . ')');
			} else {
				dbDoQuery('INSERT INTO Templates(name) VALUES(' . dbQuote($name) . ')');
				$tpl_id = dbGetInsertId();
			}

			### restore template data
			foreach my $sn ($node->xql('./*')) {
				if ($sn->getNodeName() eq 'ip_pool') {
					my $pool_id = undef;
					if (my $query = dbDoQuery('SELECT value FROM TmplData WHERE element="tmpl_pool_id" and tmpl_id=' . dbQuote($tpl_id))) {
						if (($pool_id) = $query->fetchrow_array()) {
							dbDoQuery('DELETE FROM Repository WHERE rep_id=' . dbQuote($pool_id));
						}
					} else {
						dbDoQuery('DELETE FROM TmplData WHERE element="tmpl_pool_id" AND tmpl_id=' . dbQuote($tpl_id));
						$pool_id = undef;
					}

					my $ip_address_was_restored = undef;
					foreach my $ipNode ($sn->xql('./*')) {
						my $ip_address_id = $ipNode->getAttribute('id');
						##########################################################
						### must be deprecated
						unless ($ip_address_id) {
							######################################################
							### for compatibility with old beta builds for PSA 6.0
							$ip_address_id = $ipNode->getAttribute('value');
						}
						unless ($ip_address_id) {
							print_stderr("Unable to restore ip address for ip pool in client template: unable to define ip address id.\n");
							next;
						}
						my $ip_address_node = getIPAddressNodeById($ip_address_id);
						unless ($ip_address_node) {
							printf_stderr("Unable to restore ip address for ip pool in client template: unable to define ip address node by id=%s\n", $ip_address_id);
							next;
						}

						### check "mapped" attribute for all ip's which was mapped
						### it is nessesary detecting ip addresses which must be restored for client ip pool 
						###### REASON:
						###### if ip address was mapped then ip pool collision was checked
						###### if ip address was not mapped then one don't have to restore in any case
						if ($ip_address_node->getAttribute('mapped') ne 'true') {
							next;
						}

						my $ip_address = $ip_address_node->getAttribute('ip_address');
						next if exists($ips_tmp{$ip_address});

						my $real_ip_address_id = $ip_address_node->getAttribute('real_id');
						unless ($real_ip_address_id) {
							printf_stderr("IP address '%s' restoration for ip pool in client template is skipped: ip address was not restored in system.\n"
											, $ip_address);
							next;
						}
						$ips_tmp{$ip_address} = 1;

						$ip_address_was_restored = 1;
						if ($pool_id) {
							dbDoQuery("REPLACE INTO Repository(rep_id, component_id) VALUES($pool_id, $real_ip_address_id)");
						} else {
							dbDoQuery("REPLACE INTO Repository(component_id) VALUES($real_ip_address_id)");
							$pool_id = dbGetInsertId();
						}
					}
					if ($pool_id) {
						if ($ip_address_was_restored) {
							dbDoQuery('REPLACE INTO TmplData(tmpl_id, element, value) VALUES('.
								"\"$tpl_id\", \"tmpl_pool_id\", \"$pool_id\")");
						} else {
							dbDoQuery('DELETE FROM TmplData WHERE element="tmpl_pool_id" AND tmpl_id=' . dbQuote($tpl_id));
						}
					}
				} else {
					dbDoQuery('REPLACE INTO TmplData(tmpl_id, element, value) VALUES('.
							$tpl_id . ', "'. u2a($sn->getNodeName()) . '", "' . u2a($sn->getAttribute('value')) . '")');
				}
			}

			if ($rep_id) {
				dbDoQuery("REPLACE INTO Repository(rep_id, component_id) VALUES($rep_id, $tpl_id)");
			} else {
				dbDoQuery("INSERT INTO Repository(component_id) VALUES($tpl_id)");
				$rep_id = dbGetInsertId();
				putMiscParam('cl_tmpl_list_id', $rep_id);
			}
		}

		#restore slave servers accounts
		foreach my $node ($baseNode->xql('./slave_servers/server')) {
			my $query = dbDoQuery('SELECT id FROM slaves WHERE hostname=' . dbQuote($node->getAttribute('hostname')));
			my $slave_id = undef;
			if (($slave_id) = $query->fetchrow_array()) {
				dbDoQuery(sprintf('REPLACE INTO slaves(id, hostname) VALUES(%s, %s)',
								$slave_id, dbQuote($node->getAttribute('hostname'))));
			} else {
				dbDoQuery('REPLACE INTO slaves(hostname) VALUES('.dbQuote($node->getAttribute('hostname')).')');
				$slave_id = dbGetInsertId();
			}

			foreach my $sn ($node->xql('./params/*')) {
				my $parameter = '';
				my $value = '';
				if ($sn->getNodeName() eq 'certificate') {
					$query = dbDoQuery('SELECT value FROM slave_params WHERE parameter="cert_id" AND slave_id=' . dbQuote($slave_id));
					my ($cert_id) = $query->fetchrow_array();
					restoreCert($sn, $cert_id);
					$parameter = 'cert_id';
					$value = $cert_id ? $cert_id : $sn->getAttribute('real_id');
				} else {
					$parameter = $sn->getNodeName();
					$value = u2a($sn->getAttribute('value'));
				}
				dbDoQuery('REPLACE INTO slave_params(slave_id, parameter, value) VALUES'.
						"('$slave_id', '$parameter', '$value')");
			}
		}

		foreach my $node ($baseNode->xql('./upgrade_history/upgrade')) {
			my $buff = XMLAttrs2SQL('REPLACE INTO upgrade_history(%s) VALUES(%s)', $node->getAttributes(), 1);
			dbDoQuery($buff);
		}

		foreach my $node ($baseNode->xql('./mailer_queries/query')) {
			my $query = dbDoQuery('SELECT id FROM mailer_queries WHERE query=' . dbQuote($node->getAttribute('query')));
			unless ($query) {
				printf_stderr("Unable to fetch mailer_queries.\n");
				last;
			}
			my $buff = '';
			if (my ($id) = $query->fetchrow_array()) {
				$buff = XMLAttrs2SQL('REPLACE INTO mailer_queries(id, %s) VALUES(' . $id . ', %s)', $node->getAttributes(), 1);
			} else {
				$buff = XMLAttrs2SQL('REPLACE INTO mailer_queries(%s) VALUES(%s)', $node->getAttributes(), 1);
			}
			dbDoQuery($buff) if $buff;
		}
	}

	return 1;
}


sub restoreIPAddresses
{
	my ($base_node) = @_;

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		foreach my $node ($base_node->xql('./*')) {
			my $query = dbDoQuery('SELECT * FROM IP_Addresses WHERE ip_address=' . dbQuote($node->getAttribute('ip_address')));
			if (my $hash = $query->fetchrow_hashref) {
				$node->setAttribute('restore', 'false');
				$node->setAttribute('real_id', $$hash{'id'});

				### if default domain is specified for source ip than store new (real) default_domain_id
				### it will be restored on "RESTORE" step
				if ($node->getAttribute('default_domain_id')) {
					### set real default_domain_id for this ip
					$node->setAttribute('real_default_domain_id', $$hash{'default_domain_id'} ? $$hash{'default_domain_id'} : '0');
				}
			} elsif (not $node->getAttribute('restore')) {

				$node->setAttribute('restore', 'true');

				############################################
				#	IP address will be restored and should
				#	restore certificate as well
				############################################
				if (my $certNode = getCertificateNodeById($node->getAttribute('ssl_certificate_id'))) {

					#####################################################################
					#	Increase number of IP addreses which are using this certificate
					#####################################################################
					my $used = $certNode->getAttribute('used_for_ip');
					$used++;
					$certNode->setAttribute('used_for_ip', $used);

					##################################################################
					#	If there are any IPs which use this certificate - restore it
					##################################################################
					if ($used > 0) {
						$certNode->setAttribute('restore', 'true');

						# hash certificate keys nodes
						foreach my $keyNode ($certNode->xql('./*')) {
			        		hashXMLNodeByCID($keyNode);
						}
					}
				}
			}
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		my %defaultDomainIP = ();

		foreach my $node ($base_node->xql('./*[(@restore="true") $or$ (@real_default_domain_id="0")]')) {
			if ($node->getAttribute('restore') eq 'true') {
				my $ssl_certificate_node = getCertificateNodeById($node->getAttribute('ssl_certificate_id'));
				my $ssl_certificate_id = $ssl_certificate_node ? $ssl_certificate_node->getAttribute('real_id') : 0;
				my $query = dbDoQuery('INSERT INTO IP_Addresses(iface, ip_address, mask, type, ssl_certificate_id)'
									. ' VALUES(' . dbQuote($node->getAttribute('iface'))
									. ', ' . dbQuote($node->getAttribute('ip_address'))
									. ', ' . dbQuote($node->getAttribute('mask'))
									. ', ' . dbQuote($node->getAttribute('type'))
									. ', ' . dbQuote($ssl_certificate_id ? $ssl_certificate_id : 0)
									. ')'
							);
				$node->setAttribute('real_id', dbGetInsertId());
			}

			### if default_domain_id was specified for source ip address and is not defined for target ip address
			### include real id of this ip to hash for default domain ip restoration
			if ($node->getAttribute('default_domain_id')) {
				$defaultDomainIP{$node->getAttribute('default_domain_id')} = $node->getAttribute('real_id');
			}
		}

		$conf{'DEFAULT_DOMAIN_IP_HASHREF'} = \%defaultDomainIP;
	}

	return 1;
}

############################################################
#
#	Try to find matching certificate already in repository
#	to prevent unneccessary certificates duplication
#
############################################################
sub checkCertExists {
	my ($node, $pub_key, $pvt_key, $ca_cert, $csr) = @_;

	{
		#######################################################
		#	This function works properly only in RESTORE mode
		#######################################################
		if ($flags{RESTORE_MODE} eq 'CHECK') {
			print_stderr("BU::PSA::Restore::CommonData::checkCertExists() called in 'CHECK' mode!!!\n");
			die "\n";
		}
	}

	my $rep_name;
	my $rep_id;

	################################################
	#	Check that certificate belongs to somebody
	################################################
	if (not ($rep_name = $node->getAttribute('repository'))) {
		return 0;
	}

	if ($rep_name eq 'ADMIN') {

		###############################################
		#	Certificate belongs to admin's repository
		###############################################
		if (not ($rep_id = getMiscParam('cert_rep_id'))) {

			# assume no admin's repository exists yet
			return 0;
		}

	} else {

		###############################################
		#	Certificate belongs to one of the domains
		###############################################
		my $query = dbDoQuery('SELECT cert_rep_id FROM domains WHERE name=' . dbQuote($rep_name));
		return 0 unless ($query);
		($rep_id) = $query->fetchrow_array;
		if (not $rep_id) {

			# assume no domain's repository exists yet
			return 0;
		}
	}

	##########################################
	#	Retrieve certificate characteristics
	#	Prepare data for SQL LIKE statement:
	#	'%<quoted data>%'
	##########################################
	my $name = u2a($node->getAttribute('name'));

	if ($pub_key) {
		$pub_key =~ s/%/\\%/g;
		$pub_key =~ s/^'/%/;
		$pub_key =~ s/'$/%/;
	}

	if ($pvt_key) {
		$pvt_key =~ s/%/\\%/g;
		$pvt_key =~ s/^'/%/;
		$pvt_key =~ s/'$/%/;
	}

	if ($csr) {
		$csr =~ s/%/\\%/g;
		$csr =~ s/^'/%/;
		$csr =~ s/'$/%/;
	}

	if ($ca_cert) {
		$ca_cert =~ s/%/\\%/g;
		$ca_cert =~ s/^'/%/;
		$ca_cert =~ s/'$/%/;
	}

	###############################################################
	#	Following query will have kilobytes of arguments
	#	But there is no other way to ensure that certificate in
	#	repository isn't the same... Probably we can reduce query
	#	using only private key comparsion?
	###############################################################
	my $query_str = sprintf("
SELECT
	id 
FROM
	certificates c 
LEFT JOIN 
	Repository r 
ON 
	r.component_id = c.id
WHERE 
	    r.rep_id     = '%s'
	AND c.csr        LIKE '%s'
	AND c.pvt_key    LIKE '%s'
	AND c.cert       LIKE '%s'
	AND c.ca_cert    LIKE '%s'
	AND c.name       = '%s'",

$rep_id,
$csr,
$pvt_key,
$pub_key,
$ca_cert,
$name,
	);

	my $res = dbDoQuery($query_str);
	if (not $res) {
		return 0;
	}

	if (my ($id) = $res->fetchrow_array) {
		#########################################################
		#	There is the same certificate already in repository
		#	so we should skip restoration of this certificate
		#########################################################

		$node->setAttribute('real_id', $id) if $id;
		return $id;
	}

	return 0;
}


sub restoreCertificates
{
	my ($base_node) = @_;

	if ($flags{RESTORE_MODE} eq 'CHECK') {

	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {

		#######################################################################
		#	Restore certificate only if it's needed ("restore" attribute set)
		#	or it exists in somebody repository and there is no similar 
		#	certificates in repository already (slow... doh...)
		#######################################################################
		foreach my $node ($base_node->xql('./*[(@restore = "true") $or$ @repository]')) {
			my $set_default_cert = ($node->getAttribute('default') eq 'true') && !$flags{psaIsConfigured};

			if (($node->getAttribute('restore') eq 'true') || 
				($node->getAttribute('repository')) || 
				((!$node->getAttribute('real_id') || $set_default_cert))
			) {
				restoreCert($node, undef);
			}

			###########################################################
			#	Update database if certificate was restored. If
			#	it belongs to admin or nobody - place it into admin's
			#	repository.
			###########################################################
			my $repository = $node->getAttribute('repository');
			if (($set_default_cert || ($repository eq 'ADMIN') || ($repository eq '')) && $node->getAttribute('real_id')) {
				dbDoQuery(sprintf('REPLACE INTO Repository(rep_id, component_id) VALUES("%s", "%s")',
						getMiscParam('cert_rep_id'), $node->getAttribute('real_id')));

				putMiscParam('default_certificate_id', $node->getAttribute('real_id')) if $set_default_cert;
			}
		}
	}

	return 1;
}


sub loadCertFromFile
{
	my ($cert_file) = @_;

	unless ($cert_file) {
		print_stderr("loadCertFromFile function error: certificate file is not defined\n");
		return undef;
	}

	my $pub_key = '';
	my $pvt_key = '';
	my $csr = '';

	my $dest_file = $conf{CERTIFICATES_D} . '/' . $cert_file;
	unless (open (FD, $dest_file)) {
		printf_stderr("Unable to open certificate file $dest_file: %s\n", $!);
	} else {
		binmode FD;

		while (<FD>) {
			if (!$pub_key && m/^-+\s*BEGIN\s+CERTIFICATE\s*-+$/) {
				$pub_key = $_;
				while (<FD>) {
					$pub_key .= $_;
					last if (m/^-+\s*END/);
				}
				last if ($pub_key && $pvt_key && $csr);
				next;
			}
			if (!$pvt_key && m/^-+\s*BEGIN(\s+[^\s]+)?\s+PRIVATE\s+KEY\s*-+$/) {
				$pvt_key = $_;
				while (<FD>) {
					$pvt_key .= $_;
					last if (m/^-+\s*END/);
				}
				last if ($pub_key && $pvt_key && $csr);
				next;
			}
			if (!$csr && m/^-+\s*BEGIN\s+CERTIFICATE\s+REQUEST\s*-+$/) {
				$csr = $_;
				while (<FD>) {
					$csr .= $_;
					last if (m/^-+\s*END/);
				}
				last if ($pub_key && $pvt_key && $csr);
				next;
			}
		}
		close FD;
	}

	my @result = ($pub_key, $pvt_key, $csr);

	return \@result;
}


sub restoreCert
{
	my ($node, $cert_id) = @_;

	my $name = u2a($node->getAttribute('name'));
	
	my $cert_file = $node->getAttribute('cert_file');
	my $arr = loadCertFromFile($cert_file);
	my ($pub_key, $pvt_key) = @$arr;
	$pub_key = url_encode($pub_key) if ($pub_key);
	$pvt_key = url_encode($pvt_key) if ($pvt_key);

	my $csr = '';
	if (my ($sn) = $node->xql('./csr')) {
		my $csr_file = $node->getAttribute('cert_file');
		my $arr = loadCertFromFile($csr_file);
		my ($tmp_pub_key, $tmp_pvt_key, $tmp_csr) = @$arr;
		$csr = url_encode($tmp_csr) if $tmp_csr;
	}

	my $ca_cert = '';
	my $ca_file = '';
	if (my ($sn) = $node->xql('./ca_cert')) {
		$ca_file = $sn->getAttribute('cert_file');
		my $arr = loadCertFromFile($ca_file);
		my ($tmp_pub_key, $tmp_pvt_key) = @$arr;
		$ca_cert = url_encode($tmp_pub_key) if $tmp_pub_key;
	}

	if (checkCertExists($node, $pub_key, $pvt_key, $ca_cert, $csr)) {
		### if certificate already exists then remove saved in psarestore certificate 
		### files and return
		my $query = dbDoQuery('SELECT cert_file, ca_file FROM certificates WHERE id="' . $node->getAttribute('real_id') . '"');
		my ($psa_cert_file, $psa_ca_file) = $query ? $query->fetchrow_array() : (undef, undef);

		if ($cert_file) {
			my $dest_file = $conf{CERTIFICATES_D} . '/' . $cert_file;
			### check the restorated certificate's file existing
			if (-e $dest_file) {
				$psa_cert_file = $conf{CERTIFICATES_D} . '/' . $psa_cert_file;
				### check the existing of the file for the same certificate existing in plesk
				if (-e $psa_cert_file && !(-z $psa_cert_file)) {
					### if certificate file in plesk exists then remove dublicate
					unlink($dest_file);
				} else {
					### if certificate in plesk is broken then replcate it with file for restorated certificate
					system("mv -f '$dest_file' '$psa_cert_file'");
				}
			}
		}
		if ($ca_file) {
			my $dest_file = $conf{CERTIFICATES_D} . '/' . $ca_file;
			### check the restorated certificate's file existing
			if (-e $dest_file) {
				$psa_ca_file = $conf{CERTIFICATES_D} . '/' . $psa_ca_file;
				### check the existing of the file for the same certificate existing in plesk
				if (-e $psa_ca_file && !(-z $psa_ca_file)) {
					### if certificate file in plesk exists then remove dublicate
					unlink($dest_file);
				} else {
					### if certificate in plesk is broken then replcate it with file for restorated certificate
					system("mv -f '$dest_file' '$psa_ca_file'");
				}
			}
		}

		return 1;
	}

	my $query_str = undef;
	if ($cert_id) {
		$query_str = "REPLACE INTO certificates(id, name, cert, pvt_key, csr, ca_cert, cert_file, ca_file) VALUES('$cert_id', '$name', '$pub_key', '$pvt_key', '$csr', '$ca_cert', '$cert_file', '$ca_file')";
		dbDoQuery($query_str);
	} else {
		$query_str = "INSERT INTO certificates(name, cert, pvt_key, csr, ca_cert, cert_file, ca_file) VALUES('$name', '$pub_key', '$pvt_key', '$csr', '$ca_cert', '$cert_file', '$ca_file')";
		dbDoQuery($query_str);
		$cert_id = dbGetInsertId();
	}

	$node->setAttribute('real_id', $cert_id);

	#############################################
	#	Certificate was successfuly restored
	#	Set attribute for subsequent processing
	#############################################
	$node->setAttribute('restore', 'true');
}


sub restoreReports
{
	my ($baseNode, $user_id, $user_type) = @_;

	my $old_default_id = undef;
	my $new_default_id = undef;

	my $query = dbDoQuery('SELECT id FROM report WHERE user_id=' . dbQuote($user_id) . ' AND user_type=' . dbQuote($user_type) . ' AND is_default="true"');
	if ($query) {
		($old_default_id) = $query->fetchrow_array();
	} else {
		print_stderr("Unable to define default client report for admin\n");
	}

	foreach my $node ($baseNode->xql('./custom_reports/*')) {
		my $report_id = undef;
		if (my $query = dbDoQuery('SELECT id FROM report WHERE user_id="' . dbQuote($user_id) . '" AND user_type="' . dbQuote($user_type) . '" AND type=' . dbQuote($node->getAttribute('type')))) {
			($report_id) = $query->fetchrow_array();
		} else {
			print_stderr("Unable to fetch report id.\n");
		}

		my $query_str = '';
		if ($report_id) {
			$query_str = XMLAttrs2SQL('REPLACE INTO report(id, user_id, user_type, %s) VALUES("' . $report_id . ', ' . dbQuote($user_id) . ' , '. dbQuote($user_type) . ', %s)', $node->getAttributes(), 1);
		} else {
			$query_str = XMLAttrs2SQL('INSERT INTO report(user_id, user_type, %s) VALUES(' . dbQuote($user_id) . ', '. dbQuote($user_type) .', %s)', $node->getAttributes(), 1);
			$report_id = dbGetInsertId();
		}
		if (dbDoQuery($query_str) && ($node->getAttribute('is_default') eq 'true')) {
			$new_default_id = $report_id;
		}

		{ ## restore sections
			dbQuote('DELETE FROM report_section where report_id=' . dbQuote($report_id));
			foreach my $sn ($node->xql('./section/*')) {
				my $query_str = XMLAttrs2SQL('INSERT INTO report_section(report_id, %s) VALUES('. dbQuote($report_id) .', %s)', $sn->getAttributes, 1);
				unless (dbDoQuery($query_str)) {
					print_stderr("Unable to insert into report_section table\n");
				}
			}
		}
		{ ## restore auto
			dbQuote('DELETE FROM report_auto where report_id=' . dbQuote($report_id));
			foreach my $sn ($node->xql('./auto/*')) {
				my $query_str = XMLAttrs2SQL('INSERT INTO report_section(report_id, %s) VALUES('. dbQuote($report_id) .', %s)', $sn->getAttributes, 1);
				unless (dbDoQuery($query_str)) {
					print_stderr("Unable to insert into report_section table\n");
				}
			}
		}
	}
	if ($old_default_id && $new_default_id && ($old_default_id ne $new_default_id)) {
		dbDoQuery('UPDATE report SET is_default="false" WHERE id=' . dbQuote($old_default_id));
	}
}

sub restoreCustomButtons
{
	my ($node, $level_id, $level, $level_exists) = @_;

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		foreach my $node ($node->xql('./custom_buttons/button')) {
			my $path = $conf{'PRODUCT_ROOT_D'} . '/admin/htdocs/images/custom_buttons';
			my $file = getUniqFileName($path);
			$node->setAttribute('destfile', "$path/$file");
			$node->setAttribute('file', $file);

			if ($level_exists) {
				my $query = dbDoQuery(sprintf('SELECT id FROM custom_buttons WHERE level_id="%s" AND level="%s" AND text=%s AND url=%s',
									  $level_id, $level, dbQuote(u2a($node->getAttribute('text'))), dbQuote(u2a($node->getAttribute('url'))))
				);
				if (my ($tmp) = $query->fetchrow_array()) {
					$node->setAttribute('exists', 'true');
				} else {
					hashXMLNodeByCID($node);
				}
			} else {
				hashXMLNodeByCID($node);
			}
		}
	} else {
		foreach my $node ($node->xql('./custom_buttons/button')) {
			next if ($node->getAttribute('exists') eq 'true');
			$node->removeAttribute('destfile');
			$node->removeAttribute('src');

			my $file = $node->getAttribute('file');
			my $file_path = $conf{'PRODUCT_ROOT_D'} . '/admin/htdocs/images/custom_buttons/' . $file;
			unless ( -f $file_path ) {
				$node->setAttribute('file', '');
			}
			my $buff = XMLAttrs2SQL('INSERT INTO custom_buttons(level_id, level, %s) VALUES(' . dbQuote($level_id). ', "' . $level . '" , %s)', $node->getAttributes(), 1);
			dbDoQuery($buff);
		}
	}

	return 1;
}

return 1;

