package BU::PSA::Restore::Config;

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::PSA::CommonFunc;
use BU::PSA::DB;

use BU::PSA::Restore::Server;
use BU::PSA::Restore::Admin;
use BU::PSA::Restore::Client;
use BU::PSA::Restore::System;
use BU::PSA::Restore::CommonData;

@ISA = qw(Exporter);
@EXPORT = qw(
restoreConfig
restoreSiteAppPackages
);

sub restoreConfig()
{
	if ($flags{RESTORE_MODE} eq 'CHECK') {

		print_stdout ("$ProcessName installed language packs ...\n");
		my %installedLocales = ();
		if (my $query = dbDoQuery('select id from locales')) {
			my $hasLocales = '';

			while (my ($localeId) = $query->fetchrow_array()) {
				$installedLocales{$localeId} = 1;
				$hasLocales = 1;
				print_stdout ("supported language: $localeId\n") if ($debug > 0);
			}

			unless ($hasLocales) {
				print_stderr("Restoration can not continue because there are no language packs installed in PSA.\n");
				die "\n";
			}
		} else {
			print_stderr("Error checking language packs: unable to define installed language packs in PSA.\n");
			die "\n";
		}

		my $hasUnsupportedLocales = '';
		my %tmpLocalesList = ();
		if (my ($localeNode) = $psaNode->xql('./server/params')) {
			my $locale = $localeNode->getAttribute('def_locale');

			unless (exists($installedLocales{$locale})) {
				if ($flags{forceLocales}) {
					$localeNode->setAttribute('def_locale', $flags{forceLocales});
					print_stdout ("Default PSA contol panel language is changed to english.\n");
				} elsif (!exists($tmpLocalesList{$locale})) {
					printf_stderr("Found the language which is not supported: %s\n", $locale);
					$hasUnsupportedLocales = 1;
					$tmpLocalesList{$locale} = 1;
				}
			}
		}

		my $extendetCondition = ($conf{'RESTORING_BY_CLIENTS_LIST'} || $conf{'RESTORING_BY_DOMAINS_LIST'}) ? ' [@restore_by_list = "true"]' : '';

		foreach my $localeNode ($psaNode->xql("./user$extendetCondition/card/locale")) {
			my $textNode = $localeNode->getFirstChild;

			unless ($textNode) {
				printf_stderr("An error occured during languages checking: unable to retrive user language.\n");
				next;
			}

			my $locale = $textNode->toString();

			print_stdout ("language: $locale\n") if ($debug > 0);

			unless (exists($installedLocales{$locale})) {
				if ($flags{forceLocales}) {
					$localeNode->getFirstChild->setNodeValue($flags{forceLocales});
				} elsif (!exists($tmpLocalesList{$locale})) {
					printf_stderr("Found the language which is not supported: %s\n", $locale);
					$hasUnsupportedLocales = 1;
					$tmpLocalesList{$locale} = 1;
				}
			}
		}

		if (!$flags{forceLocales} && $hasUnsupportedLocales) {
			print_stderr("You must install all the missing language packs or use -L <locale_id> key to set ".
				"all the language parameters to some existing locale (-L list - for getting locales list).\n");
				die "\n";
		}

		print_stdout ("done\n");

	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
	} else {
		print_strerr("Unknown restoring mode.\n");
		die "\n";
	}

	# check installation and restore registration info of site app packages
	if (my ($serverNode) = $psaNode->xql('./server')) {
		restoreSiteAppPackages($serverNode);
	}

	# restore PSA clients
	if ($conf{'RESTORING_BY_CLIENTS_LIST'}) {
		my $clientsListForRestore = $conf{'RESTORING_BY_CLIENTS_LIST'};
		foreach my $clientNode ($psaNode->xql('./user[(@type = "client") $and$ (@restore_by_list = "true")]')) {
			restoreClient($clientNode);
			my ($loginNode) = $clientNode->xql('./login');
			delete($$clientsListForRestore{$loginNode->getAttribute('name')});
		}

		my $notExistsClients = '';

		while (my ($login, $tmp) = each(%$clientsListForRestore)) {
			next unless $login;
			print_stderr "WARNING: client with login '$login' from list is not exist in dump.\n";
			$notExistsClients = 1;
		}

		if ($notExistsClients) {
			print_stderr("ERROR: found some clients in list which are not exist in dump.\n");
			die unless $flags{'FORCE'};
		}
	} elsif ($conf{'RESTORING_BY_DOMAINS_LIST'}) {
		foreach my $clientNode ($psaNode->xql('./user[(@type = "client") $and$ @restore_by_list="true"]')) {
			restoreClient($clientNode);
		}
	} else {
		#restore server preferences
		if ($flags{psaIsConfigured} && !$flags{'RESTORE_SERVER'}) {
			if ($flags{RESTORE_MODE} eq 'CHECK') {
				printf_stdout("Server preferences will not be restored because PSA is configured\n\t(if you want to restore it you should use --restore-server option)\n");
			}
		} else {
			my ($serverNode) = $psaNode->xql('./server');
			unless ($serverNode){
				print_stderr("Unable to retrieve server configuration from backup.\n");
			} else {
				restoreServer($serverNode);
			}
		}

		#restore PSA admin preferences
		if ($flags{psaIsConfigured} && !$flags{'RESTORE_ADMIN'}) {
			if ($flags{RESTORE_MODE} eq 'CHECK') {
				printf_stdout("Admin info and preferences will not be restored because PSA is configured\n\t(if you want to restore it you should use --restore-admin option)\n");
			}
		} else {
			my ($adminNode) = $psaNode->xql('./user[@type = "admin"]');
			unless ($adminNode){
				print_stderr("Unable to get admin info from dump\n");
			} else {
				restoreAdmin($adminNode);
			}
		}

	    foreach my $clientNode ($psaNode->xql('./user[@type = "client"]')) {
			restoreClient($clientNode);
	    }


		if (!$flags{psaIsConfigured} || $flags{'RESTORE_SERVER'}) {
	    	if (my ($systemNode) = $psaNode->xql('./system')) {
			    restoreSystem($systemNode);
		    }
		}
	}

}


sub restoreSiteAppPackages
{
	my ($commonDataNode) = @_;
	my $baseNode = $commonDataNode;

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		foreach my $node ($baseNode->xql('./site_app_packages/*')) {
			my $name = u2a($node->getAttribute('name'));
			print_stdout("Check site application '$name' package installation ... ");
			$query = dbDoQuery(sprintf('SELECT id FROM SiteAppPackages WHERE name=%s AND version=%s AND release=%s',
									dbQuote($name),
									dbQuote($node->getAttribute('version')),
									dbQuote($node->getAttribute('release'))
								));
			unless ($query) {
				print_stderr("Unable to check site application '$name' package existing: sql query error\n");
				next;
			}

			if (my ($id) = $query->fetchrow_array()) {
				print_stdout("installed\n");
				$node->setAttribute('real_id', $id);
				$node->setAttribute('exists', 'true');
			} else {
				print_stdout("not installed\n");
			}
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		foreach my $node ($baseNode->xql('./site_app_packages/*')) {
			if ($node->getAttribute('exists') eq 'true') {
				printf_stderr("Site application package '%s' (version: %s, release: %s) is exist\n", $node->getAttribute('name'), $node->getAttribute('version'), $node->getAttribute('release'));
				next;
			}
			my $name = u2a($node->getAttribute('name'));
			print_stdout("Install registration info of site application '$name' package ... ");
			my $res = dbDoQuery(sprintf('INSERT INTO SiteAppPackages(name, version, release, description, access_level, integrated) values(%s,%s,%s,%s,%s,%s)',
									dbQuote($name),
									dbQuote($node->getAttribute('version')),
									dbQuote($node->getAttribute('release')),
									dbQuote(u2a($node->getAttribute('description'))),
									dbQuote($node->getAttribute('access_level')),
									dbQuote($node->getAttribute('integrated'))
								));
			if ($res) {
				print_stdout("done\n");
				$node->setAttribute('real_id', dbGetInsertId());
			} else {
				print_stdout("error\n");
			}
		}
	}
}

return 1;

