#####################################################
package BU::PSA::Restore::DSMail;
#####################################################

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::DNSRec;
use BU::XML::CommonFunc;


@ISA = qw(Exporter);
@EXPORT = qw(restoreDSMail);

sub restoreDSMail
{
	my ($base_node, $domain) = @_;

	my $status = $base_node->getAttribute('status');
	my $domainId = $domain->Exists() ? $domain->getDBId() : undef;
	my $domainName = $domain->getField('name');
	my $domainDisplayName = u2a($domain->getField('displayName'));

	my $isExist = 0;
	if ($domainId) {
		my $query = dbDoQuery('SELECT count(*) FROM DomainServices WHERE type=\'mail\' AND dom_id=' . dbQuote($domainId));
		unless (($isExist) = $query->fetchrow_array()) {
			printf_stderr("Unable to check 'mail' domain service existing from database.\n");
			return '';
		}
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		# do not do nothing
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		$status = $status ^ ($status & $const{'STATUS_WITH_PARENT'});

		unless ($isExist) {
			dbDoQuery('INSERT INTO DomainServices(dom_id, type, status) values('
						. dbQuote($domainId) . ", 'mail', $status)");
		}

		dbDoQuery('DELETE FROM dns_recs WHERE'
				.	" type LIKE 'MX' AND host LIKE '$domainDisplayName.'"
				.	" AND val LIKE 'mail.$domainDisplayName.' AND dom_id = '$domainId'"
				);

#		if ($status == 0) {
			$dnsRec = new BU::PSA::Objects::DNSRec($domainId, "$domainDisplayName.", 'MX', "mail.$domainDisplayName.", 10);
			$dnsRec->Store();
#		}

		unless (util_exec('mailmng', (($status > 0) ? '--turn-off-domain' : '--turn-on-domain',
							'--domain-name=' . $domainName))) {
			printf_stderr("Unable to turning on domain service 'mail': " . util_exec_errormsg());
			return '';
		} else {
			dbDoQuery('UPDATE DomainServices set status=' . $status . ' WHERE type=\'mail\' AND dom_id=' . dbQuote($domainId));
		}

		util_exec('dnsmng', 'update',  $domainName, 'without-reverse');

		if (my ($paramsNode) = $base_node->xql('params')) {
			my $query = dbDoQuery('SELECT parameters_id FROM DomainServices WHERE type=\'mail\' AND dom_id=' . dbQuote($domainId));
			unless ($query) {
				print_stderr("Unable to fetch parameters_id from database\n");
			} else {
				my ($params_id) = $query->fetchrow_array();
				if ($params_id) {
					$params_id_name = 'id, ';
					$params_id = dbQuote($params_id) . ', ';
				} else {
					$params_id_name = '';
					$params_id = '';
				}

				my $type = $paramsNode->getAttribute('nonexist_mail');
				my $bounce = $paramsNode->getAttribute('bounce_mess');
				my $catch = $paramsNode->getAttribute('catch_addr');
				dbDoQuery("REPLACE INTO Parameters(" . $params_id_name . "parameter, value) values(" . $params_id . "'nonexist_mail', '$type')");
				unless ($params_id) {
					$params_id_name = 'id, ';
					$params_id = dbGetInsertId();
					dbDoQuery('UPDATE DomainServices set parameters_id=' . dbQuote($params_id) . ' WHERE type=\'mail\' AND dom_id=' . dbQuote($domainId));
					$params_id = dbQuote($params_id) . ', ';
				}
				dbDoQuery("REPLACE INTO Parameters(" . $params_id_name . "parameter, value) values(" . $params_id . "'catch_addr', '$catch')");
				dbDoQuery("REPLACE INTO Parameters(" . $params_id_name . "parameter, value) values(" . $params_id . "'bounce_mess', '$bounce')");

				if ($type eq 'bounce') {
					if ($bounce) {
						unless (util_exec('mailmng', '--set-bounce',
											'--domain-name=' . $domainName,
											'--phrase=' . shellArgQuote($bounce))) {
							print_stderr("Unable to set bounce message for domain '%s': %s\n", $domainName, util_exec_errormsg());
						}
					}
				} elsif ($type eq 'catch') {
					if ($catch) {
						unless (util_exec('mailmng', '--set-catchall',
											'--domain-name=' . $domainName,
											'--email=' . shellArgQuote($catch))) {
							printf_stderr("Unable to set catch address for domain '%s': %s\n", $domainName, util_exec_errormsg());
						}
					}
				} elsif ($type eq 'reject') {
					unless (util_exec('mailmng', '--set-reject',
										'--domain-name=' . $domainName)) {
						printf_stderr("Unable to set reject for domain '%s': %s\n", $domainName, util_exec_errormsg());
					}
				} 
			}
		}
	}

	return 1;
}


return 1;


