package BU::PSA::Restore::Server;

require 5.004;
require Exporter;

use MIME::Base64;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::XML::CommonFunc;
use BU::PSA::Restore::CommonData;

@ISA = qw(Exporter);
@EXPORT = qw(restoreServer);

sub restoreServer($)
{
	my ($serverNode) = @_;

	if ($PRETTYPRINT) {
		print_cl('Server configuration');
	} else {
		printf_stdout("%s server configuration.\n", $ProcessName);
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		if (my ($certNode) = $serverNode->xql('./certificate')) {
			$certNode->setAttribute('destfile', $conf{PRODUCT_ROOT_D} . '/admin/conf/httpsd.pem');
        	hashXMLNodeByCID($certNode);
		}

		if (my ($certNode) = $serverNode->xql('./rootchain_certificate')) {
			$certNode->setAttribute('destfile', $conf{PRODUCT_ROOT_D} . '/admin/conf/rootchain.pem');
        	hashXMLNodeByCID($certNode);
		}

		foreach my $domainDefaultsNode ($serverNode->xql('./domain[@name="default"]')) {
				$domainDefaultsNode->setAttribute('destdir', $conf{HTTPD_VHOSTS_D} . '/' . $domainDefaultsNode->getAttribute('name'));
        		hashXMLNodeByCID($domainDefaultsNode);
		}

		foreach my $mailCtlFileNode ($serverNode->xql('./MailControl/conf-file[@src]')) {
			if (my $file = $mailCtlFileNode->getAttribute('file')) {
				$mailCtlFileNode->setAttribute('destfile', $conf{QMAIL_CONTROL_D} . "/$file");
        		hashXMLNodeByCID($mailCtlFileNode);
			}
		}

		if (my ($node) = $serverNode->xql('./vhost_skeleton')) {
			$node->setAttribute('destdir', $conf{'HTTPD_VHOSTS_D'} . "/.skel/0");
			hashXMLNodeByCID($node);
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {

		if (my ($certNode) = $serverNode->xql('./rootchain_certificate')) {
			my $status = $certNode->getAttribute('status');

			if ($status eq 'true') {
				if (-e $certNode->getAttribute('destfile')) {
					system("chown root:0 ".$certNode->getAttribute('destfile'));
					chmod 0400, $certNode->getAttribute('destfile');
				} else {
					print_stderr("ERROR: admin rootchain certificate is disabled: file with certificate is not exists.\n");
					$status = 'false';
				}
			}
			putMiscParam('rootchain', $status);
		}

		unless ($flags{psaIsConfigured}) {
			$conf{'FULL_HOST_NAME'} =~ m/^([^\.]+)\.(.+)$/;

			unless (util_exec('reconfig', $1, $2)) {
				printf_stderr("Unable to configure PSA: an error occured during reconfig utility execution.\n");
				die "\n";
			}
			putMiscParam('FullHostName', $conf{'FULL_HOST_NAME'});
			putMiscParam('psa_configured', 'true');
			util_exec('dnsmng', 'restart');
		}

		if (my ($alNode) = $serverNode->xql('action_log_prefs')) {
			my $id_index = 1;							# bug fix #26792
			foreach my $sn ($alNode->xql('./*')) {
				my $name = $sn->getAttribute('name');
				my $id = $sn->getAttribute('id');
				$id = $id_index unless ($id);			# bug fix #26792
				$id_index++;
				if (dbDoQuery("DELETE FROM actions WHERE name='$name' or id='$id'")) {
					dbDoQuery(sprintf('REPLACE INTO actions(id, name, descr, enabled) values("%s", %s, %s, %s)',
								$id,
								dbQuote($name),
								dbQuote(u2a($sn->getAttribute('descr'))),
								dbQuote($sn->getAttribute('enabled'))
							));
				}
			}
		}

		if (my ($certNode) = $serverNode->xql('./certificate')) {
			if ( -f $certNode->setAttribute('destfile')) {
				chmod 0400, $certNode->setAttribute('destfile');
			}
		}

		# restore mail controls
		if (my ($mailControlNode) = $serverNode->xql('./MailControl')) {

			# restore mail control files
			foreach my $mailCtlFileNode ($mailControlNode->xql('./conf-file')) {
				my $file = $mailCtlFileNode->getAttribute('file');

				if (!$mailCtlFileNode->getAttribute('src') && $file) {
					my $file = $conf{QMAIL_CONTROL_D} . "/$file";

					my $textNode = $mailCtlFileNode->getFirstChild();
					if ($textNode) {
						my $str = $textNode->toString;
						if ($conf{'FULL_HOST_NAME_IN_DUMP'} ne $conf{'FULL_HOST_NAME'}) {
							$str =~ s/$conf{'FULL_HOST_NAME_IN_DUMP'}/$conf{'FULL_HOST_NAME'}/g;
						}
						open (MAIL_CONTROL, ">$file") || printf_stderr("Unable to write mail control file %s: %s\n", $file, $!);
						print MAIL_CONTROL $str;
						close MAIL_CONTROL || printf_stderr("An error occured during the %s file writing: %s\n", $file, $!);
					} else {
						print_stderr("ERROR: unable to retrive mail controls.\n");
					}
				}
			}
		}

		# restore white list of ip addresses
		foreach my $addressNode ($serverNode->xql('./white_list/address')) {
			my $query = dbDoQuery('SELECT ip_address FROM smtp_poplocks WHERE'
								. ' ip_address = "'.$addressNode->getAttribute('ip').'"'
								. ' and ip_mask ="'.$addressNode->getAttribute('mask').'"'
								);
			
			if ($query && !$query->fetchrow_array()) {
				dbDoQuery('REPLACE INTO smtp_poplocks(ip_address, ip_mask, lock_time) VALUES('
							.'"' . $addressNode->getAttribute('ip') . '", '
							.'"' . $addressNode->getAttribute('mask') . '", '
							.'NULL)');
			}
		}

		if (my ($dnsTemplateNode) = $serverNode->xql('./dns[@template]')) {

			putMiscParam('dns_zone_status', 
						($dnsTemplateNode->getAttribute('dns_zone_status') eq 'off') ? 'false' : 'true');

			# delete all exists DNS template records
			unless (dbDoQuery('DELETE FROM dns_recs_t')) {
				print_stderr("DNS template will not be restored: unable to delete existing records from DNS template.\n");
			} else {

				# restore DNS template
				foreach my $dnsRecNode ($dnsTemplateNode->xql('./host')) {
					my $host;
					my $val;
					my $option = '';
					my $type = $dnsRecNode->getAttribute('type');

					if ($type eq 'A') {
						$host = $dnsRecNode->getAttribute('name');
						$val = $dnsRecNode->getAttribute('ip');
					} elsif ($type eq 'NS') {
						$host = $dnsRecNode->getAttribute('name');
						$val = $dnsRecNode->getAttribute('ns');
					} elsif ($type eq 'PTR') {
						$host = $dnsRecNode->getAttribute('ip');
						$val = $dnsRecNode->getAttribute('name');
						$option = $dnsRecNode->getAttribute('netmask');
					} elsif ($type eq 'CNAME') {
						$host = $dnsRecNode->getAttribute('name');
						$val = $dnsRecNode->getAttribute('cname');
					} elsif ($type eq 'MX') {
						$host = $dnsRecNode->getAttribute('name');
						$val = $dnsRecNode->getAttribute('mx');
						$option = $dnsRecNode->getAttribute('priority');
					} elsif( $type eq 'TXT' ) {
						$host = u2a($node->getAttribute('name'));
						$val = u2a($node->getAttribute('text'));
					} else {
						print_stderr("Unable to restore DNS template record: record type is invalid.\n");
					}

					# base64 decode
					$host = u2a($host);
					$val = u2a($val);

					my $id_field = '';
					my $id_value = '';
					my $time_stamp = $dnsRecNode->getAttribute('time_stamp');

					my $query = dbDoQuery("SELECT id FROM dns_recs_t WHERE type='$type' and host='$host' and val='$val'");

					if (my ($id) = $query->fetchrow_array) {
						$id_field = ', id';
						$id_value = ", '$id'";
					}

					if  ($option) {
						dbDoQuery('REPLACE INTO dns_recs_t(time_stamp, type, host, val, opt'.$id_field.')'
								. "VALUES('$time_stamp', '$type', '$host', '$val', '$option'$id_value)");
					} else {
						dbDoQuery('REPLACE INTO dns_recs_t(time_stamp, type, host, val'.$id_field.')'
								. "VALUES('$time_stamp', '$type', '$host', '$val'$id_value)");
					}
				}
			}
		} # DNS template restoring

		#restore spam protection
		if (my ($node) = $serverNode->xql('./rbl_server')) {
			my $rbl_status = $node->getAttribute('status');
			my $rbl_server = $node->getAttribute('server');
			my $rbl_server_cmdstr = $rbl_server;
			$rbl_server_cmdstr =~ s/;/ /g; # rbl server it is servers list separated by ';' in common case
			unless (util_exec('rblmng.sh', ($rbl_status eq 'true') ? 'on' : 'off' , $rbl_server_cmdstr, '; echo $?')) {
				printf_stderr("ERROR: an error occured during rblmng.sh execution: %s\n", util_exec_errormsg());
				$rbl_status = 'false';
			}
			putMiscParam('rbl', $rbl_status);
			putMiscParam('rbl_server', $rbl_server);
		}

		# restore spamassasin server configuration
		if (my ($saNode) = $serverNode->xql('./spamassassin')) {
			dbDoQuery('DELETE FROM sa_list WHERE isnull(mailname) OR mailname=""');
			dbDoQuery('DELETE FROM sa_conf WHERE isnull(mailname) OR mailname=""');

			my $buff = sprintf("INSERT INTO sa_conf(mailname,flt_enabled,rw_subject_tag,hits_required) VALUES('',%s,%s,%s)",
							dbQuote($saNode->getAttribute('flt_enabled')),
							dbQuote($saNode->getAttribute('rw_subject_tag')),
							dbQuote($saNode->getAttribute('hits_required'))
						);
			dbDoQuery($buff);

			my @options = ();

			my $flt_enabled = $saNode->getAttribute('flt_enabled');
			if (($flt_enabled eq 'serv') || ($flt_enabled eq 'both')) {
				push @options, '--enable-server-configs';
			}
			if (($flt_enabled eq 'user') || ($flt_enabled eq 'both')) {
				push @options, '--enable-user-configs';
			}

			my $rw_subject_tag = $saNode->getAttribute('rw_subject_tag');
			if ($saNode->getAttribute('rw_subject_tag')) {
				push @options, '--modify-header=' . shellArgQuote($rw_subject_tag);
			}

			my $hits_required = $saNode->getAttribute('hits_required');
			if ($hits_required) {
				push @options, '--sensitivity=' . shellArgQuote($hits_required);
			}

			my $white_pattern = '';
			my $black_pattern = '';
			foreach my $sn ($saNode->xql('./*')) {
				$buff = sprintf("INSERT INTO sa_list(mailname, pattern, color) VALUES('', %s, %s)",
							dbQuote($sn->getAttribute('pattern')),
							dbQuote($sn->getAttribute('color'))
						);
				dbDoQuery($buff);

				if ($sn->getAttribute('color') eq 'white') {
					$white_pattern = ($white_pattern ? ',' : '') . $sn->getAttribute('pattern');
				} else {
					$black_pattern = ($black_pattern ? ',' : '') . $sn->getAttribute('pattern');
				}
			}
			if ($white_pattern) {
				push @options, '--add-to-whitelist=' . shellArgQuote($white_pattern);
			}
			if ($black_pattern) {
				push @options, '--add-to-blacklist=' . shellArgQuote($black_pattern);
			}
			push @options, 'restart';

			util_exec('spammng', @options);
		}

		#restore badmailfrom list
		foreach my $node ($serverNode->xql('./badmailfrom/domain')) {
			my $name = $node->getAttribute('name');
			$query = dbDoQuery('select count(*) from badmailfrom where domain=' . dbQuote($name));
			unless ($query) {
				printf_stderr("Unable to check bsdmailfrom list element existing.\n");
				last;
			}
			my ($count) = $query->fetchrow_array();
			next if ($count > 0);

			unless (util_exec('mailmng', '--add-badmailfrom ', '--domain-name=' . shellArgQuote($name))) {
				printf_stderr("Unable to add badmailfrom domain: %s\n", util_exec_errormsg());
			} else {
				dbDoQuery('replace into badmailfrom(domain) values(' . dbQuote($name) . ')');
			}
		}

#		### restore event handlers
#		if (my ($ehNode) = $serverNode->xql('./event_handlers_list')) {
#			dbDoQuery('DELETE FROM event_handlers');
#			foreach my $node ($ehNode->xql('./*')) {
#				$node->removeAttribute('id');
#				my $buff = XMLAttrs2SQL('REPLACE INTO event_handlers(%s) VALUES(%s)', $node->getAttributes(), 1);
#				dbDoQuery($buff);
#			}
#		}

		#restore server params
		if (my ($paramsNode) = $serverNode->xql('./params')) {

			my ($paramsNodesList) = $paramsNode->getAttributes();

			for (my $i=0; $i<$paramsNodesList->getLength; $i++) {
				my $paramNode = $paramsNodesList->item($i);
				my $nodeName = $paramNode->getNodeName();
				next if ($nodeName =~ m/^postgresql_admin_/);

				#########################################################################
				#	Restore allow_short_pop3_names parameter only if it wasn't disabled
				#	already (one-way switch)
				#########################################################################
				if (($nodeName eq 'allow_short_pop3_names') and (getMiscParam('allow_short_pop3_names') eq 'disabled')) {
					next;
				}
				putMiscParam($nodeName, u2a($paramNode->getNodeValue()));
			}
		}

		#restore server extras
		foreach my $extNode ($serverNode->xql('./additional_services/*')) {
			putMiscParam($extNode->getNodeName(), $extNode->getAttribute('enabled'));
		}

		# restore NTP service
		if (my ($ntpNode) = $serverNode->xql('./ntp_service')) {
			my $enabled = $ntpNode->getAttribute('enabled');
			my $server = $ntpNode->getAttribute('server');

			unless ($server) {
				print_stderr("Unable to restore NTP service: NTP server was not specified in the backup archive.\n");
			} else {
				putMiscParam('ntp_server', $server);
				#putMiscParam('ntp_enabled', 'false'); # see the bug # 5011
	
				unless (util_exec('timemng', '--reset-sync-time')) {
					print_stderr("NTP service was turned off: unable to stop current NTP service.\n");
				} else {
					if ($enabled eq 'true') {
						# PSA sets 'ntpdate' in crontab and adds 'ntp_enabled' in database 
						# ONLY if time synchronizing enabled (see bug #5011)
						util_exec('timemng', '--set-sync-time', '--time-server=' . $server);
						putMiscParam('ntp_enabled', $enabled);
					}
				}
			}
		}
	
		my $query = dbDoQuery('SELECT type, netaddr, netmask FROM cp_access');
		foreach my $sn ($serverNode->xql('./cp_access/rule')) {
			my $type = $sn->getAttribute('type');
			my $ip = $sn->getAttribute('ip');
			my $mask = $sn->getAttribute('mask');
			my $query = dbDoQuery(sprintf('SELECT id FROM cp_access WHERE netaddr=%s AND netmask=%s', dbQuote($ip), dbQuote($mask)));
			if (my ($id) = $query->fetchrow_array()) {
				dbDoQuery(sprintf('UPDATE cp_access SET type=%s WHERE id=%s' , dbQuote($type), dbQuote($id)));
			} else {
				dbDoQuery(sprintf('INSERT INTO cp_access(type, netaddr, netmask) VALUES(%s, %s, %s)',
									dbQuote($type), dbQuote($ip), dbQuote($mask)));
			}
		}

		############################################
		#	Restore DNS_Allow_Transfer% parameters
		############################################
		if (my ($node) = $serverNode->xql('./DNS_Allow_Transfer')) {

			#######################################################################
			#	Number of current inserting template DNS_Allow_Transfer{$current}
			#######################################################################
			my $current = 0;

			foreach my $sn ($node->xql('./transfer')) {

				my $ip = $sn->getAttribute('ip');

				if (not $ip) { next; }

				###########################################################
				#	Determine if there is already such record in psa.misc
				#	If so - don't touch anything
				###########################################################

				my $query = dbDoQuery("SELECT COUNT(*) FROM misc WHERE param LIKE 'DNS_Allow_Transfer%' AND val='$ip'");
				my ($used) = $query->fetchrow_array();

				if ($used ne '0') { next; }

				################################################
				#	There is no such record - should insert it
				################################################

				if ($current eq 0) {

					########################################################
					#	Determine number for start inserting templates
					#	Retrieve last used number in DNS_Allow_Transfer%..
					########################################################
					my $sq = dbDoQuery('SELECT param FROM misc WHERE param LIKE "DNS_Allow_Transfer%"');

					while (my ($max) = $sq->fetchrow_array()) {
						$max =~ /DNS_Allow_Transfer([\d]+)/;
						$max = $1;

						$current = ($max > $current)?$max:$current;
					}
				}

				$current++;
				dbDoQuery("INSERT INTO misc VALUES('DNS_Allow_Transfer$current','$ip')");
			}
		}

		########################################################
		#	Restore BU certificate as it not engaged by common
		#	certificate restoration procedure.
		########################################################
		if (my ($node) = $serverNode->xql('./certificates_list/certificate[@bu_cert = "true"]')) {

			################################################
			#	Determine it certificate already installed
			################################################
			my  $query       = dbDoQuery('SELECT val FROM misc WHERE param="bu_cert_id"');
			my ($bu_cert_id) = $query->fetchrow_array;

			############################################
			#	If either BU certificate not exists or
			#	it needs to be replaced...
			#	Othervise - don't change anything.
			############################################
			if ($bu_cert_id eq ''
				or (exists $conf{RESTORE_BU_CERTIFICATE}
				    and    $conf{RESTORE_BU_CERTIFICATE})) {

				#########################################
				#	If certificate exists - it needs to
				#	be replaced
				#########################################
				if ($bu_cert_id ne '') {
					dbDoQuery("DELETE FROM certificates WHERE id=$bu_cert_id");
				}

				#################################################
				#	Insert new certificate values into database
				#################################################
				my $pub_key = '';
				if (my ($sn) = $node->xql('./pub_key')) {
					my $data_sn = $sn->getFirstChild();
					$pub_key = decode_base64($data_sn->toString()) if $data_sn;
				}

				my $pvt_key = '';
				if (my ($sn) = $node->xql('./pvt_key')) {
					my $data_sn = $sn->getFirstChild();
					$pvt_key = decode_base64($data_sn->toString()) if $data_sn;
				}

				################################
				#	Restore certificate values
				################################
				dbDoQuery("INSERT INTO certificates(pvt_key,cert) VALUES('$pvt_key','$pub_key')");

				#####################
				#	Update psa.misc
				#####################
				dbDoQuery("REPLACE misc(param,val) VALUES('bu_cert_id'," . dbGetInsertId() . ")");
			}

			#################################################################
			#	Next stage - check and repair admin/htdocs/bu_pubkey.txt if
			#	it's not exists or certificate was changed
			#################################################################
			if ((not -e $conf{PRODUCT_ROOT_D}.$const{BU_PUBKEY_FILE})	# If file not exists
				or (													# or
					$bu_cert_id ne '' 									# existing certificate
					and exists $conf{RESTORE_BU_CERTIFICATE}			# was replaced
				    and        $conf{RESTORE_BU_CERTIFICATE}
					)
				) {

				######################################################
				#	Extract public key text and write it to the file
				######################################################
				my $query = dbDoQuery(
					"SELECT cert FROM certificates,misc ".
					"WHERE misc.param='bu_cert_id' AND certificates.id=misc.val");
				my $pub_key = $query->fetchrow_array;

				if (($pub_key ne '') and open(PUBKEY, ">".$conf{PRODUCT_ROOT_D}.$const{BU_PUBKEY_FILE})) {

					print PUBKEY "Public key which may be used to verify dumps made from this host\n\n$pub_key\n";
					close(PUBKEY);
				}
			}
		}
	}

	restoreCommonData($serverNode);

	return 1;
}


return 1;

