###################################################################
package BU::Params;
###################################################################

require 5.004;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(getTarget %metod_const);

use File::Basename;

# archived data transfer method
%metod_const = (
NULL	=> 0,
STD		=> 1,
FILE	=> 2,
DIR		=> 3,
TFTP	=> 4,
SSH		=> 5,
HTTP	=> 6,
FTP		=> 7
);

sub url_parser {
	my ($u) = @_;

	if( $u =~ m/^(\w+):(.+)$/ ) {
		my $protocol=$1;
		my @targets;
		my $t2=$2;
		if( $t2 =~ m,^//(.*)$, ) {
			my ($h, $f)=split('/',$1,2);
			if( $f =~ m,~/(.*)$,) {
				@targets = ($h, $1);
			} else {
				@targets = ($h, "/$f");
			}
		} else {
			@targets = split(':',$t2,2);
		}
		return ($protocol,@targets);
	}
	return 'none';
}

sub file_parser {
	my $file=shift;
	my $def_fname = shift;
	
	if( $file eq '-' ) {
		return $file;
	}
	if( -f $file ) {
		return $file;
	}
	if( $file =~ m,/$, ) {
		if( -d $file  && $def_fname) {
			return $file.$def_fname;
		}
		return '';
	}
	if( -d $file ) {
		return $file.'/'.$def_fname if $def_fname;
		return '';
	}
	if( -d  dirname($file) ) {
		return $file;
	}
	return '';
}


sub getTarget {
	my $t = shift;
	my $def_fname = shift;
	my $metod=$metod_const{NULL};
#	my $target;
	my $locate='none';
	
	($protocol,@params)=url_parser($t);

	if( $protocol eq 'none' ) {
		$locate='local';
		@params = ($t)
	} elsif($protocol eq 'file' and $params[0] =~ m,^/,) {
		$locate='local';
	} elsif($protocol eq 'file' and $params[0] eq 'localhost') {
		$locate='local';
		shift @params;
	} elsif($protocol eq 'tftp' and $params[0] ne '') {
		$metod=$metod_const{TFTP};
		$locate='net';
	} elsif($protocol eq 'ssh' and $params[0] ne '') {
		$metod=$metod_const{SSH};
		$locate='net';
	} elsif($protocol eq 'ftp' and $params[0] ne '') {
		$metod=$metod_const{FTP};
		$locate='net';
	}

	if( $locate eq 'net') {
		$params[1].=$def_fname if( $def_fname and 
			($params[1] eq '' or  $params[1] =~ m,/$,));
	} elsif( $locate eq 'local' ) {
		my $f = file_parser($params[0],$def_fname);
		if( $f eq '-' ) {
			$metod=$metod_const{STD} ;
		} elsif( $f ) {
			@params = ($f);
			$metod=$metod_const{FILE} ;
		}
	}
	return ($protocol,$metod,@params);
	
}

1;
__END__

#
# test area
# 
@urls = (
'ssh://fate/pub/zzzz',
'ssh://fate/~/work/',
'ssh:fate:/pub/zzzz',
'ssh:fate:pub/zzzz',
'tftp://fate/ssss',
'tftp://fate',
'tftp:///',
'tftp:fate:/',
'file:/tmp/rrrr',
'file://localhost/tmp/zzzzz',
'/tmp/psa20011017.21.archive.make.plesk.ru',
'zzzzz',
'/tmp/',
'/tmp/cccccc/ffff',
'/tmp/llll/',
'-',
);

foreach $i (@urls) {
 my ($p,$m,@pp)=getTarget($i,$flags{BACKUP_FILENAME});

 print "$p $m:",join(" >", @pp)," URL: $i\n";
}

__END__

=head1 NAME

PSA:: - The PSA module for ... .

=head1 SYNOPSIS

A short usage summary for programs and functions, which may someday be deemed mandatory. 

=head1 DESCRIPTION

Long drawn out discussion of the program. It's a good idea to break this up into subsections using the =head2 directives, like 

=head2 A Sample Subection

=head2 Yet Another Sample Subection

=head1 OPTIONS

Some people make this separate from the description. 

=head1 RETURN VALUE

What the program or function returns if successful. 

=head1 ERRORS

Exceptions, return codes, exit stati, and errno settings. 

=head1 EXAMPLES

Give some example uses of the program. 

=cut ENVIRONMENT

Envariables this program might care about. 

=head1 FILES

All files used by the program. You should probably use the F<> for these. 

=head1 SEE ALSO

Other man pages to check out, like man(1), man(7), makewhatis(8), or catman(8). 

=cut NOTES

Miscellaneous commentary. 

=cut CAVEATS

Things to take special care with; sometimes called WARNINGS. 

=cut DIAGNOSTICS

All possible messages the program can print out--and what they mean. 

=cut BUGS

